# -*- coding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2011 NovaPoint Group LLC (<http://www.novapointgroup.com>)
#    Copyright (C) 2004-2010 OpenERP SA (<http://www.openerp.com>)
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>
#
##############################################################################

from osv import osv,fields
import tools

class account_journal(osv.osv):
    _inherit = "account.journal"
    '''
        Add fields Allow Check writing, Use Preprinted Check and Check Sequence on journal
    '''
    _columns = {
        'allow_check_writing': fields.boolean('Allow Check writing', help='Fill this if the journal is to be used for writing checks.'),
        'use_preprint_check': fields.boolean('Use Preprinted Check'),
        'check_sequence': fields.many2one('ir.sequence', 'Check Sequence', help="This field contains the information related to the numbering of the check number."),
        }

account_journal()

class res_company(osv.osv):
    _inherit = "res.company"
    
    '''
        Add check printing options check_layout, currency_format and check language on company
    '''
    def _get_language(self, cr, uid, context):
        lang_obj = self.pool.get('res.lang')
        lang_ids = lang_obj.search(cr, uid, [('translatable', '=', True)],context=context)
        langs = lang_obj.browse(cr, uid, lang_ids, context=context)
        res = [(lang.code, lang.name) for lang in langs]
        for lang_dict in tools.scan_languages():
            if lang_dict not in res:
                res.append(lang_dict)
        return res

    _columns = {
        'check_layout': fields.selection([
            ('top', 'Check on Top'),
            ('middle', 'Check in middle'),
            ('bottom', 'Check on bottom'),
            ],"Choose Check layout",
            help="Check on top is compatible with Quicken, QuickBooks and Microsoft Money. Check in middle is compatible with Peachtree, ACCPAC and DacEasy. Check on bottom is compatible with Peachtree, ACCPAC and DacEasy only"  ),
        'currency_format': fields.selection([('us','US Format'), ('euro','Europian Format')],'Check Printing Format'),
        'lang': fields.selection(_get_language, string='Check Print Language', size=16),
        }
    _defaults = {
        'check_layout' : lambda *a: 'top',
        'currency_format':'us',
    }
res_company()
class check_log(osv.osv):
    _name = 'check.log'
    _description = 'Check Log'
    '''
        Check Log model
    '''
    _columns = {
        'name':fields.many2one('account.voucher','Reference payment'),
        'status': fields.selection([('active','Active'),
                                    ('voided', 'Voided'),
                                    ('stop_pay', 'Stop Pay Placed'),
                                    ('lost', 'Lost'),
                                    ('unk', 'Unknown'),
                                    ],"Check Status",),
        'check_no':fields.char('Check Number',size=64),
        }
    _defaults = {
        'status' :'blank',
    }
check_log()

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
