# -*- coding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2011 NovaPoint Group LLC (<http://www.novapointgroup.com>)
#    Copyright (C) 2004-2010 OpenERP SA (<http://www.openerp.com>)
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>
#
##############################################################################

from osv import fields, osv
import mx.DateTime
from mx.DateTime import RelativeDateTime
import decimal_precision as dp
from collections import defaultdict
from tools.translate import _

'''
    New function and field to calculate the write-off amount for each voucher line 
'''
class account_voucher_line(osv.osv):
    _name = 'account.voucher.line'
    _inherit = 'account.voucher.line'

    def _calc_writeoff(self, cr, uid, ids, name, args, context):
        '''
            Update Write Off amount
        '''
        res={}
        for line in self.browse(cr, uid, ids):
            writeoff_amount = 0
            for writeoff in line.writeoff_ids:
                writeoff_amount += writeoff.writeoff_amount
            res[line.id] = writeoff_amount
        return res
    _columns = {
        'writeoff_ids':fields.one2many('account.voucher.line.writeoff', 'voucher_line_id', 'Write-Off Lines'  ),
        'writeoff_amount':fields.function(_calc_writeoff, method=True, string='Write-off Amt', type='float', store=False ,),
    }

account_voucher_line()

'''
    Lines to enter write-off amount for each voucher lines
'''
class account_voucher_line_writeoff(osv.osv):
    _name = "account.voucher.line.writeoff"
    _rec_name = 'voucher_line_id'

    def _check_amount(self, cr, uid, ids):
        '''
            -ve write-off amount is not acceptable
        '''
        for rec in self.read(cr, uid, ids, ['writeoff_amount']):
            if rec['writeoff_amount'] < 0:
                return False
        return True

    def _get_gl_account(self, cr, uid, context={}):
        '''
            Find the default GL account for writeoff
        '''
        user = self.pool.get('res.users').browse(cr, uid, uid, context)
        account_id = False
        if user and user.company_id and user.company_id.writeoff_account:
            account_id = user.company_id.writeoff_account.id
        return account_id


    _columns = {
        'voucher_line_id': fields.many2one('account.voucher.line', 'Account Voucher Line', ondelete='cascade', readonly=True),
        'writeoff_amount': fields.float('Write-Off Amt',help='The amount of this invoice to write-off.', readonly=False),
        'gl_account' : fields.many2one('account.account', 'G/L Account',help='Enter the General Ledger account number to record the Bad Debt Expense.', required=True),
        }
    _defaults = {
        'gl_account': _get_gl_account,
        }

    _constraints = [
        (_check_amount, 'Warning! You can not create record with Write-Off Amt less than zero.', ['writeoff_amount'])
    ]
    def onchange_writeoff_amount(self, cr, uid, id, writeoff_amount):
        res = {}
        if writeoff_amount < 0:
            raise osv.except_osv(_('Warning!'), _('You can not create record with Write-Off Amt less than zero.'))

        return res
account_voucher_line_writeoff()

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
