# -*- coding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2004-2010 Tiny SPRL (<http://tiny.be>).
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

import time

from report import report_sxw

class consumption_report(report_sxw.rml_parse):
    def _get_consumption_lines(self,product_id):

        """ Finds the consumption details for the product
         @param self: The object pointer.
         @param product_id: ID of the product
         @comment: fetches the consumption details of the product using 'find_consumption_details'
                   which analyses stock moves to find consumption details
         @return: List of dictionaries showing the consumption details
        """
        res=[]
        detail_list=self.localcontext.get('detail_list',[])
        cr=self.cr;uid=self.uid
        details=self.pool.get("product.product").find_consumption_details(cr,uid,[product_id],context=self.localcontext)
        detail_list=details.get('detail_list',[])
        return detail_list

    def _get_us_date(self,date_string):
        """ Converts the input date string to us format
         @param self: The object pointer.
         @param date_string: Expects date in ISO format YYYY-MM-DD
         @comment: uses format_lang method to convert the date to the us format
         @return: Returns either the us format date or an empty string
        """
        us_string=''
        if date_string:
            us_string=self.formatLang(date_string,date=True)
        return us_string

    def _format_product_name(self,product_name,reference):
        """ Adds the product name and reference
         @param self: The object pointer.
         @param product_name: name of the product
         @param reference: Reference of the product
         @comment: adds the the reference and the product name if there is a reference string
         @return: returns '[reference] product_name' or 'product_name'
        """
        if reference:
            reference="[%s] "%reference
        else:
            reference=""
        return reference+product_name

    def __init__(self, cr, uid, name, context=None):
        """ Parser __init__ override
        """
        super(consumption_report, self).__init__(cr, uid, name, context=context)
        self.localcontext.update({
            'time': time,
            '_get_consumption_lines':self._get_consumption_lines,
            '_get_':self._get_consumption_lines,
            '_get_us_date':self._get_us_date,
            '_format_product_name':self._format_product_name,
            'source_loc_id':context.get('source_loc_id', False),#1. location id selected for getting consumption details in the wizard
            'start_date':context.get('start_date',False),#2. getting start date selected in the wizard
            'end_date':context.get('end_date',False),#3. getting end date selected in the wizard
        })

report_sxw.report_sxw('report.product.consumption', 'product.product', 'addons/assembly_bom/report/consumption_report.rml', parser=consumption_report, header=False)

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:

