# -*- ecoding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2011 NovaPoint Group LLC (<http://www.novapointgroup.com>)
#    Copyright (C) 2004-2010 OpenERP SA (<http://www.openerp.com>)
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>
#
##############################################################################

from osv import fields, osv
import netsvc
import datetime
import time
from tools.translate import _

class StockMove(osv.osv):
    _inherit = 'stock.move'

    def _action_explode(self, cr, uid, move, context=None):
        """ Explodes pickings to produce many moves as per the bom lines for products assembly bom.
        @param move: Stock moves
        @constraints: explode takes place for pickings related to sale order
        @return: True
        """
        result=super(StockMove, self)._action_explode(cr, uid, move, context=context)
        bom_obj = self.pool.get('mrp.bom')
        move_obj = self.pool.get('stock.move')
        procurement_obj = self.pool.get('procurement.order')
        product_obj = self.pool.get('product.product')
        wf_service = netsvc.LocalService("workflow")
        production_obj=self.pool.get('mrp.production')
        if move.product_id.supply_method == 'produce' and move.product_id.procure_method == 'make_to_order':
            bis = bom_obj.search(cr, uid, [
                ('product_id','=',move.product_id.id),
                ('bom_id','=',False),
                ('type','=','assembly')])
            if bis:
                factor = move.product_qty
                bom_point = bom_obj.browse(cr, uid, bis[0], context=context)
                res = bom_obj._bom_explode(cr, uid, bom_point, factor, [])
                dest = move.product_id.product_tmpl_id.property_stock_production.id

                if move.sale_line_id:

                    #1. partial delivery is not allowed for kit type bom product
                    move.picking_id.write({'move_type':'one'})

                    #2. manually changing the state to confirmed for each exploded move to allow checking bom stock value
                        #related to mrp_jit
                    state = 'confirmed'
                    if move.state == 'assigned':
                        state = 'assigned'
                    if res[0] and res[0][0]:
                        line=res[0][0]
                        valdef = {
                                'picking_id': move.picking_id.id,
                                'product_id': line['product_id'],
                                'product_uom': line['product_uom'],
                                'product_qty': line['product_qty'],
                                'product_uos': line['product_uos'],
                                'product_uos_qty': line['product_uos_qty'],
                                'move_dest_id': move.id,
                                'state': state,
                                'name': line['name'],
                                'location_dest_id': dest,
                                'move_history_ids': [(6,0,[move.id])],
                                'move_history_ids2': [(6,0,[])],
                                'procurements': [],
                                'sale_line_id':False,
                                "kit_id":False,
                                "parent_bom_id":bis[0],
                                'sale_id':move.sale_line_id.order_id.id
                            }
                        move.write(valdef)
                        
                    for line in res[0][1:]:
                        valdef = {
                            'picking_id': move.picking_id.id,
                            'product_id': line['product_id'],
                            'product_uom': line['product_uom'],
                            'product_qty': line['product_qty'],
                            'product_uos': line['product_uos'],
                            'product_uos_qty': line['product_uos_qty'],
                            'move_dest_id': move.id,
                            'state': state,
                            'name': line['name'],
                            'location_dest_id': dest,
                            'move_history_ids': [(6,0,[move.id])],
                            'move_history_ids2': [(6,0,[])],
                            'procurements': [],
                            'sale_line_id':False,
                            "kit_id":False,
                            "parent_bom_id":bis[0],
                            'sale_id':move.sale_line_id.order_id.id
                        }
                        mid = move_obj.copy(cr, uid, move.id, default=valdef)
                        move_obj.action_assign(cr, uid, [mid])
                        prodobj = product_obj.browse(cr, uid, line['product_id'], context=context)

                    cr.commit()
        return result


    _columns={
              'sale_id': fields.many2one('sale.order',"Sale Reference"),
              'kit_id':fields.many2one('mrp.production',"Assembly"),
              'production_ids': fields.many2many('mrp.production', 'mrp_production_move_ids', 'move_id', 'production_id', 'Related Production'),
              'parent_bom_id':fields.many2one("mrp.bom",'Related Bom')
              }
    def onchange_quantity(self, cr, uid, ids, product_id, product_qty,
                          product_uom, product_uos):
        """ On change of product quantity finds UoM and UoS quantities
        @param product_id: Product id
        @param product_qty: Changed Quantity of product
        @param product_uom: Unit of measure of product
        @param product_uos: Unit of sale of product
        @return: Dictionary of values
        """
        result=super(StockMove,self).onchange_quantity(cr, uid, ids, product_id, product_qty, product_uom, product_uos)
        print result
        if (not product_id) or (product_qty <=0.0):
            return result
        
        product_obj = self.pool.get('product.product').browse(cr, uid, [product_id])[0]
        uom2 = product_obj.uom_id
        if (product_obj.type=='product') and (product_obj.virtual_available * uom2.factor < product_qty * product_obj.uom_id.factor) \
                  and (product_obj.procure_method=='make_to_stock'):
                    warning = {
                        'title': _('Not enough stock !'),
                        'message': _('You plan to move %.2f %s but you only have %.2f %s available !\nThe real stock is %.2f %s. (without reservations)') %
                            (product_qty, uom2 and uom2.name or product_obj.uom_id.name,
                             max(0,product_obj.virtual_available), product_obj.uom_id.name,
                             max(0,product_obj.qty_available), product_obj.uom_id.name)
                    }     
                    result['warning'] = warning
                    
        if (product_obj.type=='product') and product_obj.procure_method=='make_to_order':
            bom_id = self.pool.get('mrp.bom').search(cr, uid, [
                        ('product_id', '=', product_obj.id),
                        ('bom_id', '=', False),
                        ('type', '=', 'assembly')])[0]
            bom_obj = self.pool.get('mrp.bom').browse(cr, uid, bom_id)
            
            if bom_obj and (bom_obj.bom_stock_value  < product_qty * product_obj.uom_id.factor) \
                  and (product_obj.procure_method=='make_to_order'):
                    warning = {
                        'title': _('Not enough stock !'),
                        'message': _('You plan to move %.2f %s but you only have %.2f %s available !\nThe real stock is %.2f %s. (without reservations)') %
                            (product_qty, uom2 and uom2.name or product_obj.uom_id.name,
                             max(0,bom_obj.bom_stock_value), product_obj.uom_id.name,
                             max(0,product_obj.qty_available), product_obj.uom_id.name)
                    }     
                    result['warning'] = warning                    
                    
        return result
    def onchange_product_id(self, cr, uid, ids, prod_id=False, loc_id=False,
                            loc_dest_id=False, address_id=False,product_qty=0):
        """ On change of product id, .
        @param prod_id: Changed Product id
        @param loc_id: Source location id
        @param loc_id: Destination location id
        @param address_id: Address id of partner
        @param product_qty: Product Quantity
        @return: Dictionary of values
        """
        warning = {}
        res=super(StockMove,self).onchange_product_id(cr, uid, ids, prod_id=prod_id, loc_id=loc_id,
                            loc_dest_id=loc_dest_id, address_id=address_id)
        if not prod_id:
            return res

        product_obj = self.pool.get('product.product').browse(cr, uid, [prod_id])[0]
        uom2 = product_obj.uom_id
            
        
        
        
        if (product_obj.type=='product') and (product_obj.virtual_available * uom2.factor < product_qty * product_obj.uom_id.factor) \
                  and (product_obj.procure_method=='make_to_stock'):
                    warning = {
                        'title': _('Not enough stock !'),
                        'message': _('You plan to move %.2f %s but you only have %.2f %s available !\nThe real stock is %.2f %s. (without reservations)') %
                            (product_qty, uom2 and uom2.name or product_obj.uom_id.name,
                             max(0,product_obj.virtual_available), product_obj.uom_id.name,
                             max(0,product_obj.qty_available), product_obj.uom_id.name)
                    }     
                    res['warning'] = warning
        if (product_obj.type=='product') and product_obj.procure_method=='make_to_order':
            bom_id = self.pool.get('mrp.bom').search(cr, uid, [
                        ('product_id', '=', product_obj.id),
                        ('bom_id', '=', False),
                        ('type', '=', 'assembly')])[0]
            bom_obj = self.pool.get('mrp.bom').browse(cr, uid, bom_id)
            if bom_id and (bom_obj.bom_stock_value  < product_qty * product_obj.uom_id.factor):
                    warning = {
                        'title': _('Not enough stock !'),
                        'message': _('You plan to move %.2f %s but you only have %.2f %s available !\nThe real stock is %.2f %s. (without reservations)') %
                            (product_qty, uom2 and uom2.name or product_obj.uom_id.name,
                             max(0,bom_obj.bom_stock_value), product_obj.uom_id.name,
                             max(0,product_obj.qty_available), product_obj.uom_id.name)
                    }     
                    res['warning'] = warning
        return res
    _defaults = {
        'date': lambda *a: time.strftime('%Y-%m-%d %H:%M:%S'),
        'date_expected': lambda *a: time.strftime('%Y-%m-%d %H:%M:%S'),
    }
    
StockMove()


class stock_picking(osv.osv):
    _inherit="stock.picking"

    def do_partial(self, cr, uid, ids, partial_datas, context=None):
        """Creates and processes the assembly (MO) operation for delivery of product with assembly bom during sale
         @param self: The object pointer.
         @param ids: ids of the procurement order document
         @param context: standard dictionary
         @comment: Creates Production order for assembly of the product with assembly bom and
                    deliveres the product to customers making moves to ensure stock tracking
         @return: returns dictionary carrying delivered picking details
        """
        bom_obj = self.pool.get('mrp.bom')
        picking_obj=self.pool.get('stock.picking')
        produce_wiz_obj=self.pool.get("mrp.product.produce")
        production_obj=self.pool.get('mrp.production')
        move_obj=self.pool.get("stock.move")
        sale_obj=self.pool.get("sale.order")
        wf_service=netsvc.LocalService('workflow')
        res=super(stock_picking,self).do_partial(cr, uid, ids, partial_datas)
        picking_details=res.values()
        picking_id=picking_details and picking_details[0].get('delivered_picking',False) or False
        if picking_id:
            #expecting the picking state will not be 'done'
            picking=picking_obj.browse(cr,uid,picking_id,context=context)
            sale_order=picking.sale_id
            if not sale_order:
                return res

            sale_obj.action_ship_end(cr, uid, [picking.sale_id.id], context=context)
            for line in sale_order.order_line:
                bom_ids=bom_obj.search(cr,uid,[('product_id','=',line.product_id.id)],context={})
#                procure_id=production_obj.create(cr,uid,{ 'name':'ASSEMBLY'+sale_order.name,
                bom_id=bom_ids and bom_ids[0] or False
                if bom_id:
                    bom=bom_obj.browse(cr,uid,bom_id)
                    if bom and bom.type=='normal':
                        move_ids=[move.id for move in picking.move_lines] or []
                        normal_move_ids=move_obj.search(cr,uid,[('id','in',move_ids),('product_id','=',line.product_id.id)])
                        ###Expecting
                        #1. Sale order is having only one line with the product having normal bom
                        if normal_move_ids:
                            normal_move_id=normal_move_ids[0]#Refer:#1
                            normal_move=move_obj.browse(cr,uid,normal_move_id)
                            normal_move.write({
                                        'sale_id':line.order_id.id
                                        })

                    if bom and bom.type=='assembly':
                        #1. Warehouse stock and output location selected as stock and output
                        stock_id = sale_order.shop_id.warehouse_id.lot_stock_id.id#stock location
                        output_id = sale_order.shop_id.warehouse_id.lot_output_id.id#output warehouse location
                        #2. sale partner customer location selected as customer
                        customer_loc_id=sale_order.partner_id.property_stock_customer.id or False#customer location
                        mo_sequence=self.pool.get('ir.sequence').get(cr, uid, 'mrp.production')
                        src_loc_id=line.product_id.property_stock_production.id#stock location_id
                        production_id=production_obj.create(cr,uid,{
                                                                 'name': mo_sequence or sale_order.name,
                                                                'origin':sale_order.name,
                                                                'product_id':line.product_id.id,
                                                                'product_qty':line.product_uom_qty,
                                                                'product_uom':line.product_uom.id,
                                                                'product_uos_qty':(line.product_uos and line.product_uos_qty)\
                                                                                or line.product_uom_qty,
                                                                'product_uos':(line.product_uos and line.product_uos.id)\
                                                                                or line.product_uom.id,
                                                                'location_src_id':src_loc_id,
    #                                                            'location_dest_id':sale_order.shop_id.warehouse_id.lot_stock_id.id,
                                                                'location_dest_id':stock_id,
                                                                'date_start':time.strftime('%Y-%m-%d'),
                                                                'bom_id':bom_ids and bom_ids[0] or False,
                                                                'picking_id':picking_id,
    #                                                            'procure_id':line.procurement_id.id,
                                                                },context=context)

                        # 3.To suit sale order with multiple lines its achieved by comparing the product in production order
                        # and the one sold  and the one associated with the parent_bom stored in the move object

                        for move in picking.move_lines:
                            sold_product_id=line.product_id.id
                            parent_product_id=move.parent_bom_id and move.parent_bom_id.product_id.id or False
                            if parent_product_id and parent_product_id==sold_product_id:
                                move.write({'kit_id':production_id})


                        production=production_obj.browse(cr,uid,production_id)
                        wf_service.trg_validate(uid, 'mrp.production', production_id, 'button_confirm', cr)
                        move_ids=[move.id for move in production.move_lines]
                        move_obj.write(cr,uid,move_ids,{'sale_id':line.order_id.id},context=context)
                        production_obj.force_production(cr, uid, [production_id])
                        move_ids=[move.id for move in production.move_lines]
                        production_obj.action_produce(cr, uid, production_id,production.product_qty,'consume_produce')
                        wf_service.trg_validate(uid, 'mrp.production',production_id,'button_produce_done', cr)
                        date_planned=time.strftime('%Y-%m-%d')
                        cr.commit()
                        production=production_obj.browse(cr,uid,production_id)
                        production_finished=(production.state=='done')
                        if production_finished:
                            finished_ids=[move.id for move in production.move_created_ids2]

                            move_obj.write(cr,uid,finished_ids,{'sale_id':line.order_id.id},context=context)

                            finished_move_stock_id = self.pool.get('stock.move').create(cr, uid, {
							'name': line.name[:64],
                            'picking_id': False,
                            'product_id': line.product_id.id,
                            'date': time.strftime('%Y-%m-%d'),
                            'date_expected': time.strftime('%Y-%m-%d'),
                            'product_qty': line.product_uom_qty,
                            'product_uom': line.product_uom.id,
                            'product_uos_qty': line.product_uos_qty,
                            'product_uos': (line.product_uos and line.product_uos.id)\
                                    or line.product_uom.id,
                            'product_packaging': line.product_packaging.id,
                            'address_id': line.address_allotment_id.id or sale_order.partner_shipping_id.id,
                            'location_id': stock_id,
                            'location_dest_id':output_id,#move destination to output location of the warehouse
                            'sale_line_id': line.id,
                            'tracking_id': False,
                            'state': 'draft',
                            #'state': 'waiting',
                            'note': line.notes,
                            'kit_id':False,
                            'company_id': sale_order.company_id.id,
                            'sale_id': sale_order.id,
                        })

                            move2_id=move_obj.copy(cr,uid,finished_move_stock_id,{'location_id':output_id, 'sale_line_id':False,'location_dest_id': customer_loc_id})#move to customer location

                            status=move_obj.action_confirm(cr,uid,[finished_move_stock_id,move2_id])

                            move_obj.action_done(cr,uid,[finished_move_stock_id,move2_id])


    def action_confirm(self, cr, uid, ids, context=None):        
        res=super(stock_picking,self).action_confirm(cr, uid, ids, context)
        cr.commit()
        try:
            self.action_assign(cr, uid, ids)
        except Exception, e:
            print e
        return True
    _defaults = {
        'date': lambda *a: time.strftime('%Y-%m-%d %H:%M:%S'),
    }


stock_picking()

class location(osv.osv):
    _inherit='stock.location'

    def search(self, cr, uid, args, offset=0, limit=None, order=None,
            context=None, count=False):
        """Returns the stock locations with non zero real stock for the product
         @param self: The object pointer.
         @param ids: ids of the procurement order document
         @param context: standard dictionary
         @return: returns ids of locations if its real stock for the product is non zero
        """
        if context and context.get('filter_by_real_stock',False):
            loc_obj=self.pool.get('stock.location')
            res=super(location, self).search(cr, uid, args=args, offset=offset, limit=limit, order=order,
                context=context, count=count)
            arg=None
            field_names=['stock_real']
            stock_values=loc_obj._product_value(cr, uid, res, field_names, arg, context=context)
            for loc_id,stock in stock_values.items():
                if not stock.get('stock_real',False):
                    del stock_values[loc_id]
            return stock_values.keys()
        else:
            return super(location, self).search(cr, uid, args=args, offset=offset, limit=limit, order=order,
                context=context, count=count)
location()


# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:

