# -*- encoding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2011 NovaPoint Group LLC (<http://www.novapointgroup.com>)
#    Copyright (C) 2004-2010 OpenERP SA (<http://www.openerp.com>)
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>
#
##############################################################################
import wizard
import pooler
from osv import osv,fields
import tools
from tools.translate import _

class consumption_report_wizard(osv.osv_memory):

    def print_consumption_report(self,cr,uid,ids,context={}):
        """ Updates the context with the details given in the wizard form
         @param self: The object pointer.
         @param ids: ids of the procurement order document
         @param context: standard dictionary
         @comment: The data for stock move analysis is passed through context
         @return: returns standard dictionary to make report
        """
        data = self.read(cr, uid, ids[0])
        warehouse_id=data and data.get('warehouse_id',False) or False
        start_date=data and data.get('start_date',False) or False
        end_date=data and data.get('end_date',False) or False
        warehouse=self.pool.get('stock.warehouse').browse(cr,uid,warehouse_id)
        product_ids=context.get('active_ids',[])
        source_loc_id=warehouse.lot_stock_id and warehouse.lot_stock_id.id or False
        return {
        'type': 'ir.actions.report.xml',
        'report_name':"product.consumption",
        'datas': {
                'model':'product.product',
                'id': product_ids and product_ids[0] or False,
                'ids': product_ids and product_ids or [],
                'report_type': 'pdf',
            },
        'nodestroy': False,
        'context':{
                   "source_loc_id":source_loc_id,
                   "start_date":start_date,
                   "end_date":end_date
                   },

        }

    _name="product.consumption"
    _columns={
              'warehouse_id':fields.many2one('stock.warehouse','Select Your Warehouse'),
              'start_date':fields.date("Start Date"),
              'end_date':fields.date("End Date")

              }


    def _get_warehouse(self, cr, uid, context=None):
        """ Fetches the default warehouse
         @param self: The object pointer.
         @param ids: ids of the procurement order document
         @param context: standard dictionary
         @comment: Pre-fills the warehouse field by the ware house related to user company
         @return: returns standard dictionary to make report
        """
        cmpny_id = self.pool.get('res.users')._get_company(cr, uid, context=context)
        shop = self.pool.get('sale.shop').search(cr, uid, [('company_id', '=', cmpny_id)])
        if shop:
            shop_obj = self.pool.get('sale.shop').browse(cr, uid, shop[0], context=context)
            return shop_obj.warehouse_id and shop_obj.warehouse_id.id
        return False

    _defaults = {
        'warehouse_id': _get_warehouse,
    }

consumption_report_wizard()


