# -*- coding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2011 NovaPoint Group LLC (<http://www.novapointgroup.com>)
#    Copyright (C) 2004-2010 OpenERP SA (<http://www.openerp.com>)
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>
#
##############################################################################

from osv import osv, fields
import time
import netsvc
from tools.translate import _
import tools
import base64

LOGGER = netsvc.Logger()

class email_template_mailbox(osv.osv):
    _inherit = "email_template.mailbox"

    '''
    Inherit send_this_mail function to add email on partner history after sending email. 
    '''
    def send_this_mail(self, cr, uid, ids=None, context=None):
        #previous method to send email (link with email account can be found at the revision 4172 and below
        result = True
        attachment_pool = self.pool.get('ir.attachment')
        for id in (ids or []):
            try:
                account_obj = self.pool.get('email_template.account')
                values = self.read(cr, uid, id, [], context)
                attach_to_send = None
    
                if values['attachments_ids']:
                    attach_to_send = self.pool.get('ir.attachment').read(cr, uid, values['attachments_ids'], ['datas_fname','datas', 'name'])
                    attach_to_send = map(lambda x: (x['datas_fname'] or x['name'], base64.decodestring(x['datas'])), attach_to_send)
    
                if values.get('body_html'):
                    body = values.get('body_html')
                    subtype = "html"
                else :
                    body = values.get('body_text')
                    subtype = "plain"
    #            result = tools.email_send(src,dest,sub,body)
                try:
                    result = tools.email_send(
                        values.get('email_from') or u'',
                        [values.get('email_to')],
                        values['subject'] or u'',
                        body or u'',
                        reply_to=values.get('reply_to') or u'',
                        email_bcc=values.get('email_bcc') or u'',
                        email_cc=values.get('email_cc') or u'',
                        subtype=subtype,
                        attach=attach_to_send,
                        openobject_id=values['message_id']
                    )
                except:
                    pass
    
                print result
                if result == True:
                    account = account_obj.browse(cr, uid, values['account_id'][0], context=context)
                    send_model_name = values.get('rel_model',False)
                    send_model_id = values.get('rel_model_ref',False)
                    template_id = values.get('ref_template',False)
                    if send_model_name:
                        send_model_name = self.pool.get('ir.model').browse(cr,uid,send_model_name[0]).model
                    ## Code to add sent mail details in partner history
                    try:
                        if template_id:
                            historize = self.pool.get('email.template').read(cr,uid,template_id[0],['add_to_history'])['add_to_history']
                        if send_model_name and send_model_id and historize:
                            if send_model_name in ['account.voucher','sale.order','crm.lead','calendar.attendee',
                                                   'crm.phonecall','purchase.order','account.invoice','account.voucher',]:
                                send_model_pool = self.pool.get(send_model_name)
                                if send_model_pool:
                                    send_model = send_model_pool.browse(cr,uid,send_model_id,context=context)
                                    if send_model and send_model.partner_id and send_model.partner_id.id:
                                        self.pool.get('mailgate.message').create(cr,uid,{'partner_id':send_model.partner_id.id,
                                                                                         'name':values['subject'] or u'',
                                                                                        'model':send_model_name,
                                                                                        'res_id':send_model_id,
                                                                                        'date':time.strftime('%Y-%m-%d %H:%M:%S'),
                                                                                        'history':True,
                                                                                        'user_id':uid,
                                                                                        'message':body or u'',
                                                                                        'email_from':values.get('email_from') or u'',
                                                                                        'email_to':values.get('email_to'),
                                                                                        'email_cc':values.get('email_cc') or u'',
                                                                                        'email_bcc':values.get('email_bcc') or u'',
                                                                                        'attachment_ids':[[6,0,values['attachments_ids']]],
                                                                                        'description':body,})
                    except (Exception, ), e:
                        logger = netsvc.Logger()
                        msg = "Adding email sent history in partner: "+str(e)
                        logger.notifyChannel("email_template_modification", netsvc.LOG_ERROR, msg)
                        
                    if account.auto_delete:
                        self.write(cr, uid, id, {'folder': 'trash','state':'sent', 'date_mail':time.strftime("%Y-%m-%d %H:%M:%S")}, context=context)
                        self.unlink(cr, uid, [id], context=context)
                        # Remove attachments for this mail
                        attachment_pool.unlink(cr, uid, values['attachments_ids'], context=context)
                    else:
                        self.write(cr, uid, id, {'folder':'sent', 'state':'sent', 'date_mail':time.strftime("%Y-%m-%d %H:%M:%S")}, context)
                        self.historise(cr, uid, [id], "Email sent successfully", context)
                else:
                    self.write(cr,uid,id,{'state':'exception','folder':'outbox'})
                    try:
                        error = result['error_msg'] 
                    except:
                        error = 'Unable to send mail. Please check SMTP is configured properly.'
                    self.historise(cr, uid, [id], error, context)

            except Exception, error:
                logger = netsvc.Logger()
                logger.notifyChannel("email-template", netsvc.LOG_ERROR, _("Sending of Mail %s failed. Probable Reason:Could not login to server\nError: %s") % (id, error))
                self.historise(cr, uid, [id], error, context)
        return result


    _columns = {
        'rel_model':fields.many2one('ir.model','Model',readonly=True),
        'rel_model_ref':fields.integer('Referred Document',readonly=True),
        'ref_template':fields.many2one('email.template','Template',readonly=True),
        'state':fields.selection([
#                        ('na', 'Not Applicable'),
                        ('sending', 'Sending'),
                        ('draft', 'Draft'),
                        ('sending','Sending'),
                        ('exception', 'Exception'),
                        ('sent', 'Sent'),
                        ], 'Status', required=True),
        }
    _defaults = {
                 'state': lambda * a: 'draft',
                 'folder': lambda * a: 'drafts',
    }

email_template_mailbox()

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
