# -*- coding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2011 NovaPoint Group LLC (<http://www.novapointgroup.com>)
#    Copyright (C) 2004-2010 OpenERP SA (<http://www.openerp.com>)
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>
#
##############################################################################

from osv import osv, fields
from mako.template import Template
from mako import exceptions
import netsvc
import base64
from tools.translate import _
import tools
from email_template.email_template import get_value


##    Inheriting email save function to add model and id from which the email sending wizard open
class email_template_send_wizard(osv.osv_memory):
    _inherit = 'email_template.send.wizard'
    
    def save_to_mailbox(self, cr, uid, ids, context=None):
        def get_end_value(id, value):
            if len(context['src_rec_ids']) > 1: # Multiple Mail: Gets value from the template
                return self.get_value(cr, uid, template, value, context, id)
            else:
                return value

        if context is None:
            context = {}
        mail_ids = []
        template = self._get_template(cr, uid, context)
        for id in context['src_rec_ids']:
            screen_vals = self.read(cr, uid, ids[0], [],context)

            account = self.pool.get('email_template.account').read(cr, uid, screen_vals['from'], context=context)
            vals = {
                'email_from': tools.ustr(account['name']) + "<" + tools.ustr(account['email_id']) + ">",
                'email_to': get_end_value(id, screen_vals['to']),
                'email_cc': get_end_value(id, screen_vals['cc']),
                'email_bcc': get_end_value(id, screen_vals['bcc']),
                'subject': get_end_value(id, screen_vals['subject']),
                'body_text': get_end_value(id, screen_vals['body_text']),
                'body_html': get_end_value(id, screen_vals['body_html']),
                'account_id': screen_vals['from'],
                'state':'draft',
                'mail_type':'multipart/alternative' ,
                'rel_model':screen_vals['rel_model'],
                'rel_model_ref':screen_vals['rel_model_ref'],
                'ref_template':screen_vals['ref_template'],
            }
            if screen_vals['signature']:
                signature = self.pool.get('res.users').read(cr, uid, uid, ['signature'], context)['signature']
                if signature:
                    vals['body_text'] = tools.ustr(vals['body_text'] or '') + signature
                    vals['body_html'] = tools.ustr(vals['body_html'] or '') + signature

            attachment_ids = []

            #Create partly the mail and later update attachments
            mail_id = self.pool.get('email_template.mailbox').create(cr, uid, vals, context)
            mail_ids.append(mail_id)
            if template.report_template:
                reportname = 'report.' + self.pool.get('ir.actions.report.xml').read(cr, uid, template.report_template.id, ['report_name'], context)['report_name']
                data = {}
                data['model'] = self.pool.get('ir.model').browse(cr, uid, screen_vals['rel_model'], context).model

                # Ensure report is rendered using template's language
                ctx = context.copy()
                if template.lang:
                    ctx['lang'] = self.get_value(cr, uid, template, template.lang, context, id)
                service = netsvc.LocalService(reportname)
                (result, format) = service.create(cr, uid, [id], data, ctx)
                attachment_id = self.pool.get('ir.attachment').create(cr, uid, {
                    'name': _('%s (Email Attachment)') % tools.ustr(vals['subject']),
                    'datas': base64.b64encode(result),
                    'datas_fname': tools.ustr(get_end_value(id, screen_vals['report']) or _('Report')) + "." + format,
                    'description': vals['body_text'] or _("No Description"),
                    'res_model': 'email_template.mailbox',
                    'res_id': mail_id
                }, context)
                attachment_ids.append( attachment_id )

            # Add document attachments
            for attachment_id in screen_vals.get('attachment_ids',[]):
                new_id = self.pool.get('ir.attachment').copy(cr, uid, attachment_id, {
                    'res_model': 'email_template.mailbox',
                    'res_id': mail_id,
                }, context)
                attachment_ids.append( new_id )

            if attachment_ids:
                self.pool.get('email_template.mailbox').write(cr, uid, mail_id, {
                    'attachments_ids': [[6, 0, attachment_ids]],
                    'mail_type': 'multipart/mixed'
                }, context)

        return mail_ids
email_template_send_wizard()

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
