# -*- coding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2011 NovaPoint Group LLC (<http://www.novapointgroup.com>)
#    Copyright (C) 2004-2010 OpenERP SA (<http://www.openerp.com>)
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>
#
##############################################################################

from osv import fields,osv
from tools.translate import _
import time


class res_partner_address(osv.osv):
    _description ='Partner Addresses'
    _name = 'res.partner.address'
    _inherit='res.partner.address'
    _columns = {
                'last_address_validation': fields.date('Last Address Validation', readonly=True),
                'address_validation_method': fields.selection([('ups.account','UPS'), ], 'Address Validation Method', size=32, required=True),
                }

    '''
    Change the address disply as us format
    '''
    def name_get(self, cr, user, ids, context=None):
        """
            @param self: The object pointer
            @param cr: the current row, from the database cursor,
            @param user: the current user,
            @param ids: List of partner address’s IDs
            @param context: A standard dictionary for contextual values
        """

        if not len(ids):
            return []
        res = []
        if context is None:
            context = {}
        for r in self.browse(cr, user, ids):#, ['zip', 'city', 'partner_id', 'street']):
            if context.get('contact_display', 'contact')=='partner' and r['partner_id']:
                res.append((r.id, r.partner_id.name))
            else:
                addr = str('')
                addr += r.street and (r.street + ', ') or ''
                addr += r.city and (r.city + ', ') or ''
                addr += r.state_id and r.state_id.code and (r.state_id.code + ', ') or ''
                addr += r.zip_id and r.zip_id.zipcode and (r.zip_id.zipcode + ', ') or ''
                addr += r.country_id and r.country_id.name and (r.country_id.name) or ''
                res.append((r['id'], addr.strip() or '/'))
        return res
    _defaults = {
        'address_validation_method':lambda *a:'ups.account',
                }


res_partner_address()

class sale_order(osv.osv):
    _inherit='sale.order'
    '''
    Add address validation fields on sale order
    '''
    
    
    def so_addr_validate_wiz(self, cr, uid, ids, context=None):
        
        
        
        obj_model = self.pool.get('ir.model.data')
        
        model_data_ids = obj_model.search(cr,uid,[('model','=','ir.ui.view'),('name','=','view_so_addrvalidate')])
        resource_id = obj_model.read(cr, uid, model_data_ids, fields=['res_id'])[0]['res_id']
        return {
            'view_type': 'form',
            'view_mode': 'form',
            'res_model': 'so.addr_validate',
            'views': [(resource_id,'form')],
            'type': 'ir.actions.act_window',
            'target': 'new',
            'context': context,
        }

    
    def _validated(self, cr, uid, ids, name, args, context=None):
        res = {}
        for sale_order in self.browse(cr,uid,ids,context=context):
            if sale_order.partner_invoice_id.last_address_validation and sale_order.partner_order_id.last_address_validation and sale_order.partner_shipping_id.last_address_validation:
                res[sale_order.id] = True
            else:
                res[sale_order.id] = False
        return res
    _columns={  'hide_validate':fields.function(_validated, method=True, string='Hide Validate', type='boolean', store=False, ),
                'address_validation_method': fields.selection([('ups.account','UPS'), ], 'Address Validation Method', size=32, required=True),
    }
    _defaults = {
        'address_validation_method':lambda *a:'ups.account',
                }
sale_order()

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
