# -*- coding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2011 NovaPoint Group LLC (<http://www.novapointgroup.com>)
#    Copyright (C) 2004-2010 OpenERP SA (<http://www.openerp.com>)
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>
#
##############################################################################
from osv import fields,osv
import decimal_precision as dp


class shipping_rate_card(osv.osv):
    _name = 'ups.shipping.rate.card'
    _description = "Ground Shipping Calculation Table"
    _columns = {
                'name':fields.char('Shipping Method',size=128,required=True),
                'from_date':fields.datetime('From Date', ),
                'to_date':fields.datetime('To Date', ),
                'rate_ids': fields.one2many('ups.shipping.rate','card_id','Shipping Rates', required=True),
    }
    
shipping_rate_card()



class shipping_rate(osv.osv):
    _name = 'ups.shipping.rate'
    _description = "Shipping Calculation Table"
    _columns = {
                'name':fields.char('Shipping Method',size=128),
                'from_weight': fields.integer('From Weight', required=True),
                'to_weight': fields.integer('To Weight'),
                'charge': fields.float('Shipping Charge'),
                'over_cost': fields.float('Shipping Charge per pound over'),
                'country_id':fields.many2one('res.country','Country'),
                'zone':fields.integer('Zone', required=True),
                'card_id':fields.many2one('ups.shipping.rate.card','Shipping Table')
    }
    
shipping_rate()






class sale_order(osv.osv):
    _name = "sale.order"
    _inherit="sale.order"
    _description = "Sale Order"
    
    def _make_invoice(self, cr, uid, order, lines, context=None):
        inv_id = super(sale_order, self)._make_invoice(cr, uid, order, lines, context=None)
        if inv_id and order._table_name == 'sale.order':
            inv_line_ids = self.pool.get('account.invoice.line').search(cr,uid,[('invoice_id','=',inv_id),('name','=','Shipping Charge')])
            if inv_line_ids:
                self.pool.get('account.invoice.line').write(cr,uid,inv_line_ids,{'price_unit':datas.shipping_cost})
                self.pool.get('account.invoice').write(cr,uid,inv_id,{'shipcharge':datas.shipping_cost, 'ship_method':datas.last_used,})
            else:
                if order.sale_account_id:
                    self.pool.get('account.invoice').write(cr,uid,inv_id,{
                                                            'shipcharge':order.shipcharge,
                                                            'ship_method':order.ship_method,
                                                            })
        return inv_id
    
        
    def _amount_all(self, cr, uid, ids, field_name, arg, context=None):
        cur_obj = self.pool.get('res.currency')
        if context is None:
            context = {}
        res = {}
        for order in self.browse(cr, uid, ids, context=context):
            res[order.id] = {
                'amount_untaxed': 0.0,
                'amount_tax': 0.0,
                'amount_total': 0.0,
            }
            val = val1 = 0.0
            cur = order.pricelist_id.currency_id
            for line in order.order_line:
                val1 += line.price_subtotal
                val += self._amount_line_tax(cr, uid, line, context=context)
            res[order.id]['amount_tax'] = cur_obj.round(cr, uid, cur, val)
            res[order.id]['amount_untaxed'] = cur_obj.round(cr, uid, cur, val1)
            res[order.id]['amount_total'] = res[order.id]['amount_untaxed'] + res[order.id]['amount_tax']
            res[order.id]['amount_total'] += order.shipcharge
        return res
    def _get_order(self, cr, uid, ids, context=None):
        if context is None:
            context = {}
        result = {}
        for line in self.pool.get('sale.order.line').browse(cr, uid, ids, context=context):
            result[line.order_id.id] = True
        return result.keys()
    _columns = {
        'shipcharge':  fields.float('Shipping Cost', readonly=True),
        'ship_method': fields.char('Shipping Method',size=128, readonly=True),
        'amount_total': fields.function(_amount_all, method=True, digits_compute= dp.get_precision('Sale Price'), string='Total',
            store = 
            {
                'sale.order': (lambda self, cr, uid, ids, c={}: ids, ['order_line','ship_method'], -10),
                'sale.order.line': (_get_order, ['price_unit', 'tax_id', 'discount', 'product_uom_qty'], 10),
            },
            multi='sums', help="The total amount"),
        'sale_account_id':fields.many2one('account.account','Account',help='This account represents the g/l account for booking shipping income.')

            }
    
    
sale_order()

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
