# -*- coding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2011 NovaPoint Group LLC (<http://www.novapointgroup.com>)
#    Copyright (C) 2004-2010 OpenERP SA (<http://www.openerp.com>)
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>
#
##############################################################################

from osv import fields,osv
from xml.dom.minidom import Document
import httplib

#----------------------------------------------------------
# Packages Table
#----------------------------------------------------------
class stock_packages(osv.osv):
    _name = "stock.packages"
    _rec_name = "packge_no"
    _columns = {
                'packge_no': fields.integer('Package Number',help='The number of the package associated with the delivery. Example: 3 packages may be associated with a delivery.'),
                'weight': fields.float('Weight (lbs)',help='The weight of the individual package'),
                'package_type': fields.selection([('01', 'UPS Letter'),
                                                ('02', 'Customer Supplied Package'),
                                                ('03', 'Tube'),
                                                ('04', 'PAK'),
                                                ('21', 'UPS ExpressBox'),
                                                ('24', 'UPS 25KG Box'),
                                                ('25', 'UPS 10KG Box'),
                                                ('30', 'Pallet'),
                                                ('2a', 'Small Express Box'),
                                                ('2b', 'Medium Express Box'),
                                                ('2c', 'Large Express Box')],'Package Type',help='Indicates the type of package – taken from Package Type on the Shipment or can be overridden here on individual package.'),
                'length': fields.float('Length',help='Indicates the longest length of the box in inches.'),
                'width': fields.float('Width',help='Indicates the width of the package in inches.'),
                'height': fields.float('Height',help='Indicates the height of the package inches.'),
                'decl_val': fields.integer('Declared Value',help='The declared value of the package.'),
                'ref1': fields.selection([  ('AJ','Accounts Receivable Customer Account'),
                                            ('AT','Appropriation Number'),
                                            ('BM','Bill of Lading Number'),
                                            ('9V','Collect on Delivery (COD) Number'),
                                            ('ON','Dealer Order Number'),
                                            ('DP','Department Number'),
                                            ('3Q','Food and Drug Administration (FDA) Product Code'),
                                            ('IK','Invoice Number'),
                                            ('MK','Manifest Key Number'),
                                            ('MJ','Model Number'),
                                            ('PM','Part Number'),
                                            ('PC','Production Code'),
                                            ('PO','Purchase Order Number'),
                                            ('RQ','Purchase Request Number'),
                                            ('RZ','Return Authorization Number'),
                                            ('SA','Salesperson Number'),
                                            ('SE','Serial Number'),
                                            ('ST','Store Number'),
                                            ('TN','Transaction Reference Number'),
                                            ('EI','Employee ID Number'),
                                            ('TJ','Federal Taxpayer ID No.'),
                                            ('SY','Social Security Number'),],'Reference Number',help='Indicates the type of package'),
                'ref2': fields.char('Reference Number 2', size=64,help='A second reference number associated with the package.'),
                'pick_id': fields.many2one('stock.picking','Delivery Order'),
                'ship_move_id': fields.many2one('shipping.move','Delivery Order'),
                'description': fields.text('Description'),
                }
stock_packages()



#----------------------------------------------------------
# Stock Picking
#----------------------------------------------------------
class stock_picking(osv.osv):


    _inherit = "stock.picking"

    def distribute_weight(self, cr, uid, ids, context=False):
        pack_ids_list = self.read(cr,uid,ids,['packages_ids','tot_order_weight'])
        print pack_ids_list
        for pack_ids in pack_ids_list:
            if pack_ids['tot_order_weight'] and pack_ids['packages_ids']:
                avg_weight = pack_ids['tot_order_weight']/len(pack_ids['packages_ids'])
                self.pool.get('stock.packages').write(cr,uid,pack_ids['packages_ids'],{'weight':avg_weight})
            print pack_ids
        return True
    def _total_weight_net(self, cr, uid, ids, field_name, arg, context):
        """Compute the total net weight of the given Delivery order."""
        result = {}
        for pick in self.browse(cr, uid, ids, context=context):
            result[pick.id] = 0.0
            for line in pick.packages_ids:
                if line.weight:
                    result[pick.id] += line.weight
        return result

    def _get_order(self, cr, uid, ids, context={}):
        """Get the picking ids of the given Stock Moves."""
        result = {}
        for line in self.pool.get('stock.packages').browse(cr, uid, ids,context=context):
            result[line.pick_id.id] = True
        return result.keys()

    _columns = {
                'logis_company': fields.many2one('logistic.company','Logistics Company',help='Name of the Logistics company providing the shipper services.'),
                'freight': fields.boolean('Shipment',help='Indicates if the shipment is a freight shipment.'),
                'sat_delivery': fields.boolean('Saturday Delivery',help='Indicates is it is appropriate to send delivery on Saturday.'),
                'package_type': fields.selection([('01', 'UPS Letter'),
                                                ('02', 'Customer Supplied Package'),
                                                ('03', 'Tube'),
                                                ('04', 'PAK'),
                                                ('21', 'UPS ExpressBox'),
                                                ('24', 'UPS 25KG Box'),
                                                ('25', 'UPS 10KG Box'),
                                                ('30', 'Pallet'),
                                                ('2a', 'Small Express Box'),
                                                ('2b', 'Medium Express Box'),
                                                ('2c', 'Large Express Box')],'Package Type',help='Indicates the type of package'),
                'bill_shipping': fields.selection([('shipper','Shipper'),('receiver','Receiver'),('thirdparty','Third Party')],'Bill Shipping to',help='Shipper, Receiver, or Third Party. If third party is selected a third-party address window appears and the user enters the third party information.'),
                'with_ret_service': fields.boolean('With Return Services',help='Include Return Shipping Information in the package.'),
                'tot_ship_weight': fields.function(_total_weight_net, method=True,
                                    readonly=True, string='Total Shipment Weight',
                                    help="Adds the Total Weight of all the packages in the Packages Table.",
                                    store= {
                                        'sale.order': (lambda self, cr, uid, ids, c={}: ids,
                                             ['packages_ids'], -10),
                                        'stock.packages': (_get_order,
                                             ['weight'], -10),
                                    },
                                    ),

                'packages_ids': fields.one2many("stock.packages",'pick_id','Packages Table'),
                'ship_state': fields.selection([('draft','Draft'),
                                                ('in_process','In Process'),
                                                ('ready_pick','Ready for Pickup'),
                                                ('shipped','Shipped'),
                                                ('delivered','Delivered'),
                                                ('void','Void'),
                                                ('hold','Hold'),
                                                ('cancelled','Cancelled')], 'Shipping Status', readonly=True, help='The current status of the shipment'),
                'trade_mark': fields.text('Trademarks AREA'),
                'ship_message': fields.text('Message'),
                'address_validate': fields.selection([('validate','Validate'), ('nonvalidate','No Validation')], 'Address Validation',help='''No Validatioin = No address validation.
Validate = Fail on failed address validation.
Defaults to validate. Note: Full address
validation is not performed. Therefore, it is
the responsibility of the Shipping Tool User
to ensure the address entered is correct to
avoid an address correction fee.''' ),
                'ship_description': fields.text('Description'),
                'ship_from':fields.boolean('Ship From',help='Required if pickup location is different from the shipper\'s address..'),
                'ship_from_tax_id_no':fields.char('Ship From Tax Identification Number',size=64 ,  select=1),
                'ship_from_address':fields.many2one('res.partner.address','Ship From Address'),
                'tot_order_weight': fields.related('sale_id','total_weight_net', type='float', relation='sale.order', string='Total Order Weight'),
                }
    _defaults = {
        'address_validate': 'nonvalidate',
        'ship_state':'draft'
    }
    def process_ship(self, cr, uid, ids, context=False):
        '''
        Write code to process shipping
        '''
        return True

    def cancel_ship(self, cr, uid, ids, context=False):
        self.write(cr,uid,ids,{'ship_state':'cancelled'})
        return True


    def _get_account_analytic_invoice(self, cursor, user, picking, move_line):
        partner_id = picking.address_id and picking.address_id.partner_id or False
        rec = self.pool.get('account.analytic.default', False) and self.pool.get('account.analytic.default').account_get(cursor, user, move_line.product_id.id, partner_id and partner_id.id, user, time.strftime('%Y-%m-%d'), context={})
        if rec:
            return rec.analytic_id.id
        return super(stock_picking, self)._get_account_analytic_invoice(cursor, user, picking, move_line)
#        access_string=self.create_acc_string()
stock_picking()


# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
