# -*- coding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2011 NovaPoint Group LLC (<http://www.novapointgroup.com>)
#    Copyright (C) 2004-2010 OpenERP SA (<http://www.openerp.com>)
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>
#
##############################################################################


from osv import fields,osv
from xml.dom.minidom import Document
from tools.translate import _
import httplib
import xml2dic
import base64
import time
import datetime
from urlparse import urlparse
import Image
import tempfile
import re

pack_type={'01' : 'UPS Letter',
'02' : 'Customer Supplied Package',
'03' : 'Tube',
'04' : 'PAK',
'21' : 'UPS ExpressBox',
'24' : 'UPS 25KG Box',
'25' : 'UPS 10KG Box',
'30' : 'Pallet',
'2a' : 'Small Express Box',
'2b' : 'Medium Express Box',
'2c' : 'Large Express Box'}


class email_template(osv.osv):

    _inherit = "email.template"

    _columns = {
        'ship_mail' : fields.boolean('Ship Confirm Mail', help='Check this field to send mail when shipping is done.'),
        }


email_template()



class shipping_move(osv.osv):
    _inherit = "shipping.move"
    _columns = {

            'shipment_identific_no': fields.char('ShipmentIdentificationNumber', size=64, ),
            'logo':fields.binary('Logo'),
            'tracking_url':fields.char('Tracking URL', size=512, ),
            'service': fields.many2one('ups.shipping.service.type', 'Shipping Service'),

             }
    def print_label(self, cr, uid, ids, context=None):
        if not ids: return []

#        check_layout = self.browse(cr, uid, ids[0], context=context).company_id.check_layout

        return {
            'type': 'ir.actions.report.xml',
            'report_name':'ship.log.label.print',
            'datas': {
                    'model':'shipping.move',
                    'id': ids and ids[0] or False,
                    'ids': ids and ids or [],
                    'report_type': 'pdf'
                },
            'nodestroy': True
            }
    def getTrackingUrl(self,cr, uid, ids, context=None):
        ship_log_obj = self.browse(cr, uid, ids[0], context=context)
        if ship_log_obj.tracking_no:
            tracking_url = """http://wwwapps.ups.com/WebTracking/processInputRequest?sort_by=status&tracknums_displayed=1&TypeOfInquiryNumber=T&loc=en_US&InquiryNumber1=%s&track.x=0&track.y=0""" % ship_log_obj.tracking_no


shipping_move()



class stock_picking(osv.osv):

    _inherit = "stock.picking"
    _columns={
            'ups_service': fields.many2one('ups.shipping.service.type','Service',help='The specific shipping service offered'),
            'shipper': fields.many2one('ups.account.shipping','Shipper',help='The specific user ID and shipper. Setup in the company configuration.'),
            'shipment_digest': fields.text('ShipmentDigest'),
            'negotiated_rates': fields.float('NegotiatedRates'),
            'shipment_identific_no': fields.char('ShipmentIdentificationNumber', size=64, ),
            'tracking_no': fields.char('TrackingNumber', size=64, ),
            'logo':fields.binary('Logo'),
            'trade_mark': fields.related('shipper','trademark',type='char',size=1024,string='Trademark'),
             }
    
    def action_process(self, cr, uid, ids, context=None):
        deliv_order = self.browse(cr, uid, type(ids)==type([]) and ids[0] or ids, context=context)
        do_transaction = True
        if deliv_order.sale_id and deliv_order.sale_id.payment_method == 'cc_pre_auth' and not deliv_order.sale_id.invoiced:
            
            rel_voucher_id = deliv_order.sale_id.rel_account_voucher_id.id
            if rel_voucher_id and deliv_order.sale_id.rel_account_voucher_id.state != 'posted' and deliv_order.sale_id.rel_account_voucher_id.cc_auth_code:
                do_transaction = False
                self.pool.get('account.voucher').write(cr,uid,[rel_voucher_id],{'cc_p_authorize':False,'cc_charge':True})
                do_transaction = self.pool.get('account.voucher').authorize(cr, uid, [rel_voucher_id], context=context)
        if  not do_transaction:
            self.write(cr,uid,ids,{'ship_state':'hold','ship_message':'Unable to process creditcard payment.'})
            cr.commit()
            raise osv.except_osv(_('Final credit card charge cannot be completed!'),_("Please hold shipment and contact customer service..") )
        return super(stock_picking, self).action_process(cr, uid, ids, context=context)
    
    def action_done(self, cr, uid, ids, context=None):

        res = super(stock_picking, self).action_done(cr, uid, ids, context=context)
        if ids:
            for stock_pick_obj in self.pool.get('stock.picking').browse(cr,uid,ids):
                vals={}
                if stock_pick_obj.sale_id:
                    if stock_pick_obj.sale_id.ship_method:
                        ups_shipping_service_type_ids=self.pool.get('ups.shipping.service.type').search(cr,uid,[('description','like',stock_pick_obj.sale_id.ship_method)])
                        if ups_shipping_service_type_ids:
                          vals['ups_service']=ups_shipping_service_type_ids[0]
                          ups_shipping_service_type_obj=self.pool.get('ups.shipping.service.type').browse(cr,uid,ups_shipping_service_type_ids[0])
                          if ups_shipping_service_type_obj.ups_account_id:
                            vals['shipper']=ups_shipping_service_type_obj.ups_account_id.id
                            if ups_shipping_service_type_obj.ups_account_id.logistic_company_id:
                                vals['logis_company']=ups_shipping_service_type_obj.ups_account_id.logistic_company_id.id
                if vals:
                    self.write(cr,uid,ids,vals)
        return True
    def print_labels(self, cr, uid, ids, context=None):
        if not ids: return []
#        check_layout = self.browse(cr, uid, ids[0], context=context).company_id.check_layout
        return {
            'type': 'ir.actions.report.xml',
            'report_name':'multiple.label.print',
            'datas': {
                    'model':'stock.picking',
                    'id': ids and ids[0] or False,
                    'ids': ids and ids or [],
                    'report_type': 'pdf'
                },
            'nodestroy': True
            }

    def on_change_sale_id(self, cr, uid, ids, sale_id=False, state=False, context=None):
            vals={}
            if sale_id:
                sale_order_obj=self.pool.get('sale.order').browse(cr,uid,sale_id)
                ups_shipping_service_ids=self.pool.get('ups.shipping.service.type').search(cr,uid,[('description','=',sale_order_obj.ship_method)])

                if type(ups_shipping_service_ids)==type([]) and len(ups_shipping_service_ids)>0:
                      vals['ups_service']=ups_shipping_service_ids[0]
                      ups_shipping_account_ids=self.pool.get('ups.account.shipping').search(cr,uid,[('ups_shipping_service_ids','in',ups_shipping_service_ids[0])])

                      if type(ups_shipping_account_ids)==type([]) and len(ups_shipping_account_ids)>0:
                        vals['shipper']=ups_shipping_account_ids[0]
                        logistic_company_ids=self.pool.get('logistic.company').search(cr,uid,[('ups_shipping_account_ids','in',ups_shipping_account_ids[0])])

                        if type(logistic_company_ids)==type([]) and len(logistic_company_ids)>0:
                            vals['logis_company']=logistic_company_ids[0]

            return {'value':vals}


    def process_void(self,cr, uid, ids, context=None):
        error_flag = False
        if type(ids)==type([]):
            pick_obj=self.pool.get('stock.picking').browse(cr, uid, ids[0])
        else:
            pick_obj=self.pool.get('stock.picking').browse(cr, uid, ids)


###create the acces req xml first
        data_for_Access_Request={
                                    'AccessLicenseNumber':pick_obj.shipper.id and pick_obj.shipper.accesslicensenumber or '',
                                    'UserId':pick_obj.shipper.id and pick_obj.shipper.userid or '',
                                    'Password':pick_obj.shipper.id and pick_obj.shipper.password or ''
                                }
        response = ''
        for pack in pick_obj.packages_ids:

            data_for_void_shipment={'VoidShipmentRequest':{'Request':{
                                                        'RequestAction':"1",
                                                      'TransactionReference':{'CustomerContext':"",},
                                                      },
                                            'ShipmentIdentificationNumber':pack.shipment_identific_no or '',

                                                'ExpandedVoidShipment':{
                                                                        'ShipmentIdentificationNumber':pack.shipment_identific_no or '',
                                                                        'TrackingNumber':pack.tracking_no or ''
                                                                        }
                                                            }
                                            }


            doc2=Document()
            VoidShipmentRequest = doc2.createElement("VoidShipmentRequest")
            doc2.appendChild(VoidShipmentRequest)

            Request = doc2.createElement("Request")
            VoidShipmentRequest.appendChild(Request)


            RequestAction = doc2.createElement("RequestAction")
            ptext = doc2.createTextNode(data_for_void_shipment["VoidShipmentRequest"]['Request']['RequestAction'])
            RequestAction.appendChild(ptext)
            Request.appendChild(RequestAction)

            TransactionReference = doc2.createElement("TransactionReference")
            Request.appendChild(TransactionReference)

            CustomerContext = doc2.createElement("CustomerContext")
            ptext = doc2.createTextNode(data_for_void_shipment["VoidShipmentRequest"]['Request']['TransactionReference']['CustomerContext'])
            CustomerContext.appendChild(ptext)
            TransactionReference.appendChild(CustomerContext)

#            ShipmentIdentificationNumber = doc2.createElement("ShipmentIdentificationNumber")
#            ptext = doc2.createTextNode(data_for_void_shipment["VoidShipmentRequest"]['ShipmentIdentificationNumber'])
#            ShipmentIdentificationNumber.appendChild(ptext)
#            VoidShipmentRequest.appendChild(ShipmentIdentificationNumber)
    #


            ExpandedVoidShipment = doc2.createElement("ExpandedVoidShipment")
            VoidShipmentRequest.appendChild(ExpandedVoidShipment)

            ShipmentIdentificationNumber = doc2.createElement("ShipmentIdentificationNumber")
            ptext = doc2.createTextNode(data_for_void_shipment["VoidShipmentRequest"]['ExpandedVoidShipment']['ShipmentIdentificationNumber'])
            ShipmentIdentificationNumber.appendChild(ptext)
            ExpandedVoidShipment.appendChild(ShipmentIdentificationNumber)

            TrackingNumber = doc2.createElement("TrackingNumber")
            ptext = doc2.createTextNode(data_for_void_shipment["VoidShipmentRequest"]['ExpandedVoidShipment']['TrackingNumber'])
            TrackingNumber.appendChild(ptext)
            ExpandedVoidShipment.appendChild(TrackingNumber)

            Request_string2=doc2.toprettyxml()



            doc1 = Document()
            AccessRequest = doc1.createElement("AccessRequest")
            AccessRequest.setAttribute("xml:lang", "en-US")
            doc1.appendChild(AccessRequest)


            AccessLicenseNumber = doc1.createElement("AccessLicenseNumber")
            ptext = doc1.createTextNode(data_for_Access_Request["AccessLicenseNumber"])
            AccessLicenseNumber.appendChild(ptext)
            AccessRequest.appendChild(AccessLicenseNumber)

            UserId = doc1.createElement("UserId")
            ptext = doc1.createTextNode(data_for_Access_Request["UserId"])
            UserId.appendChild(ptext)
            AccessRequest.appendChild(UserId)

            Password = doc1.createElement("Password")
            ptext = doc1.createTextNode(data_for_Access_Request["Password"])
            Password.appendChild(ptext)
            AccessRequest.appendChild(Password)

            Request_string1=doc1.toprettyxml()

            Request_string=Request_string1+Request_string2


#            Request_string = '''<?xml version="1.0" ?>
#<AccessRequest xml:lang='en-US'>
#<AccessLicenseNumber>3C70E997ECE04388</AccessLicenseNumber>
#<UserId>kdunn.cva</UserId>
#<Password>heckethorn</Password>
#</AccessRequest>
#<?xml version="1.0" encoding="UTF-8" ?>
#<VoidShipmentRequest>
#<Request>
#<TransactionReference>
#<CustomerContext>Customer Transaction ID</CustomerContext>
#<XpciVersion>1.0001</XpciVersion>
#</TransactionReference>
#<RequestAction>Void</RequestAction>
#<RequestOption />
#</Request>
#<ExpandedVoidShipment>
#<ShipmentIdentificationNumber>
#1Z2220060294314162
#</ShipmentIdentificationNumber>
#<TrackingNumber>1Z2220060291994175</TrackingNumber>
#</ExpandedVoidShipment>
#</VoidShipmentRequest>'''
            if pick_obj.logis_company.test_mode:
                void_web = pick_obj.logis_company.ship_void_test_web or ''
                void_port = pick_obj.logis_company.ship_void_test_port
            else:
                void_web = pick_obj.logis_company.ship_void_web or ''
                void_port = pick_obj.logis_company.ship_void_port
            if void_web:
                parse_url = urlparse(void_web)
                serv = parse_url.netloc
                serv_path = parse_url.path
            else:
                raise osv.except_osv(_('Unable to find Shipping URL!'),_("Please configure the shipping company with websites.") )


            conn=httplib.HTTPSConnection(serv,void_port)
            res=conn.request("POST",serv_path,Request_string)
            res=conn.getresponse()
            result = res.read()
            response_dic=xml2dic.main(result)
            response = ''
            error_flag=False
            for elm in response_dic['VoidShipmentResponse'][0]['Response']:
                if 'ResponseStatusCode' in elm.keys():
                    response += 'ResponseStatusCode:'+str(elm['ResponseStatusCode'])+'\n'
                if 'ResponseStatusDescription' in elm.keys():
                    response += 'ResponseStatusDescription:'+str(elm['ResponseStatusDescription'])+'\n'
                if 'Error' in elm.keys():
                    error_flag = True
                    response += 'Error:'+str(elm['Error'])+'\n'


            response = 'Void:\n'+response
            self.pool.get('stock.packages').write(cr,uid,pack.id,{'ship_message':str(response)})

        if not error_flag:
            self.pool.get('stock.picking').write(cr, uid, ids,{'ship_state':'void','ship_message':''})
        else:
            self.pool.get('stock.picking').write(cr, uid, ids,{'ship_message':response})
        return True


    def get_package(self,pack_obj):
        if not pack_obj:
            return []
        else:
            ret = []
            ret.append({
                'Description':"",'PackagingType':{'Code':pack_obj.package_type or "",'Description':pack_type.get(pack_obj.package_type , "")},#PackagingType
                        'Dimensions':{'UnitOfMeasurement':{'Code':"IN",'Description':"Inches"},'Length':str(pack_obj.length or "0"),'Width':str(pack_obj.width or "0"),'Height':str(pack_obj.height or "0")},#Dimensions
                        'PackageWeight':{'UnitOfMeasurement':{'Code':"LBS",'Description':"Pounds"},'Weight':str(pack_obj.weight or "0")},
#                            'LargePackageIndicator':"",
                        'ReferenceNumber':{'BarCodeIndicator':"",'Code':pack_obj.ref1 or "",'Value':""},
#                            'AdditionalHandling':"",
#                            'PackageServiceOptions':{'DeliveryConfirmation':{'DCISType':"",'DCISNumber':""},#DeliveryConfirmation
#                                        'InsuredValue':{'Type':"",'Code':"",'Description':"",
#                                                'CurrencyCode':"",'MonetaryValue':""
#                                         },#InsuredValue
#                                        'COD':{'CODCode':"",'CODFundsCode':"",
#                                                'CODAmount':{'CurrencyCode':"",
#                                                        'MonetaryValue':""
#                                                  },#CODAmount
#                                            },#COD
#                                        'VerbalConfirmation':{ContactInfo:{'Name':"",'PhoneNumber':""}#ContactInfo
#                                            },#VerbalConfirmation
#                                        'ShipperReleaseIndicator':"",
#                                        'Notification':{'NotificationCode':"",
#                                            'EMailMessage':{'EMailAddress':"",'UndeliverableEMailAddress':"",
#                                                'FromEMailAddress':"",'FromName':"",'Memo':"",
#                                                'Subject':"",'SubjectCode':""}#EMailMessage
#                                        },#Notification,
#                                        'ReturnsFlexibleAccessIndicator':"",
#                        }#PackageServiceOptions
                })
            return ret

    def fill_addr(self,addr_id):
        ret = {'AddressLine1':addr_id and addr_id.street or '',
         'AddressLine2':addr_id and addr_id.street2 or '',
         'AddressLine3':"",
        'City':addr_id and addr_id.city or '',
        'StateProvinceCode':addr_id and addr_id.state_id.id and addr_id.state_id.code or '',
        'PostalCode':addr_id and addr_id.zip or '',
        'CountryCode':addr_id and addr_id.country_id.id and addr_id.country_id.code or '',
        'ResidentialAddress':"",
        }

        if addr_id and ('zip_id' in addr_id._columns.keys()):
            ret['PostalCode'] = addr_id.zip_id.id and str(addr_id.zip_id.zipcode) or ''
        return ret

    def create_ship_accept_request(self,cr, uid, ids,package_id):
        package_obj=self.pool.get('stock.packages').browse(cr, uid, package_id)

###create the acces req xml first
        data_for_Access_Request={
                                    'AccessLicenseNumber':package_obj.pick_id.shipper.id and package_obj.pick_id.shipper.accesslicensenumber or '',
                                    'UserId':package_obj.pick_id.shipper.id and package_obj.pick_id.shipper.userid or '',
                                    'Password':package_obj.pick_id.shipper.id and package_obj.pick_id.shipper.password or ''
                                }

        doc1 = Document()
        AccessRequest = doc1.createElement("AccessRequest")
        AccessRequest.setAttribute("xml:lang", "en-US")
        doc1.appendChild(AccessRequest)


        AccessLicenseNumber = doc1.createElement("AccessLicenseNumber")
        ptext = doc1.createTextNode(data_for_Access_Request["AccessLicenseNumber"])
        AccessLicenseNumber.appendChild(ptext)
        AccessRequest.appendChild(AccessLicenseNumber)

        UserId = doc1.createElement("UserId")
        ptext = doc1.createTextNode(data_for_Access_Request["UserId"])
        UserId.appendChild(ptext)
        AccessRequest.appendChild(UserId)

        Password = doc1.createElement("Password")
        ptext = doc1.createTextNode(data_for_Access_Request["Password"])
        Password.appendChild(ptext)
        AccessRequest.appendChild(Password)

        Request_string1=doc1.toprettyxml()



        data_for_Ship_Accept = {'ShipmentAcceptRequest':{
                                                        'Request':{
                                                                  'RequestAction':'ShipAccept',
                                                                  'TransactionReference':{'CustomerContext':''}
                                                                  },
                                                        'ShipmentDigest':package_obj.shipment_digest
                                                        }
                                    }


        doc2 = Document()
        ShipmentAcceptRequest = doc2.createElement("ShipmentAcceptRequest")
        ShipmentAcceptRequest.setAttribute("xml:lang", "en-US")
        doc2.appendChild(ShipmentAcceptRequest)



        Request = doc2.createElement("Request")
        ShipmentAcceptRequest.appendChild(Request)


        RequestAction = doc2.createElement("RequestAction")
        ptext = doc1.createTextNode(data_for_Ship_Accept['ShipmentAcceptRequest']["Request"]['RequestAction'])
        RequestAction.appendChild(ptext)
        Request.appendChild(RequestAction)


        TransactionReference = doc2.createElement("TransactionReference")
        Request.appendChild(TransactionReference)

        CustomerContext = doc2.createElement("CustomerContext")
        ptext = doc1.createTextNode(data_for_Ship_Accept['ShipmentAcceptRequest']["Request"]['TransactionReference']['CustomerContext'])
        CustomerContext.appendChild(ptext)
        TransactionReference.appendChild(CustomerContext)

        ShipmentDigest = doc2.createElement("ShipmentDigest")
        ptext = doc1.createTextNode(data_for_Ship_Accept['ShipmentAcceptRequest']["ShipmentDigest"])
        ShipmentDigest.appendChild(ptext)
        ShipmentAcceptRequest.appendChild(ShipmentDigest)

        Request_string2=doc2.toprettyxml()
        request_string=Request_string1+Request_string2

        return request_string

    def process_ship_accept(self,cr, uid, ids,ship_move_ids,pack_obj,context=None):


        shipment_accept_request_xml=self.create_ship_accept_request(cr, uid, ids,pack_obj.id)

        if pack_obj.pick_id.logis_company.test_mode:
            acce_web = pack_obj.pick_id.logis_company.ship_accpt_test_web or ''
            acce_port = pack_obj.pick_id.logis_company.ship_accpt_test_port
        else:
            acce_web = pack_obj.pick_id.logis_company.ship_accpt_web or ''
            acce_port = pack_obj.pick_id.logis_company.ship_accpt_port
        if acce_web:
            parse_url = urlparse(acce_web)
            serv = parse_url.netloc
            serv_path = parse_url.path
        else:
            raise osv.except_osv(_('Unable to find Shipping URL!'),_("Please configure the shipping company with websites.") )



        conn=httplib.HTTPSConnection(serv,acce_port)
        res=conn.request("POST",serv_path,shipment_accept_request_xml)
        res=conn.getresponse()
        result = res.read()
        response_dic=xml2dic.main(result)
        NegotiatedRates=''
        ShipmentIdentificationNumber=''
        TrackingNumber=''
        label_image=''
        if response_dic['ShipmentAcceptResponse'][0]['Response'][0]['ResponseStatusCode'] == '1':
            if len(response_dic['ShipmentAcceptResponse'])>1 and response_dic['ShipmentAcceptResponse'][1].has_key('ShipmentResults'):
                if len(response_dic['ShipmentAcceptResponse'][1]['ShipmentResults'])>1 and response_dic['ShipmentAcceptResponse'][1]['ShipmentResults'][1].has_key('NegotiatedRates'):
                    if  len(response_dic['ShipmentAcceptResponse'][1]['ShipmentResults'][1]['NegotiatedRates'])>0 and response_dic['ShipmentAcceptResponse'][1]['ShipmentResults'][1]['NegotiatedRates'][0].has_key('NetSummaryCharges'):
                        if len(response_dic['ShipmentAcceptResponse'][1]['ShipmentResults'][1]['NegotiatedRates'][0]['NetSummaryCharges'])>0 and response_dic['ShipmentAcceptResponse'][1]['ShipmentResults'][1]['NegotiatedRates'][0]['NetSummaryCharges'][0].has_key('GrandTotal'):
                           NegotiatedRates=response_dic['ShipmentAcceptResponse'][1]['ShipmentResults'][1]['NegotiatedRates'][0]['NetSummaryCharges'][0]['GrandTotal'][1]['MonetaryValue']

                if len(response_dic['ShipmentAcceptResponse'][1]['ShipmentResults'])>0 and response_dic['ShipmentAcceptResponse'][1]['ShipmentResults'][3].has_key('ShipmentIdentificationNumber'):
                    ShipmentIdentificationNumber=response_dic['ShipmentAcceptResponse'][1]['ShipmentResults'][3]['ShipmentIdentificationNumber']

                if len(response_dic['ShipmentAcceptResponse'][1]['ShipmentResults'])>4 and response_dic['ShipmentAcceptResponse'][1]['ShipmentResults'][4].has_key('PackageResults') :
                    if len(response_dic['ShipmentAcceptResponse'][1]['ShipmentResults'][4]['PackageResults'])>0:
                        TrackingNumber=response_dic['ShipmentAcceptResponse'][1]['ShipmentResults'][4]['PackageResults'][0]['TrackingNumber']

                    if len(response_dic['ShipmentAcceptResponse'][1]['ShipmentResults'][4]['PackageResults'])>2 and response_dic['ShipmentAcceptResponse'][1]['ShipmentResults'][4]['PackageResults'][2].has_key('LabelImage') :
                        label_image =response_dic['ShipmentAcceptResponse'][1]['ShipmentResults'][4]['PackageResults'][2]['LabelImage'][1]['GraphicImage']
        if TrackingNumber:
            tracking_url = pack_obj.pick_id.logis_company.ship_tracking_url
            if tracking_url:
                try:
                    tracking_url = tracking_url%TrackingNumber
                except Exception, e:
                    tracking_url = "Invalid tracking url on shipping company"
#            tracking_url = """http://wwwapps.ups.com/WebTracking/processInputRequest?sort_by=status&tracknums_displayed=1&TypeOfInquiryNumber=T&loc=en_US&InquiryNumber1=%s&track.x=0&track.y=0""" % TrackingNumber
        else:
            tracking_url = ''
        if label_image:
            im_in_raw=base64.decodestring(label_image)
            path=tempfile.mktemp()
            temp=file(path,'wb')
            temp.write(im_in_raw)
            temp.close()
            new_im=Image.open(path)
            new_im=new_im.rotate(270)
            new_im.save(path,'JPEG')
            label_from_file=open(path,'rb')
            label_image=base64.encodestring(label_from_file.read())
            label_from_file.close()

        self.pool.get('stock.packages').write(cr,uid,[pack_obj.id],{'tracking_no':TrackingNumber or "",
                                                                     'shipment_identific_no':ShipmentIdentificationNumber or "",
                                                                     'negotiated_rates':NegotiatedRates or '',
                                                                     'logo':label_image,
                                                                     'ship_state':'in_process',
                                                                     'tracking_url':tracking_url})

        if ship_move_ids.get(pack_obj.id):
            if TrackingNumber:
                tracking_url = pack_obj.pick_id.logis_company.ship_tracking_url
                if tracking_url:
                    try:
                        tracking_url = tracking_url%TrackingNumber
                    except Exception, e:
                        tracking_url = "Invalid tracking url on shipping company"
            else:
                tracking_url = ''
            self.pool.get('shipping.move').write(cr, uid, [ship_move_ids.get(pack_obj.id)], {'tracking_no':TrackingNumber or "",
                                                                     'shipment_identific_no':ShipmentIdentificationNumber or "",
                                                                     'ship_cost':NegotiatedRates or '',
                                                                     'logo':label_image,
                                                                     'state':'ready_pick',
                                                                     'tracking_url':tracking_url})
#        t_string="Package id : "+str(pack_obj.id)
#        t_string+="\n"+(pack_obj.pick_id.ship_message or '')
#        self.pool.get('stock.picking').write(cr, uid,[pack_obj.pick_id.id], {'ship_message': t_string})

        return True


    def create_ship_confirm_request(self,cr, uid,package_id):
            package_obj=self.pool.get('stock.packages').browse(cr, uid, package_id)

###create the acces req xml first
            data_for_Access_Request={
                                        'AccessLicenseNumber':package_obj.pick_id.shipper.id and package_obj.pick_id.shipper.accesslicensenumber or '',
                                        'UserId':package_obj.pick_id.shipper.id and package_obj.pick_id.shipper.userid or '',
                                        'Password':package_obj.pick_id.shipper.id and package_obj.pick_id.shipper.password or ''
                                    }

            doc1 = Document()
            AccessRequest = doc1.createElement("AccessRequest")
            AccessRequest.setAttribute("xml:lang", "en-US")
            doc1.appendChild(AccessRequest)


            AccessLicenseNumber = doc1.createElement("AccessLicenseNumber")
            ptext = doc1.createTextNode(data_for_Access_Request["AccessLicenseNumber"])
            AccessLicenseNumber.appendChild(ptext)
            AccessRequest.appendChild(AccessLicenseNumber)

            UserId = doc1.createElement("UserId")
            ptext = doc1.createTextNode(data_for_Access_Request["UserId"])
            UserId.appendChild(ptext)
            AccessRequest.appendChild(UserId)

            Password = doc1.createElement("Password")
            ptext = doc1.createTextNode(data_for_Access_Request["Password"])
            Password.appendChild(ptext)
            AccessRequest.appendChild(Password)

            Request_string1=doc1.toprettyxml()


###create the shipment acces req xml
            data_for_confirm_request={'ShipmentConfirmRequest':{

                                  'Request':{'RequestAction':"ShipConfirm",#-----must be ShipConfirm
                                            'RequestOption':package_obj.pick_id.address_validate or "nonvalidate",#----------adress validation
                                            'TransactionReference':{'CustomerContext':"",},#---------optional data
                                            },#Request

                                  'Shipment':{'Description':package_obj.description or "Shipping",
                                              'ReturnService':{'Code':"8", 'DocumentsOnly':"" },
                                              'Shipper':{ 'Name':package_obj.pick_id.shipper.id and package_obj.pick_id.shipper.name or "",
                                                          'AttentionName':package_obj.pick_id.shipper.id and package_obj.pick_id.shipper.atten_name or "",
                                                          'ShipperNumber':package_obj.pick_id.shipper.id and package_obj.pick_id.shipper.acc_no or "",
                                                          'TaxIdentificationNumber':package_obj.pick_id.shipper.id and package_obj.pick_id.shipper.tax_id_no or "",
                                                          'PhoneNumber':package_obj.pick_id.shipper.id and package_obj.pick_id.shipper.address and package_obj.pick_id.shipper.address.phone or "",
                                                          'FaxNumber':package_obj.pick_id.shipper.id and package_obj.pick_id.shipper.address and package_obj.pick_id.shipper.address.fax or "",
                                                          'EMailAddress':package_obj.pick_id.shipper.id and package_obj.pick_id.shipper.address and package_obj.pick_id.shipper.address.email or "",
                                                          'Address':self.fill_addr(package_obj.pick_id.shipper.id and package_obj.pick_id.shipper.address or '')
                                                         },#Shipper

                                              'ShipTo': {'CompanyName':package_obj.pick_id.address_id.partner_id and package_obj.pick_id.address_id.partner_id.name or '',
                                                        'AttentionName':"",
                                                        'TaxIdentificationNumber':"",
                                                        'PhoneNumber':package_obj.pick_id.address_id.id and package_obj.pick_id.address_id.phone or '',
                                                        'FaxNumber':package_obj.pick_id.address_id.id and package_obj.pick_id.address_id.fax or '',
                                                        'EMailAddress':package_obj.pick_id.address_id.id and package_obj.pick_id.address_id.email or '',
                                                        'Address':self.fill_addr(package_obj.pick_id.address_id.id and package_obj.pick_id.address_id),
                                                        'LocationID':"",
                                                         },#ShipTo



                                              'ShipFrom':{'CompanyName':package_obj.pick_id.shipper.id and package_obj.pick_id.shipper.name or "",
                                                          'AttentionName':package_obj.pick_id.shipper.id and package_obj.pick_id.shipper.atten_name or "",
                                                          'TaxIdentificationNumber':package_obj.pick_id.ship_from_tax_id_no or '',
                                                          'PhoneNumber':package_obj.pick_id.address_id.id and package_obj.pick_id.ship_from_address.phone or '',
                                                          'FaxNumber':package_obj.pick_id.address_id.id and package_obj.pick_id.ship_from_address.fax or '',
                                                          'Address':self.fill_addr(package_obj.pick_id.ship_from_address.id and package_obj.pick_id.ship_from_address),
                                                          },

#                                              'SoldTo':{'Option':"",
#                                                        'CompanyName':"",
#                                                        'AttentionName':"",
#                                                        'TaxIdentificationNumber':"",
#                                                        'PhoneNumber':"",
#                                                        'Address':{'AddressLine1':"",
#                                                                   'AddressLine2':"",
#                                                                   'City':"",
#                                                                   'StateProvinceCode':"",
#                                                                   'PostalCode':"",
#                                                                   'CountryCode':"",}
#                                                      },#SoldTo,

                                              'PaymentInformation':{'Prepaid':{
                                                                            'BillShipper':{'AccountNumber':package_obj.pick_id.shipper and package_obj.pick_id.shipper.acc_no or "",
#                                                                                           'CreditCard':{'Type':"",
#                                                                                                      'Number':"",
#                                                                                                      'ExpirationDate':"",
#                                                                                                      'SecurityCode':"",
#                                                                                                      'Address':{'AddressLine1':"",
#                                                                                                                 'AddressLine2':"",
#                                                                                                                 'AddressLine3':"",
#                                                                                                                 'City':"",
#                                                                                                                 'StateProvinceCode':"",
#                                                                                                                 'PostalCode':"",
#                                                                                                                 'CountryCode':"",},
#                                                                                                      },#CreditCard
                                                                                            },#BillShipper
                                                                                },#Prepaid
#                                                                            'BillThirdParty':{'BillThirdPartyShipper':{'AccountNumber':"",
#                                                                                                                            'ThirdParty':{
#                                                                                                                                          'Address':{'PostalCode':"",
#                                                                                                                                                     'CountryCode':"",},#Address
#                                                                                                                                         },#ThirdParty
#                                                                                                                        },#BillThirdPartyShipper
#
#                                                                                                        },#BillThirdParty
#
#                                                                            'FreightCollect':{'BillReceiver':{'AccountNumber':"",
#                                                                                                              'Address':{'PostalCode':"",},#Address
#                                                                                                 },#BillReceiver
#                                                                                 },#FreightCollect
#
#                                                                           'ConsigneeBilled':"",
                                                                  },#PaymentInformation,

#                                 'ItemizedPaymentInformation':{ 'ShipmentCharge':{'Type':"",
#                                                                                 'BillShipper':{'AccountNumber':"",
#                                                                                                'CreditCard':{'Type':"",
#                                                                                                              'Number':"",
#                                                                                                               'ExpirationDate':"",
#                                                                                                               'SecurityCode':"",
#                                                                                                               'Address':{'AddressLine1':"",
#                                                                                                                          'AddressLine2':"",
#                                                                                                                          'AddressLine3':"",
#                                                                                                                          'City':"",
#                                                                                                                          'StateProvinceCode':"",
#                                                                                                                          'PostalCode':"",
#                                                                                                                          'CountryCode':"",},
#                                                                                                                },#CreditCard
#                                                                                                    },#BillShipper
#                                                                             'BillReceiver':{'AccountNumber':"",'Address':{'PostalCode':"",},#Address
#                                                                                            },#BillReceiver
#                                                                             'BillThirdParty':{'BillThirdPartyShipper':{'AccountNumber':"",
#                                                                                                                        'ThirdParty':{
#                                                                                                                    'Address':{'PostalCode':"",'CountryCode':"",},#Address
#                                                                                                                                    },#ThirdParty
#                                                                                                                   },#BillThirdPartyShipper
#
#                                                                                                'BillThirdPartyConsignee':{'AccountNumber':"",'ThirdParty':{'Address':{'PostalCode':"",
#                                                                                                                                                                    'CountryCode':"",}, #Address
#                                                                                                                                                            },#ThirdParty
#                                                                                                                           },#BillThirdPartyConsignee
#                                                                                                },#BillThirdParty
#                                                                             'ConsigneeBilled':"",
#                                                                                     },#ShipmentCharge
#                                                                'SplitDutyVATIndicator':"",
#                                                            },#ItemizedPaymentInformation

                                 'GoodsNotInFreeCirculationIndicator':"",
                                 'RateInformation':{'NegotiatedRatesIndicator':""},#RateInformation
#                                 'MovementReferenceNumber':"",
#                                 'ReferenceNumber':{'BarCodeIndicator':"",'Code':"",'Value':"",},#ReferenceNumber,
                                 'Service':{'Code':package_obj.pick_id.ups_service.id and str(package_obj.pick_id.ups_service.shipping_service_code) or "",
                                            'Description':package_obj.pick_id.ups_service.id and package_obj.pick_id.ups_service.description or ""},#Service
                                 'InvoiceLineTotal':{'CurrencyCode':"",'MonetaryValue':str(package_obj.pick_id.sale_id.amount_total) or ""},#InvoiceLineTotal
                                 'ShipmentServiceOptions':{'SaturdayDelivery':"",
#                                                           'COD':{'CODCode':"",'CODFundsCode':"",
#                                                                                        'CODAmount':{'CurrencyCode':"",
#                                                                                                     'MonetaryValue':""
#                                                                                                    },#CODAmount
#                                                                                        },#COD
                                                           'Notification':[],
                                                           'LabelDelivery':{'LabelLinkIndicator':"",
                                                                            'EMailMessage':{'EMailAddress':"mail@mail.com",
                                                                                           'UndeliverableEMailAddress':"",
                                                                                           'FromEMailAddress':"mail@mail.com",
                                                                                           'FromName':"",
                                                                                           'Memo':"",
                                                                                           'Subject':"",
                                                                                           'SubjectCode':""}#EMailMessage
                                                                            },#LabelDelivery

                                                           'InternationalForms':{'FormType':"01",
                                                                                 'AdditionalDocumentIndicator':"",
                                                                                 'FormGroupIdName':"",
                                                                                 'SEDFilingOption':"",
                                                                                 'Contacts':{
                                                                                             'ForwardAgent':{'CompanyName':"",
                                                                                                             'TaxIdentificationNumber':"",
                                                                                                             'Address':{'AddressLine1':"",
                                                                                                                        'AddressLine2':"",
                                                                                                                        'City':"",
                                                                                                                        'StateProvinceCode':"",
                                                                                                                        'PostalCode':"",
                                                                                                                        'CountryCode':"",
                                                                                                                        },#Address

                                                                                                                },#ForwardAgent
                                                                                               'UltimateConsignee':{'CompanyName':"",
                                                                                                                    'Address':{'AddressLine1':"",
                                                                                                                               'AddressLine2':"",
                                                                                                                               'City':"",
                                                                                                                                'StateProvinceCode':"",
                                                                                                                                'PostalCode':"",
                                                                                                                                'CountryCode':"",
                                                                                                                                },#Address
                                                                                                                    },#UltimateConsignee

                                                                                               'IntermediateConsignee':{'CompanyName':"",
                                                                                                                        'Address':{'AddressLine1':"",
                                                                                                                                   'AddressLine2':"",
                                                                                                                                   'City':"",
                                                                                                                                   'StateProvinceCode':"",
                                                                                                                                   'PostalCode':"",
                                                                                                                                   'CountryCode':"",
                                                                                                                                   },#Address
                                                                                                                        },#IntermediateConsignee

                                                                                                'Producer':{'Option':"",
                                                                                                            'CompanyName':"",
                                                                                                            'TaxIdentificationNumber':"",
                                                                                                            'Address':{'AddressLine1':"",
                                                                                                                       'AddressLine2':"",
                                                                                                                       'City':"",
                                                                                                                       'StateProvinceCode':"",
                                                                                                                       'PostalCode':"",
                                                                                                                       'CountryCode':"",
                                                                                                                       },#Address
                                                                                                            },#Producer
                                                                                             },#Contacts
                                                                                 'Product':[],##details in another file
                                                                                'InvoiceNumber':"",
                                                                                'InvoiceDate':"",
                                                                                'PurchaseOrderNumber':"",
                                                                                'TermsOfShipment':"",
                                                                                'ReasonForExport':"SALE",
                                                                                'Comments':"",
                                                                                'DeclarationStatement':"",
#                                                                                'Discount':{'MonetaryValue':"0"},#Discount
#                                                                                'FreightCharges':{'MonetaryValue':""},#FreightCharges
#                                                                                'InsuranceCharges':{'MonetaryValue':""},#InsuranceCharges
#                                                                                'OtherCharges':{'MonetaryValue':"",'Description':""},#OtherCharges
                                                                                'CurrencyCode':"",
                                                                                'BlanketPeriod':{'BeginDate':"",'EndDate':""},#BlanketPeriod
                                                                                'ExportDate':"",'ExportingCarrier':"",'CarrierID':"",'InBondCode':"",'EntryNumber':"",
                                                                                'PointOfOrigin':"",'ModeOfTransport':"",'PortOfExport':"",'PortOfUnloading':"",
                                                                                'LoadingPier':"",'PartiesToTransaction':"",'RoutedExportTransactionIndicator':"",
                                                                                'ContainerizedIndicator':"",'License':"",'Number':"",'Date':"",'ExceptionCode':"",
                                                                                'ECCNNumber':"",
                                                                            },#InternationalForms
#                                                           'ReturnOfDocumentIndicator':"",
#                                                           'ImportControlIndicator':"",
#                                                           'LabelMethod':{'Code':"",'Description':"",},#LabelMethod
#                                                           'CommercialInvoiceRemovalIndicator':"",
#                                                           'UPScarbonneutralIndicator':"",
#                                                           'DeliveryConfirmation':{'DCISType':"1",}#DeliveryConfirmation
                                                         },#ShipmentServiceOptions,
                                 'Package':self.get_package(package_obj)## this is done in different file
                                                },#Shipment
                'LabelSpecification':{'LabelPrintMethod':{'Code':"GIF", 'Description':"GIF",},
                                      'HTTPUserAgent':"",
                                      'LabelStockSize':{'Height':"6",'Width':"6"},
                                      'LabelImageFormat':{'Code':"GIF",'Description':"GIF"},
                                    },#LabelSpecification

            },#ShipmentConfirmRequest
        }



            doc2 = Document()



            ShipmentConfirmRequest = doc2.createElement("ShipmentConfirmRequest")
            doc2.appendChild(ShipmentConfirmRequest)

            Request = doc2.createElement("Request")
            ShipmentConfirmRequest.appendChild(Request)

            RequestAction = doc2.createElement("RequestAction")
            ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Request"]["RequestAction"])
            RequestAction.appendChild(ptext)
            Request.appendChild(RequestAction)

            RequestOption = doc2.createElement("RequestOption")
            ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Request"]["RequestOption"])
            RequestOption.appendChild(ptext)
            Request.appendChild(RequestOption)

            TransactionReference = doc2.createElement("TransactionReference")
            Request.appendChild(TransactionReference)

            CustomerContext = doc2.createElement("CustomerContext")
            ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Request"]["TransactionReference"]['CustomerContext'])
            CustomerContext.appendChild(ptext)
            TransactionReference.appendChild(CustomerContext)

            Shipment = doc2.createElement("Shipment")
            ShipmentConfirmRequest.appendChild(Shipment)

            Description = doc2.createElement("Description")
            ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["Description"])
            Description.appendChild(ptext)
            Shipment.appendChild(Description)
            if package_obj.pick_id.with_ret_service:
                ReturnService = doc2.createElement("ReturnService")
                Shipment.appendChild(ReturnService)


                Code = doc2.createElement("Code")
                ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ReturnService"]["Code"])
                Code.appendChild(ptext)
                ReturnService.appendChild(Code)
    #
    #        DocumentsOnly = doc2.createElement("DocumentsOnly")
    #        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ReturnService"]["DocumentsOnly"])
    #        DocumentsOnly.appendChild(ptext)
    #        ReturnService.appendChild(DocumentsOnly)

            Shipper = doc2.createElement("Shipper")
            Shipment.appendChild(Shipper)

            Name = doc2.createElement("Name")
            ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["Shipper"]["Name"])
            Name.appendChild(ptext)
            Shipper.appendChild(Name)

            AttentionName = doc2.createElement("AttentionName")
            ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["Shipper"]["AttentionName"])
            AttentionName.appendChild(ptext)
            Shipper.appendChild(AttentionName)

            ShipperNumber = doc2.createElement("ShipperNumber")
            ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["Shipper"]["ShipperNumber"])
            ShipperNumber.appendChild(ptext)
            Shipper.appendChild(ShipperNumber)

            TaxIdentificationNumber = doc2.createElement("TaxIdentificationNumber")
            ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["Shipper"]["TaxIdentificationNumber"])
            TaxIdentificationNumber.appendChild(ptext)
            Shipper.appendChild(TaxIdentificationNumber)
            PhoneNumber = doc2.createElement("PhoneNumber")
            ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["Shipper"]["PhoneNumber"])
            PhoneNumber.appendChild(ptext)
            Shipper.appendChild(PhoneNumber)

            FaxNumber = doc2.createElement("FaxNumber")
            ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["Shipper"]["FaxNumber"])
            FaxNumber.appendChild(ptext)
            Shipper.appendChild(FaxNumber)

            EMailAddress = doc2.createElement("EMailAddress")
            ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["Shipper"]["EMailAddress"])
            EMailAddress.appendChild(ptext)
            Shipper.appendChild(EMailAddress)

            #################address
            Address = doc2.createElement("Address")
            Shipper.appendChild(Address)

            AddressLine1 = doc2.createElement("AddressLine1")
            ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["Shipper"]["Address"]["AddressLine1"])
            AddressLine1.appendChild(ptext)
            Address.appendChild(AddressLine1)

            AddressLine2 = doc2.createElement("AddressLine2")
            ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["Shipper"]["Address"]["AddressLine2"])
            AddressLine2.appendChild(ptext)
            Address.appendChild(AddressLine2)

            AddressLine3 = doc2.createElement("AddressLine3")
            ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["Shipper"]["Address"]["AddressLine3"])
            AddressLine3.appendChild(ptext)
            Address.appendChild(AddressLine3)

            City = doc2.createElement("City")
            ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["Shipper"]["Address"]["City"])
            City.appendChild(ptext)
            Address.appendChild(City)

            StateProvinceCode = doc2.createElement("StateProvinceCode")
            ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["Shipper"]["Address"]["StateProvinceCode"])
            StateProvinceCode.appendChild(ptext)
            Address.appendChild(StateProvinceCode)

            PostalCode = doc2.createElement("PostalCode")
            ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["Shipper"]["Address"]["PostalCode"])
            PostalCode.appendChild(ptext)
            Address.appendChild(PostalCode)
            CountryCode = doc2.createElement("CountryCode")
            ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["Shipper"]["Address"]["CountryCode"])
            CountryCode.appendChild(ptext)
            Address.appendChild(CountryCode)

            ShipTo = doc2.createElement("ShipTo")
            Shipment.appendChild(ShipTo)
            CompanyName = doc2.createElement("CompanyName")
            ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipTo"]["CompanyName"])
            CompanyName.appendChild(ptext)
            ShipTo.appendChild(CompanyName)

            AttentionName = doc2.createElement("AttentionName")
            ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipTo"]["AttentionName"])
            AttentionName.appendChild(ptext)
            ShipTo.appendChild(AttentionName)

            TaxIdentificationNumber = doc2.createElement("TaxIdentificationNumber")
            ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipTo"]["TaxIdentificationNumber"])
            TaxIdentificationNumber.appendChild(ptext)
            ShipTo.appendChild(TaxIdentificationNumber)

            PhoneNumber = doc2.createElement("PhoneNumber")
            ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipTo"]["PhoneNumber"])
            PhoneNumber.appendChild(ptext)
            ShipTo.appendChild(PhoneNumber)

            FaxNumber = doc2.createElement("FaxNumber")
            ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipTo"]["FaxNumber"])
            FaxNumber.appendChild(ptext)
            ShipTo.appendChild(FaxNumber)

            EMailAddress = doc2.createElement("EMailAddress")
            ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipTo"]["EMailAddress"])
            EMailAddress.appendChild(ptext)
            ShipTo.appendChild(EMailAddress)

            LocationID = doc2.createElement("LocationID")
            ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipTo"]["LocationID"])
            LocationID.appendChild(ptext)
            ShipTo.appendChild(LocationID)

            Address = doc2.createElement("Address")
            ShipTo.appendChild(Address)
            AddressLine1 = doc2.createElement("AddressLine1")
            ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipTo"]["Address"]["AddressLine1"])
            AddressLine1.appendChild(ptext)
            Address.appendChild(AddressLine1)

            AddressLine2 = doc2.createElement("AddressLine2")
            ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipTo"]["Address"]["AddressLine2"])
            AddressLine2.appendChild(ptext)
            Address.appendChild(AddressLine2)

            AddressLine2 = doc2.createElement("AddressLine3")
            ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipTo"]["Address"]["AddressLine3"])
            AddressLine3.appendChild(ptext)
            Address.appendChild(AddressLine3)

            City = doc2.createElement("City")
            ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipTo"]["Address"]["City"])
            City.appendChild(ptext)
            Address.appendChild(City)

            StateProvinceCode = doc2.createElement("StateProvinceCode")
            ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipTo"]["Address"]["StateProvinceCode"])
            StateProvinceCode.appendChild(ptext)
            Address.appendChild(StateProvinceCode)

            PostalCode = doc2.createElement("PostalCode")
            ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipTo"]["Address"]["PostalCode"])
            PostalCode.appendChild(ptext)
            Address.appendChild(PostalCode)

            CountryCode = doc2.createElement("CountryCode")
            ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipTo"]["Address"]["CountryCode"])
            CountryCode.appendChild(ptext)
            Address.appendChild(CountryCode)
            if package_obj.pick_id.ship_from:
                ShipFrom = doc2.createElement("ShipFrom")
                Shipment.appendChild(ShipFrom)
                CompanyName = doc2.createElement("CompanyName")
                ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipFrom"]["CompanyName"])
                CompanyName.appendChild(ptext)
                ShipFrom.appendChild(CompanyName)

                AttentionName = doc2.createElement("AttentionName")
                ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipFrom"]["AttentionName"])
                AttentionName.appendChild(ptext)
                ShipFrom.appendChild(AttentionName)

                TaxIdentificationNumber = doc2.createElement("TaxIdentificationNumber")
                ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipFrom"]["TaxIdentificationNumber"])
                TaxIdentificationNumber.appendChild(ptext)
                ShipFrom.appendChild(TaxIdentificationNumber)

                PhoneNumber = doc2.createElement("PhoneNumber")
                ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipFrom"]["PhoneNumber"])
                PhoneNumber.appendChild(ptext)
                ShipFrom.appendChild(PhoneNumber)

                FaxNumber = doc2.createElement("FaxNumber")
                ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipFrom"]["FaxNumber"])
                FaxNumber.appendChild(ptext)
                ShipFrom.appendChild(FaxNumber)

                Address = doc2.createElement("Address")
                ShipFrom.appendChild(Address)
                AddressLine1 = doc2.createElement("AddressLine1")
                ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipFrom"]["Address"]["AddressLine1"])
                AddressLine1.appendChild(ptext)
                Address.appendChild(AddressLine1)

                AddressLine2 = doc2.createElement("AddressLine2")
                ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipFrom"]["Address"]["AddressLine2"])
                AddressLine2.appendChild(ptext)
                Address.appendChild(AddressLine2)

                AddressLine3 = doc2.createElement("AddressLine3")
                ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipFrom"]["Address"]["AddressLine3"])
                AddressLine3.appendChild(ptext)
                Address.appendChild(AddressLine3)

                City = doc2.createElement("City")
                ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipFrom"]["Address"]["City"])
                City.appendChild(ptext)
                Address.appendChild(City)

                StateProvinceCode = doc2.createElement("StateProvinceCode")
                ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipFrom"]["Address"]["StateProvinceCode"])
                StateProvinceCode.appendChild(ptext)
                Address.appendChild(StateProvinceCode)

#                PostalCode = doc2.createElement("PostalCode")
#                ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipFrom"]["Address"]["PostalCode"])
#                PostalCode.appendChild(ptext)
#                Address.appendChild(PostalCode)

                CountryCode = doc2.createElement("CountryCode")
                ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipFrom"]["Address"]["CountryCode"])
                CountryCode.appendChild(ptext)
                Address.appendChild(CountryCode)



    #            SoldTo = doc2.createElement("SoldTo")
    #            Shipment.appendChild(SoldTo)
    #            CompanyName = doc2.createElement("CompanyName")
    #            ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["SoldTo"]["CompanyName"])
    #            CompanyName.appendChild(ptext)
    #            SoldTo.appendChild(CompanyName)
    #
    #            AttentionName = doc2.createElement("AttentionName")
    #            ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["SoldTo"]["AttentionName"])
    #            AttentionName.appendChild(ptext)
    #            SoldTo.appendChild(AttentionName)
    #
    #            TaxIdentificationNumber = doc2.createElement("TaxIdentificationNumber")
    #            ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["SoldTo"]["TaxIdentificationNumber"])
    #            TaxIdentificationNumber.appendChild(ptext)
    #            SoldTo.appendChild(TaxIdentificationNumber)
    #
    #            PhoneNumber = doc2.createElement("PhoneNumber")
    #            ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["SoldTo"]["PhoneNumber"])
    #            PhoneNumber.appendChild(ptext)
    #            SoldTo.appendChild(PhoneNumber)
    #
    #            Option = doc2.createElement("Option")
    #            ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["SoldTo"]["Option"])
    #            Option.appendChild(ptext)
    #            SoldTo.appendChild(Option)
    #
    #            Address = doc2.createElement("Address")
    #            SoldTo.appendChild(Address)
    #            AddressLine1 = doc2.createElement("AddressLine1")
    #            ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["SoldTo"]["Address"]["AddressLine1"])
    #            AddressLine1.appendChild(ptext)
    #            Address.appendChild(AddressLine1)
    #
    #            AddressLine2 = doc2.createElement("AddressLine2")
    #            ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["SoldTo"]["Address"]["AddressLine2"])
    #            AddressLine2.appendChild(ptext)
    #            Address.appendChild(AddressLine2)
    #
    #            AddressLine3 = doc2.createElement("AddressLine3")
    #            ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["SoldTo"]["Address"]["AddressLine3"])
    #            AddressLine3.appendChild(ptext)
    #            Address.appendChild(AddressLine3)
    #
    #            City = doc2.createElement("City")
    #            ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["SoldTo"]["Address"]["City"])
    #            City.appendChild(ptext)
    #            Address.appendChild(City)
    #
    #            StateProvinceCode = doc2.createElement("StateProvinceCode")
    #            ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["SoldTo"]["Address"]["StateProvinceCode"])
    #            StateProvinceCode.appendChild(ptext)
    #            Address.appendChild(StateProvinceCode)
    #
    #            PostalCode = doc2.createElement("PostalCode")
    #            ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["SoldTo"]["Address"]["PostalCode"])
    #            PostalCode.appendChild(ptext)
    #            Address.appendChild(PostalCode)
    #
    #            CountryCode = doc2.createElement("CountryCode")
    #            ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["SoldTo"]["Address"]["CountryCode"])
    #            CountryCode.appendChild(ptext)
    #            Address.appendChild(CountryCode)


            PaymentInformation = doc2.createElement("PaymentInformation")
            Shipment.appendChild(PaymentInformation)
            Prepaid = doc2.createElement("Prepaid")
            PaymentInformation.appendChild(Prepaid)

            BillShipper = doc2.createElement("BillShipper")
            Prepaid.appendChild(BillShipper)

            AccountNumber = doc2.createElement("AccountNumber")
            ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["PaymentInformation"]["Prepaid"]["BillShipper"]["AccountNumber"])
            AccountNumber.appendChild(ptext)
            BillShipper.appendChild(AccountNumber)

    #        CreditCard = doc2.createElement("CreditCard")
    #        BillShipper.appendChild(CreditCard)
    #        Type = doc2.createElement("Type")
    #        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["PaymentInformation"]["Prepaid"]["BillShipper"]["CreditCard"]["Type"])
    #        Type.appendChild(ptext)
    #        CreditCard.appendChild(Type)
    #
    #        ExpirationDate = doc2.createElement("ExpirationDate")
    #        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["PaymentInformation"]["Prepaid"]["BillShipper"]["CreditCard"]["ExpirationDate"])
    #        ExpirationDate.appendChild(ptext)
    #        CreditCard.appendChild(ExpirationDate)
    #
    #        Number = doc2.createElement("Number")
    #        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["PaymentInformation"]["Prepaid"]["BillShipper"]["CreditCard"]["Number"])
    #        Number.appendChild(ptext)
    #        CreditCard.appendChild(Number)
    #
    #        SecurityCode = doc2.createElement("SecurityCode")
    #        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["PaymentInformation"]["Prepaid"]["BillShipper"]["CreditCard"]["SecurityCode"])
    #        SecurityCode.appendChild(ptext)
    #        CreditCard.appendChild(SecurityCode)
    #
    #
    #        Address = doc2.createElement("Address")
    #        CreditCard.appendChild(Address)
    #        AddressLine1 = doc2.createElement("AddressLine1")
    #        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["PaymentInformation"]["Prepaid"]["BillShipper"]["CreditCard"]["Address"]["AddressLine1"])
    #        AddressLine1.appendChild(ptext)
    #        Address.appendChild(AddressLine1)
    #
    #        AddressLine2 = doc2.createElement("AddressLine2")
    #        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["PaymentInformation"]["Prepaid"]["BillShipper"]["CreditCard"]["Address"]["AddressLine2"])
    #        AddressLine2.appendChild(ptext)
    #        Address.appendChild(AddressLine2)
    #
    #        AddressLine3 = doc2.createElement("AddressLine3")
    #        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["PaymentInformation"]["Prepaid"]["BillShipper"]["CreditCard"]["Address"]["AddressLine3"])
    #        AddressLine3.appendChild(ptext)
    #        Address.appendChild(AddressLine3)
    #
    #
    #        City = doc2.createElement("City")
    #        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["PaymentInformation"]["Prepaid"]["BillShipper"]["CreditCard"]["Address"]["City"])
    #        City.appendChild(ptext)
    #        Address.appendChild(City)
    #
    #        StateProvinceCode = doc2.createElement("StateProvinceCode")
    #        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["PaymentInformation"]["Prepaid"]["BillShipper"]["CreditCard"]["Address"]["StateProvinceCode"])
    #        StateProvinceCode.appendChild(ptext)
    #        Address.appendChild(StateProvinceCode)
    #
    #        PostalCode = doc2.createElement("PostalCode")
    #        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["PaymentInformation"]["Prepaid"]["BillShipper"]["CreditCard"]["Address"]["PostalCode"])
    #        PostalCode.appendChild(ptext)
    #        Address.appendChild(PostalCode)
    #
    #        CountryCode = doc2.createElement("CountryCode")
    #        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["PaymentInformation"]["Prepaid"]["BillShipper"]["CreditCard"]["Address"]["CountryCode"])
    #        PostalCode.appendChild(ptext)
    #        Address.appendChild(CountryCode)




    #        BillThirdParty = doc2.createElement("BillThirdParty")
    #        PaymentInformation.appendChild(BillThirdParty)
    #        BillThirdPartyShipper = doc2.createElement("BillThirdPartyShipper")
    #        BillThirdParty.appendChild(BillThirdPartyShipper)
    #
    #        AccountNumber = doc2.createElement("AccountNumber")
    #        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["PaymentInformation"]["BillThirdParty"]['BillThirdPartyShipper']['AccountNumber'])
    #        AccountNumber.appendChild(ptext)
    #        BillThirdPartyShipper.appendChild(AccountNumber)
    #
    #        ThirdParty = doc2.createElement("ThirdParty")
    #        BillThirdPartyShipper.appendChild(ThirdParty)
    #        Address = doc2.createElement("Address")
    #        ThirdParty.appendChild(Address)
    #
    #        PostalCode = doc2.createElement("PostalCode")
    #        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["PaymentInformation"]["BillThirdParty"]['BillThirdPartyShipper']['ThirdParty']['Address']['PostalCode'])
    #        PostalCode.appendChild(ptext)
    #        Address.appendChild(PostalCode)
    #
    #        CountryCode = doc2.createElement("CountryCode")
    #        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["PaymentInformation"]["BillThirdParty"]['BillThirdPartyShipper']['ThirdParty']['Address']['CountryCode'])
    #        CountryCode.appendChild(ptext)
    #        Address.appendChild(CountryCode)



    #        FreightCollect = doc2.createElement("FreightCollect")
    #        PaymentInformation.appendChild(FreightCollect)
    #        BillReceiver = doc2.createElement("BillReceiver")
    #        FreightCollect.appendChild(BillReceiver)
    #        AccountNumber = doc2.createElement("AccountNumber")
    #        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["PaymentInformation"]["FreightCollect"]["BillReceiver"]["AccountNumber"])
    #        AccountNumber.appendChild(ptext)
    #        BillReceiver.appendChild(AccountNumber)
    #
    #        Address = doc2.createElement("Address")
    #        BillReceiver.appendChild(Address)
    #        PostalCode = doc2.createElement("PostalCode")
    #        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["PaymentInformation"]["FreightCollect"]["BillReceiver"]["Address"]["PostalCode"])
    #        AccountNumber.appendChild(ptext)
    #        Address.appendChild(PostalCode)

    #        ConsigneeBilled = doc2.createElement("ConsigneeBilled")
    #        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["PaymentInformation"]["ConsigneeBilled"])
    #        ConsigneeBilled.appendChild(ptext)
    #        PaymentInformation.appendChild(ConsigneeBilled)




    #        ItemizedPaymentInformation = doc2.createElement("ItemizedPaymentInformation")
    #        Shipment.appendChild(ItemizedPaymentInformation)
    #        ShipmentCharge = doc2.createElement("ShipmentCharge")
    #        ItemizedPaymentInformation.appendChild(ShipmentCharge)
    #        Type = doc2.createElement("Type")
    #        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ItemizedPaymentInformation"]["ShipmentCharge"]["Type"])
    #        Type.appendChild(ptext)
    #        ShipmentCharge.appendChild(Type)
    #
    #        BillShipper = doc2.createElement("BillShipper")
    #        ShipmentCharge.appendChild(BillShipper)
    #
    #        AccountNumber = doc2.createElement("AccountNumber")
    #        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ItemizedPaymentInformation"]["ShipmentCharge"]["BillShipper"]["AccountNumber"])
    #        AccountNumber.appendChild(ptext)
    #        BillShipper.appendChild(AccountNumber)
    #
    #        CreditCard = doc2.createElement("CreditCard")
    #        BillShipper.appendChild(CreditCard)
    #
    #        Type = doc2.createElement("Type")
    #        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ItemizedPaymentInformation"]["ShipmentCharge"]["BillShipper"]["CreditCard"]["Type"])
    #        Type.appendChild(ptext)
    #        CreditCard.appendChild(Type)
    #
    #        ExpirationDate = doc2.createElement("ExpirationDate")
    #        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ItemizedPaymentInformation"]["ShipmentCharge"]["BillShipper"]["CreditCard"]["ExpirationDate"])
    #        ExpirationDate.appendChild(ptext)
    #        CreditCard.appendChild(ExpirationDate)
    #
    #        SecurityCode = doc2.createElement("SecurityCode")
    #        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ItemizedPaymentInformation"]["ShipmentCharge"]["BillShipper"]["CreditCard"]["SecurityCode"])
    #        SecurityCode.appendChild(ptext)
    #        CreditCard.appendChild(SecurityCode)
    #
    #        Address = doc2.createElement("Address")
    #        CreditCard.appendChild(Address)
    #        AddressLine1 = doc2.createElement("AddressLine1")
    #        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ItemizedPaymentInformation"]["ShipmentCharge"]["BillShipper"]['CreditCard']["Address"]["AddressLine1"])
    #        AddressLine1.appendChild(ptext)
    #        Address.appendChild(AddressLine1)
    #
    #        AddressLine2 = doc2.createElement("AddressLine2")
    #        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ItemizedPaymentInformation"]["ShipmentCharge"]["BillShipper"]['CreditCard']["Address"]["AddressLine2"])
    #        AddressLine2.appendChild(ptext)
    #        Address.appendChild(AddressLine2)
    #
    #        AddressLine3 = doc2.createElement("AddressLine3")
    #        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ItemizedPaymentInformation"]["ShipmentCharge"]["BillShipper"]['CreditCard']["Address"]["AddressLine3"])
    #        AddressLine3.appendChild(ptext)
    #        Address.appendChild(AddressLine3)
    #
    #        City = doc2.createElement("City")
    #        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ItemizedPaymentInformation"]["ShipmentCharge"]["BillShipper"]['CreditCard']["Address"]["City"])
    #        City.appendChild(ptext)
    #        Address.appendChild(City)
    #
    #        StateProvinceCode = doc2.createElement("StateProvinceCode")
    #        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ItemizedPaymentInformation"]["ShipmentCharge"]["BillShipper"]['CreditCard']["Address"]["StateProvinceCode"])
    #        StateProvinceCode.appendChild(ptext)
    #        Address.appendChild(StateProvinceCode)
    #
    #        PostalCode = doc2.createElement("PostalCode")
    #        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ItemizedPaymentInformation"]["ShipmentCharge"]["BillShipper"]['CreditCard']["Address"]["PostalCode"])
    #        PostalCode.appendChild(ptext)
    #        Address.appendChild(PostalCode)
    #
    #        CountryCode = doc2.createElement("CountryCode")
    #        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ItemizedPaymentInformation"]["ShipmentCharge"]["BillShipper"]['CreditCard']["Address"]["CountryCode"])
    #        CountryCode.appendChild(ptext)
    #        Address.appendChild(CountryCode)
    #
    #        BillReceiver = doc2.createElement("BillReceiver")
    #        ShipmentCharge.appendChild(BillReceiver)
    #
    #        AccountNumber = doc2.createElement("AccountNumber")
    #        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ItemizedPaymentInformation"]["ShipmentCharge"]["BillReceiver"]["AccountNumber"])
    #        AccountNumber.appendChild(ptext)
    #        BillReceiver.appendChild(AccountNumber)
    #
    #        Address = doc2.createElement("Address")
    #        BillReceiver.appendChild(Address)
    #        PostalCode = doc2.createElement("PostalCode")
    #        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ItemizedPaymentInformation"]["ShipmentCharge"]["BillReceiver"]["Address"]["PostalCode"])
    #        PostalCode.appendChild(ptext)
    #        Address.appendChild(PostalCode)
    #
    #
    #        BillThirdParty = doc2.createElement("BillThirdParty")
    #        ShipmentCharge.appendChild(BillThirdParty)
    #        BillThirdPartyShipper = doc2.createElement("BillThirdPartyShipper")
    #        BillThirdParty.appendChild(BillThirdPartyShipper)
    #
    #        AccountNumber = doc2.createElement("AccountNumber")
    #        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ItemizedPaymentInformation"]["ShipmentCharge"]["BillThirdParty"]["BillThirdPartyShipper"]["AccountNumber"])
    #        AccountNumber.appendChild(ptext)
    #        BillThirdPartyShipper.appendChild(AccountNumber)
    #
    #        ThirdParty = doc2.createElement("ThirdParty")
    #        BillThirdPartyShipper.appendChild(ThirdParty)
    #        Address = doc2.createElement("Address")
    #        ThirdParty.appendChild(Address)
    #        PostalCode = doc2.createElement("PostalCode")
    #        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ItemizedPaymentInformation"]["ShipmentCharge"]["BillThirdParty"]["BillThirdPartyShipper"]["ThirdParty"]["Address"]["PostalCode"])
    #        PostalCode.appendChild(ptext)
    #        Address.appendChild(PostalCode)
    #
    #        CountryCode = doc2.createElement("CountryCode")
    #        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ItemizedPaymentInformation"]["ShipmentCharge"]["BillThirdParty"]["BillThirdPartyShipper"]["ThirdParty"]["Address"]["CountryCode"])
    #        CountryCode.appendChild(ptext)
    #        Address.appendChild(CountryCode)
    #
    #
    #        BillThirdPartyConsignee = doc2.createElement("BillThirdPartyConsignee")
    #        BillThirdParty.appendChild(BillThirdPartyConsignee)
    #
    #        AccountNumber = doc2.createElement("AccountNumber")
    #        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ItemizedPaymentInformation"]["ShipmentCharge"]["BillThirdParty"]["BillThirdPartyConsignee"]["AccountNumber"])
    #        AccountNumber.appendChild(ptext)
    #        BillThirdPartyConsignee.appendChild(AccountNumber)
    #
    #        ThirdParty = doc2.createElement("ThirdParty")
    #        BillThirdPartyConsignee.appendChild(ThirdParty)
    #        Address = doc2.createElement("Address")
    #        ThirdParty.appendChild(Address)
    #        PostalCode = doc2.createElement("PostalCode")
    #        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ItemizedPaymentInformation"]["ShipmentCharge"]["BillThirdParty"]["BillThirdPartyConsignee"]["ThirdParty"]["Address"]["PostalCode"])
    #        PostalCode.appendChild(ptext)
    #        Address.appendChild(PostalCode)
    #
    #        CountryCode = doc2.createElement("CountryCode")
    #        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ItemizedPaymentInformation"]["ShipmentCharge"]["BillThirdParty"]["BillThirdPartyConsignee"]["ThirdParty"]["Address"]["CountryCode"])
    #        CountryCode.appendChild(ptext)
    #        Address.appendChild(CountryCode)
    #
    #        ConsigneeBilled = doc2.createElement("ConsigneeBilled")
    #        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ItemizedPaymentInformation"]["ShipmentCharge"]["ConsigneeBilled"])
    #        ConsigneeBilled.appendChild(ptext)
    #        ShipmentCharge.appendChild(ConsigneeBilled)
    #
    #        SplitDutyVATIndicator = doc2.createElement("SplitDutyVATIndicator")
    #        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ItemizedPaymentInformation"]["SplitDutyVATIndicator"])
    #        SplitDutyVATIndicator.appendChild(ptext)
    #        ItemizedPaymentInformation.appendChild(SplitDutyVATIndicator)

            GoodsNotInFreeCirculationIndicator = doc2.createElement("GoodsNotInFreeCirculationIndicator")
    #        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["GoodsNotInFreeCirculationIndicator"])
    #        GoodsNotInFreeCirculationIndicator.appendChild(ptext)
            Shipment.appendChild(GoodsNotInFreeCirculationIndicator)

            RateInformation = doc2.createElement("RateInformation")
            Shipment.appendChild(RateInformation)
            NegotiatedRatesIndicator = doc2.createElement("NegotiatedRatesIndicator")
    #        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["RateInformation"]["NegotiatedRatesIndicator"])
    #        NegotiatedRatesIndicator.appendChild(ptext)
            RateInformation.appendChild(NegotiatedRatesIndicator)

    #        MovementReferenceNumber = doc2.createElement("MovementReferenceNumber")
    #        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["MovementReferenceNumber"])
    #        MovementReferenceNumber.appendChild(ptext)
    #        Shipment.appendChild(MovementReferenceNumber)

    #        ReferenceNumber = doc2.createElement("ReferenceNumber")
    #        Shipment.appendChild(ReferenceNumber)
    #        BarCodeIndicator = doc2.createElement("BarCodeIndicator")
    #        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ReferenceNumber"]["BarCodeIndicator"])
    #        BarCodeIndicator.appendChild(ptext)
    #        ReferenceNumber.appendChild(BarCodeIndicator)
    #
    #        Code = doc2.createElement("Code")
    #        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ReferenceNumber"]["Code"])
    #        Code.appendChild(ptext)
    #        ReferenceNumber.appendChild(Code)
    #
    #        Value = doc2.createElement("Value")
    #        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ReferenceNumber"]["Value"])
    #        Value.appendChild(ptext)
    #        ReferenceNumber.appendChild(Value)

            Service = doc2.createElement("Service")
            Shipment.appendChild(Service)
            Code = doc2.createElement("Code")
            ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["Service"]["Code"])
            Code.appendChild(ptext)
            Service.appendChild(Code)
            Description = doc2.createElement("Description")
            ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["Service"]["Description"])
            Description.appendChild(ptext)
            Service.appendChild(Description)

            if (package_obj.pick_id.ship_from and package_obj.pick_id.ship_from_address.id and package_obj.pick_id.ship_from_address.country_id.code == 'US' or package_obj.pick_id.shipper.id and package_obj.pick_id.shipper.address.id and package_obj.pick_id.shipper.address.country_id.id and package_obj.pick_id.shipper.address.country_id.code == 'US') and  package_obj.pick_id.address_id.id and package_obj.pick_id.address_id.country_id.code in ('PR','CA'):
                InvoiceLineTotal = doc2.createElement("InvoiceLineTotal")
                Shipment.appendChild(InvoiceLineTotal)

    #            CurrencyCode = doc2.createElement("CurrencyCode")
    #            ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["InvoiceLineTotal"]["CurrencyCode"])
    #            CurrencyCode.appendChild(ptext)
    #            InvoiceLineTotal.appendChild(CurrencyCode)

                MonetaryValue = doc2.createElement("MonetaryValue")
                ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["InvoiceLineTotal"]["MonetaryValue"])
                MonetaryValue.appendChild(ptext)
                InvoiceLineTotal.appendChild(MonetaryValue)


            ShipmentServiceOptions = doc2.createElement("ShipmentServiceOptions")
            Shipment.appendChild(ShipmentServiceOptions)
            if package_obj.pick_id.sat_delivery:
                SaturdayDelivery = doc2.createElement("SaturdayDelivery")
        #        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["SaturdayDelivery"])
        #        SaturdayDelivery.appendChild(ptext)
                ShipmentServiceOptions.appendChild(SaturdayDelivery)
    #        COD = doc2.createElement("COD")
    #        ShipmentServiceOptions.appendChild(COD)
    #        CODCode = doc2.createElement("CODCode")
    #        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["COD"]["CODCode"])
    #        CODCode.appendChild(ptext)
    #        COD.appendChild(CODCode)
    #
    #        CODFundsCode = doc2.createElement("CODFundsCode")
    #        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["COD"]["CODFundsCode"])
    #        CODFundsCode.appendChild(ptext)
    #        COD.appendChild(CODFundsCode)
    #
    #        CODAmount = doc2.createElement("CODAmount")
    #        COD.appendChild(CODAmount)
    #        CurrencyCode = doc2.createElement("CurrencyCode")
    #        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["COD"]["CODAmount"]["CurrencyCode"])
    #        CurrencyCode.appendChild(ptext)
    #        CODAmount.appendChild(CurrencyCode)
    #
    #        MonetaryValue = doc2.createElement("MonetaryValue")
    #        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["COD"]["CODAmount"]["MonetaryValue"])
    #        MonetaryValue.appendChild(ptext)
    #        CODAmount.appendChild(MonetaryValue)

            Notification_count = len(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["Notification"])
            for i in range (0,Notification_count):
                Notification = doc2.createElement("Notification")
                ShipmentServiceOptions.appendChild(Notification)
                NotificationCode = doc2.createElement("NotificationCode")
                ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["Notification"][i]["NotificationCode"])
                NotificationCode.appendChild(ptext)
                Notification.appendChild(NotificationCode)
                EMailMessage = doc2.createElement("EMailMessage")
                Notification.appendChild(EMailMessage)
                UndeliverableEMailAddress = doc2.createElement("UndeliverableEMailAddress")
                ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["Notification"][i]["EMailMessage"]["UndeliverableEMailAddress"])
                UndeliverableEMailAddress.appendChild(ptext)
                EMailMessage.appendChild(UndeliverableEMailAddress)

                EMailAddress = doc2.createElement("EMailAddress")
                ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["Notification"][i]["EMailMessage"]["EMailAddress"])
                EMailAddress.appendChild(ptext)
                EMailMessage.appendChild(EMailAddress)

                FromEMailAddress = doc2.createElement("FromEMailAddress")
                ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["Notification"][i]["EMailMessage"]["FromEMailAddress"])
                FromEMailAddress.appendChild(ptext)
                EMailMessage.appendChild(FromEMailAddress)

                FromName = doc2.createElement("FromName")
                ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["Notification"][i]["EMailMessage"]["FromName"])
                FromName.appendChild(ptext)
                EMailMessage.appendChild(FromName)

                Memo = doc2.createElement("Memo")
                ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["Notification"][i]["EMailMessage"]["Memo"])
                Memo.appendChild(ptext)
                EMailMessage.appendChild(Memo)

                Subject = doc2.createElement("Subject")
                ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["Notification"][i]["EMailMessage"]["Subject"])
                Subject.appendChild(ptext)
                EMailMessage.appendChild(Subject)

                SubjectCode = doc2.createElement("SubjectCode")
                ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["Notification"][i]["EMailMessage"]["SubjectCode"])
                SubjectCode.appendChild(ptext)
                EMailMessage.appendChild(SubjectCode)
            ##//////////////////////end for loop in code
            if package_obj.pick_id.with_ret_service:
                LabelDelivery = doc2.createElement("LabelDelivery")
                ShipmentServiceOptions.appendChild(LabelDelivery)
                LabelLinkIndicator = doc2.createElement("LabelLinkIndicator")
        #        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["LabelDelivery"]["LabelLinkIndicator"])
        #        LabelLinkIndicator.appendChild(ptext)
                LabelDelivery.appendChild(LabelLinkIndicator)
    #        EMailMessage = doc2.createElement("EMailMessage")
    #        LabelDelivery.appendChild(EMailMessage)
    #        UndeliverableEMailAddress = doc2.createElement("UndeliverableEMailAddress")
    #        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["LabelDelivery"]["EMailMessage"]["UndeliverableEMailAddress"])
    #        UndeliverableEMailAddress.appendChild(ptext)
    #        EMailMessage.appendChild(UndeliverableEMailAddress)
    #
    #
    #        EMailAddress = doc2.createElement("EMailAddress")
    #        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["LabelDelivery"]["EMailMessage"]["EMailAddress"])
    #        EMailAddress.appendChild(ptext)
    #        EMailMessage.appendChild(EMailAddress)
    #
    #        FromEMailAddress = doc2.createElement("FromEMailAddress")
    #        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["LabelDelivery"]["EMailMessage"]["FromEMailAddress"])
    #        FromEMailAddress.appendChild(ptext)
    #        EMailMessage.appendChild(FromEMailAddress)
    #
    #        FromName = doc2.createElement("FromName")
    #        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["LabelDelivery"]["EMailMessage"]["FromName"])
    #        FromName.appendChild(ptext)
    #        EMailMessage.appendChild(FromName)
    #
    #        Memo = doc2.createElement("Memo")
    #        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["LabelDelivery"]["EMailMessage"]["Memo"])
    #        Memo.appendChild(ptext)
    #        EMailMessage.appendChild(Memo)
    #
    #        Subject = doc2.createElement("Subject")
    #        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["LabelDelivery"]["EMailMessage"]["Subject"])
    #        Subject.appendChild(ptext)
    #        EMailMessage.appendChild(Subject)
    #
    #        SubjectCode = doc2.createElement("SubjectCode")
    #        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["LabelDelivery"]["EMailMessage"]["SubjectCode"])
    #        SubjectCode.appendChild(ptext)
    #        EMailMessage.appendChild(SubjectCode)

    #        InternationalForms = doc2.createElement("InternationalForms")
    #        ShipmentServiceOptions.appendChild(InternationalForms)
    #        FormType = doc2.createElement("FormType")
    #        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["InternationalForms"]["FormType"])
    #        FormType.appendChild(ptext)
    #        InternationalForms.appendChild(FormType)
    #
    #        AdditionalDocumentIndicator = doc2.createElement("AdditionalDocumentIndicator")
    #        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["InternationalForms"]["AdditionalDocumentIndicator"])
    #        AdditionalDocumentIndicator.appendChild(ptext)
    #        InternationalForms.appendChild(AdditionalDocumentIndicator)
    #
    #        FormGroupIdName = doc2.createElement("FormGroupIdName")
    #        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["InternationalForms"]["FormGroupIdName"])
    #        FormGroupIdName.appendChild(ptext)
    #        InternationalForms.appendChild(FormGroupIdName)
    #
    #        Contacts = doc2.createElement("Contacts")
    #        InternationalForms.appendChild(Contacts)
    #        ForwardAgent = doc2.createElement("ForwardAgent")
    #        Contacts.appendChild(ForwardAgent)
    #
    #
    #        CompanyName = doc2.createElement("CompanyName")
    #        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["InternationalForms"]["Contacts"]['ForwardAgent']['CompanyName'])
    #        CompanyName.appendChild(ptext)
    #        ForwardAgent.appendChild(CompanyName)
    #
    #        TaxIdentificationNumber = doc2.createElement("TaxIdentificationNumber")
    #        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["InternationalForms"]["Contacts"]['ForwardAgent']['TaxIdentificationNumber'])
    #        TaxIdentificationNumber.appendChild(ptext)
    #        ForwardAgent.appendChild(TaxIdentificationNumber)
    #
    #        Address = doc2.createElement("Address")
    #        ForwardAgent.appendChild(Address)
    #
    #        AddressLine1 = doc2.createElement("AddressLine1")
    #        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["InternationalForms"]["Contacts"]['ForwardAgent']['Address']['AddressLine1'])
    #        AddressLine1.appendChild(ptext)
    #        Address.appendChild(AddressLine1)
    #
    #        AddressLine2 = doc2.createElement("AddressLine2")
    #        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["InternationalForms"]["Contacts"]['ForwardAgent']['Address']['AddressLine2'])
    #        AddressLine2.appendChild(ptext)
    #        Address.appendChild(AddressLine2)
    #
    #        #AddressLine3 = doc2.createElement("AddressLine3")
    #        #ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["InternationalForms"]["Contacts"]['ForwardAgent']['Address']['AddressLine3'])
    #        #AddressLine3.appendChild(ptext)
    #        #Address.appendChild(AddressLine3)
    #
    #        City = doc2.createElement("City")
    #        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["InternationalForms"]["Contacts"]['ForwardAgent']['Address']['City'])
    #        City.appendChild(ptext)
    #        Address.appendChild(City)
    #
    #        StateProvinceCode = doc2.createElement("StateProvinceCode")
    #        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["InternationalForms"]["Contacts"]['ForwardAgent']['Address']['StateProvinceCode'])
    #        StateProvinceCode.appendChild(ptext)
    #        Address.appendChild(StateProvinceCode)
    #
    #        PostalCode = doc2.createElement("PostalCode")
    #        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["InternationalForms"]["Contacts"]['ForwardAgent']['Address']['PostalCode'])
    #        PostalCode.appendChild(ptext)
    #        Address.appendChild(PostalCode)
    #
    #        CountryCode = doc2.createElement("CountryCode")
    #        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["InternationalForms"]["Contacts"]['ForwardAgent']['Address']['CountryCode'])
    #        CountryCode.appendChild(ptext)
    #        Address.appendChild(CountryCode)
    #
    #
    #
    #        UltimateConsignee = doc2.createElement("UltimateConsignee")
    #        Contacts.appendChild(UltimateConsignee)
    #
    #
    #        CompanyName = doc2.createElement("CompanyName")
    #        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["InternationalForms"]["Contacts"]['UltimateConsignee']['CompanyName'])
    #        CompanyName.appendChild(ptext)
    #        UltimateConsignee.appendChild(CompanyName)
    #
    #
    #
    #        Address = doc2.createElement("Address")
    #        UltimateConsignee.appendChild(Address)
    #
    #        AddressLine1 = doc2.createElement("AddressLine1")
    #        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["InternationalForms"]["Contacts"]['UltimateConsignee']['Address']['AddressLine1'])
    #        AddressLine1.appendChild(ptext)
    #        Address.appendChild(AddressLine1)
    #
    #        AddressLine2 = doc2.createElement("AddressLine2")
    #        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["InternationalForms"]["Contacts"]['UltimateConsignee']['Address']['AddressLine2'])
    #        AddressLine2.appendChild(ptext)
    #        Address.appendChild(AddressLine2)
    #
    #        #AddressLine3 = doc2.createElement("AddressLine3")
    #        #ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["InternationalForms"]["Contacts"]['UltimateConsignee']['Address']['AddressLine3'])
    #        #AddressLine3.appendChild(ptext)
    #        #Address.appendChild(AddressLine3)
    #
    #        City = doc2.createElement("City")
    #        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["InternationalForms"]["Contacts"]['UltimateConsignee']['Address']['City'])
    #        City.appendChild(ptext)
    #        Address.appendChild(City)
    #
    #        StateProvinceCode = doc2.createElement("StateProvinceCode")
    #        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["InternationalForms"]["Contacts"]['UltimateConsignee']['Address']['StateProvinceCode'])
    #        StateProvinceCode.appendChild(ptext)
    #        Address.appendChild(StateProvinceCode)
    #
    #        PostalCode = doc2.createElement("PostalCode")
    #        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["InternationalForms"]["Contacts"]['UltimateConsignee']['Address']['PostalCode'])
    #        PostalCode.appendChild(ptext)
    #        Address.appendChild(PostalCode)
    #
    #        CountryCode = doc2.createElement("CountryCode")
    #        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["InternationalForms"]["Contacts"]['UltimateConsignee']['Address']['CountryCode'])
    #        CountryCode.appendChild(ptext)
    #        Address.appendChild(CountryCode)
    #
    #
    #
    #
    #        IntermediateConsignee = doc2.createElement("IntermediateConsignee")
    #        Contacts.appendChild(IntermediateConsignee)
    #
    #
    #        CompanyName = doc2.createElement("CompanyName")
    #        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["InternationalForms"]["Contacts"]['IntermediateConsignee']['CompanyName'])
    #        CompanyName.appendChild(ptext)
    #        IntermediateConsignee.appendChild(CompanyName)
    #
    #
    #
    #        Address = doc2.createElement("Address")
    #        IntermediateConsignee.appendChild(Address)
    #
    #        AddressLine1 = doc2.createElement("AddressLine1")
    #        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["InternationalForms"]["Contacts"]['IntermediateConsignee']['Address']['AddressLine1'])
    #        AddressLine1.appendChild(ptext)
    #        Address.appendChild(AddressLine1)
    #
    #        AddressLine2 = doc2.createElement("AddressLine2")
    #        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["InternationalForms"]["Contacts"]['IntermediateConsignee']['Address']['AddressLine2'])
    #        AddressLine2.appendChild(ptext)
    #        Address.appendChild(AddressLine2)
    #
    #        #AddressLine3 = doc2.createElement("AddressLine3")
    #        #ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["InternationalForms"]["Contacts"]['IntermediateConsignee']['Address']['AddressLine3'])
    #        #AddressLine3.appendChild(ptext)
    #        #Address.appendChild(AddressLine3)
    #
    #        City = doc2.createElement("City")
    #        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["InternationalForms"]["Contacts"]['IntermediateConsignee']['Address']['City'])
    #        City.appendChild(ptext)
    #        Address.appendChild(City)
    #
    #        StateProvinceCode = doc2.createElement("StateProvinceCode")
    #        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["InternationalForms"]["Contacts"]['IntermediateConsignee']['Address']['StateProvinceCode'])
    #        StateProvinceCode.appendChild(ptext)
    #        Address.appendChild(StateProvinceCode)
    #
    #        PostalCode = doc2.createElement("PostalCode")
    #        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["InternationalForms"]["Contacts"]['IntermediateConsignee']['Address']['PostalCode'])
    #        PostalCode.appendChild(ptext)
    #        Address.appendChild(PostalCode)
    #
    #        CountryCode = doc2.createElement("CountryCode")
    #        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["InternationalForms"]["Contacts"]['IntermediateConsignee']['Address']['CountryCode'])
    #        CountryCode.appendChild(ptext)
    #        Address.appendChild(CountryCode)
    #
    #
    #
    #
    #        Producer = doc2.createElement("Producer")
    #        Contacts.appendChild(Producer)
    #        Option = doc2.createElement("Option")
    #        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["InternationalForms"]["Contacts"]['Producer']['Option'])
    #        Option.appendChild(ptext)
    #        Producer.appendChild(Option)
    #
    #
    #        CompanyName = doc2.createElement("CompanyName")
    #        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["InternationalForms"]["Contacts"]['Producer']['CompanyName'])
    #        CompanyName.appendChild(ptext)
    #        Producer.appendChild(CompanyName)
    #
    #        TaxIdentificationNumber = doc2.createElement("TaxIdentificationNumber")
    #        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["InternationalForms"]["Contacts"]['Producer']['TaxIdentificationNumber'])
    #        TaxIdentificationNumber.appendChild(ptext)
    #        Producer.appendChild(TaxIdentificationNumber)
    #
    #        Address = doc2.createElement("Address")
    #        Producer.appendChild(Address)
    #
    #        AddressLine1 = doc2.createElement("AddressLine1")
    #        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["InternationalForms"]["Contacts"]['Producer']['Address']['AddressLine1'])
    #        AddressLine1.appendChild(ptext)
    #        Address.appendChild(AddressLine1)
    #
    #        AddressLine2 = doc2.createElement("AddressLine2")
    #        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["InternationalForms"]["Contacts"]['Producer']['Address']['AddressLine2'])
    #        AddressLine2.appendChild(ptext)
    #        Address.appendChild(AddressLine2)
    #
    #        #AddressLine3 = doc2.createElement("AddressLine3")
    #        #ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["InternationalForms"]["Contacts"]['Producer']['Address']['AddressLine3'])
    #        #AddressLine3.appendChild(ptext)
    #        #Address.appendChild(AddressLine3)
    #
    #        City = doc2.createElement("City")
    #        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["InternationalForms"]["Contacts"]['Producer']['Address']['City'])
    #        City.appendChild(ptext)
    #        Address.appendChild(City)
    #
    #        StateProvinceCode = doc2.createElement("StateProvinceCode")
    #        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["InternationalForms"]["Contacts"]['Producer']['Address']['StateProvinceCode'])
    #        StateProvinceCode.appendChild(ptext)
    #        Address.appendChild(StateProvinceCode)
    #
    #        PostalCode = doc2.createElement("PostalCode")
    #        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["InternationalForms"]["Contacts"]['Producer']['Address']['PostalCode'])
    #        PostalCode.appendChild(ptext)
    #        Address.appendChild(PostalCode)
    #
    #        CountryCode = doc2.createElement("CountryCode")
    #        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["InternationalForms"]["Contacts"]['Producer']['Address']['CountryCode'])
    #        CountryCode.appendChild(ptext)
    #        Address.appendChild(CountryCode)
    #        Product_count = len(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["InternationalForms"]["Product"])
    #        for i in range (0,Product_count):
    #            #///////////
    #            #########code for product dictionary
    #            #///////////
    #
    #            Product = doc2.createElement("Product")
    #            InternationalForms.appendChild(Product)
    #
    #            Description = doc2.createElement("Description")
    #            ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["InternationalForms"]["Product"][i]['Description'])
    #            Product.appendChild(ptext)
    #            Product.appendChild(Description)
    #
    #            Unit = doc2.createElement("Unit")
    #            Product.appendChild(Unit)
    #            Number = doc2.createElement("Number")
    #            ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["InternationalForms"]["Product"][i]['Unit']['Number'])
    #            Number.appendChild(ptext)
    #            Unit.appendChild(Number)
    #
    #            Value = doc2.createElement("Value")
    #            ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["InternationalForms"]["Product"][i]['Unit']['Value'])
    #            Value.appendChild(ptext)
    #            Unit.appendChild(Value)
    #
    #            UnitOfMeasurement = doc2.createElement("UnitOfMeasurement")
    #            Unit.appendChild(UnitOfMeasurement)
    #            Code = doc2.createElement("Code")
    #            ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["InternationalForms"]["Product"][i]['Unit']['UnitOfMeasurement']['Code'])
    #            Code.appendChild(ptext)
    #            UnitOfMeasurement.appendChild(Code)
    #
    #            Description = doc2.createElement("Description")
    #            ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["InternationalForms"]["Product"][i]['Unit']['UnitOfMeasurement']['Description'])
    #            Description.appendChild(ptext)
    #            UnitOfMeasurement.appendChild(Description)
    #
    #            CommodityCode = doc2.createElement("CommodityCode")
    #            ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["InternationalForms"]["Product"][i]['CommodityCode'])
    #            CommodityCode.appendChild(ptext)
    #            Product.appendChild(CommodityCode)
    #
    #            PartNumber = doc2.createElement("PartNumber")
    #            ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["InternationalForms"]["Product"][i]['PartNumber'])
    #            CommodityCode.appendChild(ptext)
    #            Product.appendChild(PartNumber)
    #
    #            OriginCountryCode = doc2.createElement("OriginCountryCode")
    #            ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["InternationalForms"]["Product"][i]['OriginCountryCode'])
    #            OriginCountryCode.appendChild(ptext)
    #            Product.appendChild(OriginCountryCode)
    #
    #            JointProductionIndicator = doc2.createElement("JointProductionIndicator")
    #            ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["InternationalForms"]["Product"][i]['JointProductionIndicator'])
    #            JointProductionIndicator.appendChild(ptext)
    #            Product.appendChild(JointProductionIndicator)
    #
    #            NetCostCode = doc2.createElement("NetCostCode")
    #            ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["InternationalForms"]["Product"][i]['NetCostCode'])
    #            NetCostCode.appendChild(ptext)
    #            Product.appendChild(NetCostCode)
    #
    #            NetCostDateRange = doc2.createElement("NetCostDateRange")
    #            Product.appendChild(NetCostDateRange)
    #            BeginDate = doc2.createElement("BeginDate")
    #            ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["InternationalForms"]["Product"][i]['NetCostDateRange']['BeginDate'])
    #            BeginDate.appendChild(ptext)
    #            NetCostDateRange.appendChild(BeginDate)
    #
    #            EndDate = doc2.createElement("EndDate")
    #            ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["InternationalForms"]["Product"][i]['NetCostDateRange']['EndDate'])
    #            EndDate.appendChild(ptext)
    #            NetCostDateRange.appendChild(EndDate)
    #
    #
    #            ProducerInfo = doc2.createElement("ProducerInfo")
    #            ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["InternationalForms"]["Product"][i]['ProducerInfo'])
    #            ProducerInfo.appendChild(ptext)
    #            Product.appendChild(ProducerInfo)
    #
    #            MarksAndNumbers = doc2.createElement("MarksAndNumbers")
    #            ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["InternationalForms"]["Product"][i]['MarksAndNumbers'])
    #            MarksAndNumbers.appendChild(ptext)
    #            Product.appendChild(MarksAndNumbers)
    #
    #            NumberOfPackagesPerCommodity = doc2.createElement("NumberOfPackagesPerCommodity")
    #            ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["InternationalForms"]["Product"][i]['NumberOfPackagesPerCommodity'])
    #            NumberOfPackagesPerCommodity.appendChild(ptext)
    #            Product.appendChild(NumberOfPackagesPerCommodity)
    #
    #
    #            ProductWeight = doc2.createElement("ProductWeight")
    #            Product.appendChild(ProductWeight)
    #            UnitOfMeasurement = doc2.createElement("UnitOfMeasurement")
    #            ProductWeight.appendChild(UnitOfMeasurement)
    #            Code = doc2.createElement("Code")
    #            ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["InternationalForms"]["Product"][i]['ProductWeight']['UnitOfMeasurement']['Code'])
    #            Code.appendChild(ptext)
    #            UnitOfMeasurement.appendChild(Code)
    #
    #            Description = doc2.createElement("Description")
    #            ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["InternationalForms"]["Product"][i]['ProductWeight']['UnitOfMeasurement']['Description'])
    #            Description.appendChild(ptext)
    #            UnitOfMeasurement.appendChild(Description)
    #
    #            Weight = doc2.createElement("Weight")
    #            ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["InternationalForms"]["Product"][i]['ProductWeight']['Weight'])
    #            Weight.appendChild(ptext)
    #            ProductWeight.appendChild(Weight)
    #
    #            VehicleID = doc2.createElement("VehicleID")
    #            ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["InternationalForms"]["Product"][i]['VehicleID'])
    #            VehicleID.appendChild(ptext)
    #            Product.appendChild(VehicleID)
    #
    #            ScheduleB = doc2.createElement("ScheduleB")
    #            Product.appendChild(ScheduleB)
    #            Number = doc2.createElement("Number")
    #            ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["InternationalForms"]["Product"][i]['ScheduleB']['Number'])
    #            Number.appendChild(ptext)
    #            ScheduleB.appendChild(Number)
    #
    #            Quantity = doc2.createElement("Quantity")
    #            ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["InternationalForms"]["Product"][i]['ScheduleB']['Quantity'])
    #            Quantity.appendChild(ptext)
    #            ScheduleB.appendChild(Quantity)
    #
    #            UnitOfMeasurement = doc2.createElement("UnitOfMeasurement")
    #            ScheduleB.appendChild(UnitOfMeasurement)
    #            Code = doc2.createElement("Code")
    #            ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["InternationalForms"]["Product"][i]['ScheduleB']['UnitOfMeasurement']['Code'])
    #            Code.appendChild(ptext)
    #            UnitOfMeasurement.appendChild(Code)
    #
    #            Description = doc2.createElement("Description")
    #            ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["InternationalForms"]["Product"][i]['ScheduleB']['UnitOfMeasurement']['Description'])
    #            Description.appendChild(ptext)
    #            UnitOfMeasurement.appendChild(Description)
    #
    #            ExportType = doc2.createElement("ExportType")
    #            ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["InternationalForms"]["Product"][i]['ExportType'])
    #            Description.appendChild(ptext)
    #            Product.appendChild(ExportType)
    #
    #            SEDTotalValue = doc2.createElement("SEDTotalValue")
    #            ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["InternationalForms"]["Product"][i]['SEDTotalValue'])
    #            SEDTotalValue.appendChild(ptext)
    #            Product.appendChild(SEDTotalValue)
    #
    #
    #        #/////////end loop in code
    #
    #
    #        InvoiceNumber = doc2.createElement("InvoiceNumber")
    #        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["InternationalForms"]["InvoiceNumber"])
    #        InvoiceNumber.appendChild(ptext)
    #        InternationalForms.appendChild(InvoiceNumber)
    #
    #        InvoiceDate = doc2.createElement("InvoiceDate")
    #        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["InternationalForms"]["InvoiceDate"])
    #        InvoiceNumber.appendChild(ptext)
    #        InternationalForms.appendChild(InvoiceDate)
    #
    #        PurchaseOrderNumber = doc2.createElement("PurchaseOrderNumber")
    #        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["InternationalForms"]["PurchaseOrderNumber"])
    #        InvoiceNumber.appendChild(ptext)
    #        InternationalForms.appendChild(PurchaseOrderNumber)
    #
    #        TermsOfShipment = doc2.createElement("TermsOfShipment")
    #        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["InternationalForms"]["TermsOfShipment"])
    #        TermsOfShipment.appendChild(ptext)
    #        InternationalForms.appendChild(TermsOfShipment)
    #
    #        ReasonForExport = doc2.createElement("ReasonForExport")
    #        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["InternationalForms"]["ReasonForExport"])
    #        ReasonForExport.appendChild(ptext)
    #        InternationalForms.appendChild(ReasonForExport)
    #
    #        Comments = doc2.createElement("Comments")
    #        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["InternationalForms"]["Comments"])
    #        Comments.appendChild(ptext)
    #        InternationalForms.appendChild(Comments)
    #
    #        DeclarationStatement = doc2.createElement("DeclarationStatement")
    #        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["InternationalForms"]["DeclarationStatement"])
    #        DeclarationStatement.appendChild(ptext)
    #        InternationalForms.appendChild(DeclarationStatement)
    #
    ##        Discount = doc2.createElement("Discount")
    ##        InternationalForms.appendChild(Discount)
    ##
    ##        MonetaryValue = doc2.createElement("MonetaryValue")
    ##        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["InternationalForms"]['Discount']["MonetaryValue"])
    ##        MonetaryValue.appendChild(ptext)
    ##        Discount.appendChild(MonetaryValue)
    ##
    ##        FreightCharges = doc2.createElement("FreightCharges")
    ##        InternationalForms.appendChild(FreightCharges)
    ##        MonetaryValue = doc2.createElement("MonetaryValue")
    ##        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["InternationalForms"]['FreightCharges']["MonetaryValue"])
    ##        MonetaryValue.appendChild(ptext)
    ##        FreightCharges.appendChild(MonetaryValue)
    ##
    ##        InsuranceCharges = doc2.createElement("InsuranceCharges")
    ##        InternationalForms.appendChild(InsuranceCharges)
    ##        MonetaryValue = doc2.createElement("MonetaryValue")
    ##        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["InternationalForms"]['InsuranceCharges']["MonetaryValue"])
    ##        MonetaryValue.appendChild(ptext)
    ##        InsuranceCharges.appendChild(MonetaryValue)
    ##
    ##        OtherCharges = doc2.createElement("OtherCharges")
    ##        InternationalForms.appendChild(OtherCharges)
    ##        MonetaryValue = doc2.createElement("MonetaryValue")
    ##        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["InternationalForms"]['OtherCharges']["MonetaryValue"])
    ##        MonetaryValue.appendChild(ptext)
    ##        OtherCharges.appendChild(MonetaryValue)
    ##
    ##        Description = doc2.createElement("Description")
    ##        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["InternationalForms"]['OtherCharges']["Description"])
    ##        Description.appendChild(ptext)
    ##        OtherCharges.appendChild(Description)
    #
    #        CurrencyCode = doc2.createElement("CurrencyCode")
    #        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["InternationalForms"]["CurrencyCode"])
    #        CurrencyCode.appendChild(ptext)
    #        InternationalForms.appendChild(CurrencyCode)
    #
    #        BlanketPeriod = doc2.createElement("BlanketPeriod")
    #        InternationalForms.appendChild(BlanketPeriod)
    #        BeginDate = doc2.createElement("BeginDate")
    #        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["InternationalForms"]['BlanketPeriod']["BeginDate"])
    #        BeginDate.appendChild(ptext)
    #        BlanketPeriod.appendChild(BeginDate)
    #
    #        EndDate = doc2.createElement("EndDate")
    #        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["InternationalForms"]['BlanketPeriod']["EndDate"])
    #        EndDate.appendChild(ptext)
    #        BlanketPeriod.appendChild(EndDate)
    #
    #        ExportDate = doc2.createElement("ExportDate")
    #        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["InternationalForms"]['ExportDate'])
    #        ExportDate.appendChild(ptext)
    #        InternationalForms.appendChild(ExportDate)
    #
    #        ExportingCarrier = doc2.createElement("ExportingCarrier")
    #        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["InternationalForms"]['ExportingCarrier'])
    #        ExportingCarrier.appendChild(ptext)
    #        InternationalForms.appendChild(ExportingCarrier)
    #
    #
    #        CarrierID = doc2.createElement("CarrierID")
    #        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["InternationalForms"]['CarrierID'])
    #        CarrierID.appendChild(ptext)
    #        InternationalForms.appendChild(CarrierID)
    #
    #        InBondCode = doc2.createElement("InBondCode")
    #        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["InternationalForms"]['InBondCode'])
    #        InBondCode.appendChild(ptext)
    #        InternationalForms.appendChild(InBondCode)
    #
    #        EntryNumber = doc2.createElement("EntryNumber")
    #        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["InternationalForms"]['EntryNumber'])
    #        EntryNumber.appendChild(ptext)
    #        InternationalForms.appendChild(EntryNumber)
    #
    #        PointOfOrigin = doc2.createElement("PointOfOrigin")
    #        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["InternationalForms"]['PointOfOrigin'])
    #        PointOfOrigin.appendChild(ptext)
    #        InternationalForms.appendChild(PointOfOrigin)
    #
    #        ModeOfTransport = doc2.createElement("ModeOfTransport")
    #        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["InternationalForms"]['ModeOfTransport'])
    #        ModeOfTransport.appendChild(ptext)
    #        InternationalForms.appendChild(ModeOfTransport)
    #
    #        PortOfExport = doc2.createElement("PortOfExport")
    #        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["InternationalForms"]['PortOfExport'])
    #        PortOfExport.appendChild(ptext)
    #        InternationalForms.appendChild(PortOfExport)
    #
    #        PortOfUnloading = doc2.createElement("PortOfUnloading")
    #        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["InternationalForms"]['PortOfUnloading'])
    #        PortOfUnloading.appendChild(ptext)
    #        InternationalForms.appendChild(PortOfUnloading)
    #
    #        LoadingPier = doc2.createElement("LoadingPier")
    #        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["InternationalForms"]['LoadingPier'])
    #        LoadingPier.appendChild(ptext)
    #        InternationalForms.appendChild(LoadingPier)
    #
    #        PartiesToTransaction = doc2.createElement("PartiesToTransaction")
    #        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["InternationalForms"]['PartiesToTransaction'])
    #        PartiesToTransaction.appendChild(ptext)
    #        InternationalForms.appendChild(PartiesToTransaction)
    #
    #        RoutedExportTransactionIndicator = doc2.createElement("RoutedExportTransactionIndicator")
    #        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["InternationalForms"]['RoutedExportTransactionIndicator'])
    #        RoutedExportTransactionIndicator.appendChild(ptext)
    #        InternationalForms.appendChild(RoutedExportTransactionIndicator)
    #
    #        ContainerizedIndicator = doc2.createElement("ContainerizedIndicator")
    #        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["InternationalForms"]['ContainerizedIndicator'])
    #        ContainerizedIndicator.appendChild(ptext)
    #        InternationalForms.appendChild(ContainerizedIndicator)
    #
    #        License = doc2.createElement("License")
    #        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["InternationalForms"]['License'])
    #        License.appendChild(ptext)
    #        InternationalForms.appendChild(License)
    #
    #        Number = doc2.createElement("Number")
    #        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["InternationalForms"]['Number'])
    #        Number.appendChild(ptext)
    #        InternationalForms.appendChild(Number)
    #
    #        Date = doc2.createElement("Date")
    #        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["InternationalForms"]['Date'])
    #        Date.appendChild(ptext)
    #        InternationalForms.appendChild(Date)
    #
    #        ExceptionCode = doc2.createElement("ExceptionCode")
    #        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["InternationalForms"]['ExceptionCode'])
    #        ExceptionCode.appendChild(ptext)
    #        InternationalForms.appendChild(ExceptionCode)
    #
    #        ECCNNumber = doc2.createElement("ECCNNumber")
    #        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["InternationalForms"]['ECCNNumber'])
    #        ECCNNumber.appendChild(ptext)
    #        InternationalForms.appendChild(ECCNNumber)

    #        ReturnOfDocumentIndicator = doc2.createElement("ReturnOfDocumentIndicator")
    #        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["ReturnOfDocumentIndicator"])
    #        ReturnOfDocumentIndicator.appendChild(ptext)
    #        ShipmentServiceOptions.appendChild(ReturnOfDocumentIndicator)
    #
    #        ImportControlIndicator = doc2.createElement("ImportControlIndicator")
    #        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["ImportControlIndicator"])
    #        ImportControlIndicator.appendChild(ptext)
    #        ShipmentServiceOptions.appendChild(ImportControlIndicator)
    #
    #        LabelMethod = doc2.createElement("LabelMethod")
    #        ShipmentServiceOptions.appendChild(LabelMethod)
    #        Code = doc2.createElement("Code")
    #        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["LabelMethod"]["Code"])
    #        Code.appendChild(ptext)
    #        LabelMethod.appendChild(Code)
    #
    #        Description = doc2.createElement("Description")
    #        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["LabelMethod"]["Description"])
    #        Description.appendChild(ptext)
    #        LabelMethod.appendChild(Description)

    #        CommercialInvoiceRemovalIndicator = doc2.createElement("CommercialInvoiceRemovalIndicator")
    #        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["CommercialInvoiceRemovalIndicator"])
    #        CommercialInvoiceRemovalIndicator.appendChild(ptext)
    #        ShipmentServiceOptions.appendChild(CommercialInvoiceRemovalIndicator)

    #        UPScarbonneutralIndicator = doc2.createElement("UPScarbonneutralIndicator")
    #        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["UPScarbonneutralIndicator"])
    #        UPScarbonneutralIndicator.appendChild(ptext)
    #        ShipmentServiceOptions.appendChild(UPScarbonneutralIndicator)

    #        DeliveryConfirmation = doc2.createElement("DeliveryConfirmation")
    #        ShipmentServiceOptions.appendChild(DeliveryConfirmation)
    #        DCISType = doc2.createElement("DCISType")
    #        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["DeliveryConfirmation"]["DCISType"])
    #        DCISType.appendChild(ptext)
    #        DeliveryConfirmation.appendChild(DCISType)

            Package_count = len(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["Package"])
            for i in range(0,Package_count):
                Package = doc2.createElement("Package")
                Shipment.appendChild(Package)

                Description = doc2.createElement("Description")
                ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["Package"][i]["Description"])
                Description.appendChild(ptext)
                Package.appendChild(Description)
                PackagingType = doc2.createElement("PackagingType")
                Package.appendChild(PackagingType)

                Code = doc2.createElement("Code")
                ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["Package"][i]["PackagingType"]['Code'])
                Code.appendChild(ptext)
                PackagingType.appendChild(Code)
                Description = doc2.createElement("Description")
                ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["Package"][i]["PackagingType"]['Description'])
                Description.appendChild(ptext)
                PackagingType.appendChild(Description)

                Dimensions = doc2.createElement("Dimensions")
                Package.appendChild(Dimensions)

                Length = doc2.createElement("Length")
                ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["Package"][i]["Dimensions"]["Length"])
                Length.appendChild(ptext)
                Dimensions.appendChild(Length)
                Width = doc2.createElement("Width")
                ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["Package"][i]["Dimensions"]["Width"])
                Width.appendChild(ptext)
                Dimensions.appendChild(Width)
                Height = doc2.createElement("Height")
                ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["Package"][i]["Dimensions"]["Height"])
                Height.appendChild(ptext)
                Dimensions.appendChild(Height)
                UnitOfMeasurement = doc2.createElement("UnitOfMeasurement")
                Dimensions.appendChild(UnitOfMeasurement)

                Code = doc2.createElement("Code")
                ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["Package"][i]["Dimensions"]['UnitOfMeasurement']["Code"])
                Code.appendChild(ptext)
                UnitOfMeasurement.appendChild(Code)
                Description = doc2.createElement("Description")
                ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["Package"][i]["Dimensions"]['UnitOfMeasurement']["Description"])
                Description.appendChild(ptext)
                UnitOfMeasurement.appendChild(Description)


                PackageWeight = doc2.createElement("PackageWeight")
                Package.appendChild(PackageWeight)

                UnitOfMeasurement = doc2.createElement("UnitOfMeasurement")
                PackageWeight.appendChild(UnitOfMeasurement)
                Code = doc2.createElement("Code")
                ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["Package"][i]['PackageWeight']['UnitOfMeasurement']["Code"])
                Code.appendChild(ptext)
                UnitOfMeasurement.appendChild(Code)

                Description = doc2.createElement("Description")
                ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["Package"][i]['PackageWeight']['UnitOfMeasurement']["Description"])
                Description.appendChild(ptext)
                UnitOfMeasurement.appendChild(Description)

                Weight = doc2.createElement("Weight")
                ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["Package"][i]['PackageWeight']["Weight"])
                Weight.appendChild(ptext)
                PackageWeight.appendChild(Weight)
    #            LargePackageIndicator = doc2.createElement("LargePackageIndicator")
    ##            ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["Package"][i]["LargePackageIndicator"])
    ##            LargePackageIndicator.appendChild(ptext)
    #            Package.appendChild(LargePackageIndicator)

                ReferenceNumber = doc2.createElement("ReferenceNumber")
                Package.appendChild(ReferenceNumber)

                BarCodeIndicator = doc2.createElement("BarCodeIndicator")
    #            ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["Package"][i]["ReferenceNumber"]["BarCodeIndicator"])
    #            BarCodeIndicator.appendChild(ptext)
                ReferenceNumber.appendChild(BarCodeIndicator)

                Code = doc2.createElement("Code")
                ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["Package"][i]["ReferenceNumber"]["Code"])
                Code.appendChild(ptext)
                ReferenceNumber.appendChild(Code)

                Value = doc2.createElement("Value")
                ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["Package"][i]["ReferenceNumber"]["Value"])
                Value.appendChild(ptext)
                ReferenceNumber.appendChild(Value)

    #            AdditionalHandling = doc2.createElement("AdditionalHandling")
    #            ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["Package"][i]["AdditionalHandling"])
    #            AdditionalHandling.appendChild(ptext)
    #            Package.appendChild(AdditionalHandling)

    #            PackageServiceOptions = doc2.createElement("PackageServiceOptions")
    #            Package.appendChild(PackageServiceOptions)
    #
    #            DeliveryConfirmation = doc2.createElement("DeliveryConfirmation")
    #            PackageServiceOptions.appendChild(DeliveryConfirmation)
    #
    #            DCISType = doc2.createElement("DCISType")
    #            ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["Package"][i]["PackageServiceOptions"]['DeliveryConfirmation']['DCISType'])
    #            DCISType.appendChild(ptext)
    #            DeliveryConfirmation.appendChild(DCISType)
    #
    #            DCISNumber = doc2.createElement("DCISNumber")
    #            ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["Package"][i]["PackageServiceOptions"]['DeliveryConfirmation']['DCISNumber'])
    #            DCISNumber.appendChild(ptext)
    #            DeliveryConfirmation.appendChild(DCISNumber)
    #
    #            InsuredValue = doc2.createElement("InsuredValue")
    #            PackageServiceOptions.appendChild(InsuredValue)
    #
    #
    #            Type = doc2.createElement("Type")
    #            ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["Package"][i]["PackageServiceOptions"]['InsuredValue']['Type'])
    #            Type.appendChild(ptext)
    #            InsuredValue.appendChild(Type)
    #
    #            Code = doc2.createElement("Code")
    #            ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["Package"][i]["PackageServiceOptions"]['InsuredValue']['Code'])
    #            Code.appendChild(ptext)
    #            InsuredValue.appendChild(Code)
    #
    #
    #            Description = doc2.createElement("Description")
    #            ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["Package"][i]["PackageServiceOptions"]['InsuredValue']['Description'])
    #            Description.appendChild(ptext)
    #            InsuredValue.appendChild(Description)
    #
    #            Description = doc2.createElement("CurrencyCode")
    #            ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["Package"][i]["PackageServiceOptions"]['InsuredValue']['CurrencyCode'])
    #            CurrencyCode.appendChild(ptext)
    #            InsuredValue.appendChild(CurrencyCode)
    #
    #            Description = doc2.createElement("MonetaryValue")
    #            ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["Package"][i]["PackageServiceOptions"]['InsuredValue']['MonetaryValue'])
    #            MonetaryValue.appendChild(ptext)
    #            InsuredValue.appendChild(MonetaryValue)
    #
    #
    #            COD = doc2.createElement("COD")
    #            PackageServiceOptions.appendChild(COD)
    #
    #
    #            CODCode = doc2.createElement("CODCode")
    #            ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["Package"][i]["PackageServiceOptions"]['COD']['CODCode'])
    #            CODCode.appendChild(ptext)
    #            COD.appendChild(CODCode)
    #
    #            CODFundsCode = doc2.createElement("CODFundsCode")
    #            ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["Package"][i]["PackageServiceOptions"]['COD']['CODFundsCode'])
    #            CODFundsCode.appendChild(ptext)
    #            COD.appendChild(CODFundsCode)
    #
    #            CODAmount = doc2.createElement("CODAmount")
    #            ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["Package"][i]["PackageServiceOptions"]['COD']['CODAmount'])
    #            CODAmount.appendChild(ptext)
    #            COD.appendChild(CODAmount)
    #
    #            CurrencyCode = doc2.createElement("CurrencyCode")
    #            ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["Package"][i]["PackageServiceOptions"]['InsuredValue']['COD']['CODAmount']['CurrencyCode'])
    #            CurrencyCode.appendChild(ptext)
    #            InsuredValue.appendChild(CurrencyCode)
    #
    #            MonetaryValue = doc2.createElement("MonetaryValue")
    #            ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["Package"][i]["PackageServiceOptions"]['InsuredValue']['COD']['CODAmount']['MonetaryValue'])
    #            MonetaryValue.appendChild(ptext)
    #            InsuredValue.appendChild(MonetaryValue)
    #
    #            VerbalConfirmation = doc2.createElement("VerbalConfirmation")
    #            PackageServiceOptions.appendChild(VerbalConfirmation)
    #
    #            ContactInfo = doc2.createElement("ContactInfo")
    #            VerbalConfirmation.appendChild(ContactInfo)
    #
    #            Name = doc2.createElement("Name")
    #            ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["Package"][i]["PackageServiceOptions"]['VerbalConfirmation']['VerbalConfirmation']['Name'])
    #            Name.appendChild(ptext)
    #            ContactInfo.appendChild(Name)
    #
    #            PhoneNumber = doc2.createElement("PhoneNumber")
    #            ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["Package"][i]["PackageServiceOptions"]['VerbalConfirmation']['VerbalConfirmation']['PhoneNumber'])
    #            PhoneNumber.appendChild(ptext)
    #            ContactInfo.appendChild(PhoneNumber)
    #
    #            ShipperReleaseIndicator = doc2.createElement("ShipperReleaseIndicator")
    #            ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["Package"][i]["PackageServiceOptions"]['ShipperReleaseIndicator'])
    #            ShipperReleaseIndicator.appendChild(ptext)
    #            PackageServiceOptions.appendChild(ShipperReleaseIndicator)
    #
    #
    #            Notification = doc2.createElement("Notification")
    #            PackageServiceOptions.appendChild(Notification)
    #            NotificationCode = doc2.createElement("NotificationCode")
    #            ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["Package"][i]["PackageServiceOptions"]['Notification'])
    #            NotificationCode.appendChild(ptext)
    #            Notification.appendChild(NotificationCode)
    #
    #            EMailMessage = doc2.createElement("EMailMessage")
    #            Notification.appendChild(EMailMessage)
    #            UndeliverableEMailAddress = doc2.createElement("UndeliverableEMailAddress")
    #            ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["Package"][i]["PackageServiceOptions"]["Notification"]["EMailMessage"]["UndeliverableEMailAddress"])
    #            UndeliverableEMailAddress.appendChild(ptext)
    #            EMailMessage.appendChild(UndeliverableEMailAddress)
    #
    #            EMailAddress = doc2.createElement("EMailAddress")
    #            ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["Package"][i]["PackageServiceOptions"]["Notification"]["EMailMessage"]["EMailAddress"])
    #            EMailAddress.appendChild(ptext)
    #            EMailMessage.appendChild(EMailAddress)
    #
    #            FromEMailAddress = doc2.createElement("FromEMailAddress")
    #            ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["Package"][i]["PackageServiceOptions"]["Notification"]["EMailMessage"]["FromEMailAddress"])
    #            FromEMailAddress.appendChild(ptext)
    #            EMailMessage.appendChild(FromEMailAddress)
    #
    #            FromName = doc2.createElement("FromName")
    #            ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["Package"][i]["PackageServiceOptions"]["Notification"]["EMailMessage"]["FromName"])
    #            FromName.appendChild(ptext)
    #            EMailMessage.appendChild(FromName)
    #
    #            Memo = doc2.createElement("Memo")
    #            ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["Package"][i]["PackageServiceOptions"]["Notification"]["EMailMessage"]["Memo"])
    #            Memo.appendChild(ptext)
    #            EMailMessage.appendChild(Memo)
    #
    #            Subject = doc2.createElement("Subject")
    #            ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["Package"][i]["PackageServiceOptions"]["Notification"]["EMailMessage"]["Subject"])
    #            Subject.appendChild(ptext)
    #            EMailMessage.appendChild(Subject)
    #
    #            SubjectCode = doc2.createElement("SubjectCode")
    #            ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["Package"][i]["PackageServiceOptions"]["Notification"]["EMailMessage"]["SubjectCode"])
    #            SubjectCode.appendChild(ptext)
    #            EMailMessage.appendChild(SubjectCode)
    #
    #            ReturnsFlexibleAccessIndicator = doc2.createElement("ReturnsFlexibleAccessIndicator")
    #            ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["Package"][i]["PackageServiceOptions"]['ReturnsFlexibleAccessIndicator'])
    #            ReturnsFlexibleAccessIndicator.appendChild(ptext)
    #            PackageServiceOptions.appendChild(ReturnsFlexibleAccessIndicator)






                #/////////end loop in code



            LabelSpecification = doc2.createElement("LabelSpecification")
            ShipmentConfirmRequest.appendChild(LabelSpecification)

            LabelPrintMethod = doc2.createElement("LabelPrintMethod")
            LabelSpecification.appendChild(LabelPrintMethod)
            Code = doc2.createElement("Code")
            ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["LabelSpecification"]["LabelPrintMethod"]["Code"])
            Code.appendChild(ptext)
            LabelPrintMethod.appendChild(Code)

            Description = doc2.createElement("Description")
            ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["LabelSpecification"]["LabelPrintMethod"]["Description"])
            Description.appendChild(ptext)
            LabelPrintMethod.appendChild(Description)

    #        HTTPUserAgent = doc2.createElement("HTTPUserAgent")
    #        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["LabelSpecification"]["LabelPrintMethod"]["HTTPUserAgent"])
    #        HTTPUserAgent.appendChild(ptext)
    #        LabelSpecification.appendChild(HTTPUserAgent)

            LabelStockSize = doc2.createElement("LabelStockSize")
            LabelSpecification.appendChild(LabelStockSize)
            Height = doc2.createElement("Height")
            ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["LabelSpecification"]["LabelStockSize"]["Height"])
            Height.appendChild(ptext)
            LabelStockSize.appendChild(Height)
            Width = doc2.createElement("Width")
            ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["LabelSpecification"]["LabelStockSize"]["Width"])
            Width.appendChild(ptext)
            LabelStockSize.appendChild(Width)

            LabelImageFormat = doc2.createElement("LabelImageFormat")
            LabelSpecification.appendChild(LabelImageFormat)

            Code = doc2.createElement("Code")
            ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["LabelSpecification"]["LabelImageFormat"]["Code"])
            Code.appendChild(ptext)
            LabelImageFormat.appendChild(Code)

            Description = doc2.createElement("Description")
            ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["LabelSpecification"]["LabelImageFormat"]["Description"])
            Description.appendChild(ptext)
            LabelImageFormat.appendChild(Description)


            request_string2=doc2.toprettyxml()

            request_string=Request_string1+request_string2

            return request_string

    def _get_value(self,obj,template_obj,var):
        try :
            ret=''
            temp1='template_obj.'+var
            temp1=temp1.strip(' ')
            temp1=eval(temp1)
            if temp1 and  '${' in temp1:
                temp=temp1
                temp=''.join(temp.split('\n'))
                temp_list=re.findall('\$\{.*?\}',temp)
                for item in temp_list:
                    ret+=temp[0:temp.index(item)]
                    item=item.strip('')
                    list_item=item.strip('${')
                    list_item=list_item.strip('}')
                    list_item=list_item.split('object.')
                    ret+=eval('obj.'+list_item[-1]) or ''
                    temp=temp[temp.index(item)+len(item):]
                ret+=temp
                return ret
            else:
                return temp1
        except:
            try:
                return eval('template_obj.'+var)
            except:
                return ''
    def send_conf_mail(self,cr,uid,ids,context=None):
        val={}
        for id in ids:
            obj=self.browse(cr, uid, id)
            if obj and obj.address_id and obj.address_id.email:
                template_id=self.pool.get('email.template').search(cr,uid,[('object_name.model','=','stock.picking'),('ship_mail','=',True)] )
                if template_id:
                   template_obj_list=self.pool.get('email.template').browse(cr,uid,template_id)
                   for template_obj in template_obj_list:
                             vals={'email_to':self._get_value(obj,template_obj,'def_to') or '',
                              'body_text':self._get_value(obj,template_obj,'def_body_text') or '',
                              'body_html':self._get_value(obj,template_obj,'def_body_html') or '',
                              'account_id':template_obj.from_account.id,
                              'folder':'outbox',
                              'state':'draft',
                              'subject':self._get_value(obj,template_obj,'def_subject') or '',
                              'email_cc':self._get_value(obj,template_obj,'def_cc') or '',
                              'email_bcc':self._get_value(obj,template_obj,'def_bcc') or '',
                              'email_from':template_obj.from_account.email_id or '',
                              'reply_to':self._get_value(obj,template_obj,'reply_to') or '' ,
                              'date_mail':time.strftime('%Y-%m-%d %H:%M:%S'),
                                      }
                             account = self.pool.get('email_template.mailbox').create(cr, uid,vals)
        return True

    def process_ship(self,cr, uid, ids, context=None):
        deliv_order = self.browse(cr, uid, type(ids)==type([]) and ids[0] or ids, context=context)
        error_flag=False
        ship_move_ids={}
        do_transaction = True
        response_dic = {}
        if not (deliv_order.logis_company or deliv_order.shipper or deliv_order.ups_service):
            raise osv.except_osv("Warning", "Please select Logistics Company, Shipper and Shipping Service")
#        if deliv_order.sale_id.id and deliv_order.sale_id.payment_method == 'cc_pre_auth' and not deliv_order.sale_id.invoiced:
#            do_transaction = False
#            rel_voucher_id = deliv_order.sale_id.rel_account_voucher_id.id
#            if rel_voucher_id and deliv_order.sale_id.rel_account_voucher_id.state != 'posted' and deliv_order.sale_id.rel_account_voucher_id.cc_auth_code:
#                self.pool.get('account.voucher').write(cr,uid,[rel_voucher_id],{'cc_p_authorize':False,'cc_charge':True})
#                do_transaction = self.pool.get('account.voucher').authorize(cr, uid, [rel_voucher_id], context=context)
#        if  not do_transaction:
#            self.write(cr,uid,ids,{'ship_state':'hold','ship_message':'Unable to process creditcard payment.'})
#            cr.commit()
#            raise osv.except_osv(_('Final credit card charge cannot be completed!'),_("Please hold shipment and contact customer service..") )
        warning_error = False
        for pack_obj in deliv_order.packages_ids:
            ship_confirm_request_xml=self.create_ship_confirm_request(cr, uid,pack_obj.id)
            ship_confirm_web=''
            ship_confirm_port=''

            if deliv_order.logis_company:
                if deliv_order.logis_company.test_mode:
                    ship_confirm_web=deliv_order.logis_company.ship_req_test_web
                    ship_confirm_port=deliv_order.logis_company.ship_req_test_port

                else:
                    ship_confirm_web=deliv_order.logis_company.ship_req_web
                    ship_confirm_port=deliv_order.logis_company.ship_req_port

                if ship_confirm_web:
                    parse_url = urlparse(ship_confirm_web)
                    serv = parse_url.netloc
                    serv_path = parse_url.path
                else:
                    raise osv.except_osv(_('Unable to find Shipping URL!'),_("Please configure the shipping company with websites."))

                conn=httplib.HTTPSConnection(serv,ship_confirm_port)
                res=conn.request("POST",serv_path,ship_confirm_request_xml)
                """1.make and call function to send request/ 2.make function to process the response and write it """


                res=conn.getresponse()
                result = res.read()
                response_dic=xml2dic.main(result)


                response = ''
                
            if not response_dic:
                continue
            
            for res_elm in  response_dic['ShipmentConfirmResponse'][0]['Response']:
                if res_elm.get('ResponseStatusCode'):
                    response = '\nResponse Status Code: '+str(res_elm['ResponseStatusCode'])
                if res_elm.get('ResponseStatusDescription'):
                    response += '\nResponse Description: '+str(res_elm['ResponseStatusDescription'])
                if res_elm.get('Error'):
                    
                    error_flag =True
                    for err_elm in res_elm.get('Error'):
                        if err_elm.get('ErrorSeverity',False):
                            response += '\nErrorSeverity: '+str(err_elm['ErrorSeverity'])
                            if str(err_elm['ErrorSeverity']) == 'Warning':
                                warning_error = True
                                error_flag = False
                        if err_elm.get('ErrorCode',False):
                            response += '\nErrorCode: '+str(err_elm['ErrorCode'])
                        if err_elm.get('ErrorDescription',False):
                            response += '\nErrorDescription: '+str(err_elm['ErrorDescription'])
                    self.write(cr,uid,ids,{'ship_message':'Ship Processing:'+str(response)})
                    if not warning_error:
                        break
            temp=""
            if len(response_dic['ShipmentConfirmResponse'])>5 and response_dic['ShipmentConfirmResponse'][5].has_key('ShipmentDigest') :
                temp +=str(response_dic['ShipmentConfirmResponse'][5]['ShipmentDigest'])
            self.pool.get('stock.packages').write(cr,uid,[pack_obj.id],{'ship_message':str(response),'shipment_digest':str(temp)})


            if int(time.strftime("%w")) in range(1,6) or (time.strftime("%w") == '6' and deliv_order.sat_delivery):
                next_pic_date = time.strftime("%Y-%m-%d")
            else:
                timedelta = datetime.timedelta(7-int(time.strftime("%w")))
                next_pic_date = (datetime.datetime.today()+timedelta).strftime("%Y-%m-%d")

            value_dic={'pick_id':deliv_order.id ,
                       'package_weight':pack_obj.weight,
                       'state':deliv_order.ship_state,
                       'partner_id':deliv_order.address_id.id and deliv_order.address_id.partner_id.id,
                       'service':deliv_order.ups_service.id,
                       'ship_to':deliv_order.address_id.id,
                       'ship_from':deliv_order.ship_from and deliv_order.ship_from_address.id  or deliv_order.shipper.id and deliv_order.shipper.address.id,
                       'package':pack_obj.description and str(pack_obj.description)[:126],
                       'pic_date':next_pic_date,
                       'sale_id':deliv_order.sale_id.id and deliv_order.sale_id.id or False,
                       }
            if not error_flag:
                ship_move_ids[pack_obj.id] = self.pool.get('shipping.move').create(cr, uid, value_dic)
                self.process_ship_accept(cr, uid, ids,ship_move_ids,pack_obj,context=context)
#            else:
#                error_flag =True
#                try:
        if not error_flag:
            if warning_error:
                ship_msg = 'Shipping success with warning. Please see the individual package details.'
            else:
                ship_msg = 'Success'
            self.write(cr,uid,ids,{'ship_state':'ready_pick','ship_message':ship_msg})
            try:
                self.send_conf_mail(cr,uid,ids,context=context)
            except Exception,e:
                pass
            if ship_move_ids:
                self.pool.get('shipping.move').write(cr, uid, ship_move_ids.values(),{'state':'ready_pick'})
            return {
                    'type': 'ir.actions.report.xml',
                    'report_name':'multiple.label.print',
                    'datas': {
                            'model':'stock.picking',
                            'id': ids and ids[0] or False,
                            'ids': ids and ids or [],
                            'report_type': 'pdf'
                        },
                    'nodestroy': True
                    }


#                except Exception,e:
#                    pass
        return True


stock_picking()

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
