# -*- coding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2011 NovaPoint Group LLC (<http://www.novapointgroup.com>)
#    Copyright (C) 2004-2010 OpenERP SA (<http://www.openerp.com>)
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>
#
##############################################################################

from osv import fields,osv
import re



class ups_account_shipping(osv.osv):
    _name = "ups.account.shipping"
    _columns = {
                'name':fields.char('Name', size=64, required=True, select=1),
                'atten_name':fields.char('AttentionName', size=64, required=True, select=1),
                'accesslicensenumber': fields.char('AccessLicenseNumber', size= 64, required=True, select=1),
                'userid': fields.char('UserId',size=64 , required=True, select=1),
                'password': fields.char('Password', size=64, required=True, select=1),
                'active':fields.boolean('Active', size=64,),
                'acc_no':fields.char('Account Number',size=64 , required=True, select=1, help="Shipper's six digit account number."),
                'tax_id_no':fields.char('Tax Identification Number',size=64 ,  select=1, help="Shipper's Tax Identification Number."),
                'logistic_company_id':fields.many2one('logistic.company','Parent Logistic Company'),
                'ups_shipping_service_ids':fields.one2many('ups.shipping.service.type','ups_account_id','Shipping Service'),
                'address':fields.many2one('res.partner.address','Shipper Address'),
                'trademark':fields.char('Trademark', size=1024, select=1),
                }
    _defaults={
               'active':True
               }
ups_account_shipping()


class ups_account_shipping_service(osv.osv):
    _name = "ups.shipping.service.type"
    _rec_name = "description"
    _columns = {
                    'description': fields.char('Description', size=32, required=True, select=1),
                    'category': fields.char('Category', size=32, select=1),
                    'shipping_service_code': fields.char('Shipping Service Code', size=8, select=1),
                    'rating_service_code': fields.char('Rating Service Code', size=8, select=1),
                    'ups_account_id':fields.many2one('ups.account.shipping','Parent Shipping Account'),
                }
ups_account_shipping_service()




class logistic_company(osv.osv):
    _name = "logistic.company"
    _inherit="logistic.company"
    _columns = {
                    'company_id':fields.many2one('res.company','Company'),
                    'ups_shipping_account_ids': fields.one2many('ups.account.shipping','logistic_company_id','Shipping Account'),
                }


    def onchange_shipping_number(self, cr, uid, ids, shipping_no,url,context=None):
        ret={}
        if url:
            b=url[url.rindex('/'):len(url)]
            b=b.strip('/')
            if re.match("^[0-9]*$",b):
                url=url[0:url.rindex('/')]
            url+=('/'+shipping_no)
            ret['url']=url
        return{'value':ret}

logistic_company()


class res_company(osv.osv):
        _inherit = "res.company"
        _columns = {
                    'logistic_company_ids': fields.one2many('logistic.company','company_id','Shipping Account'),
                   }

res_company()


class shipping_move(osv.osv):
    _inherit = "shipping.move"
    _columns = {
                    'shipper': fields.many2one('ups.account.shipping','Shipper',help='The specific user ID and shipper. Setup in the company configuration.'),
                }
shipping_move()







# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4: