# -*- coding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2011 NovaPoint Group LLC (<http://www.novapointgroup.com>)
#    Copyright (C) 2004-2010 OpenERP SA (<http://www.openerp.com>)
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>
#
##############################################################################

from osv import fields, osv
from tools.translate import _
import tools
from xml.dom.minidom import Document
import httplib
import xml2dic
import base64
from urlparse import urlparse
pack_type={ '01' : 'UPS Letter',
            '02' : 'Customer Supplied Package',
            '03' : 'Tube',
            '04' : 'PAK',
            '21' : 'UPS ExpressBox',
            '24' : 'UPS 25KG Box',
            '25' : 'UPS 10KG Box',
            '30' : 'Pallet',
            '2a' : 'Small Express Box',
            '2b' : 'Medium Express Box',
            '2c' : 'Large Express Box'}


class quick_ship(osv.osv_memory):
    _name = "quick.ship"
    _description = "Quick Ship"
    _rec_name = 'saleorder_id'
    _columns = {
                'ship_log_id':fields.many2one('shipping.move','Ship Move'),
                'saleorder_id':fields.many2one('sale.order','Sale Order'),
                'sender':fields.many2one('res.partner','Sender'),
                'send_addr':fields.many2one('res.partner.address','Sender Address'),
                'recipient':fields.many2one('res.partner','Recipient'),
                'address_id':fields.many2one('res.partner.address','Address',help="Ship to address"),
                'quantum_view':fields.boolean('Send QuantumView Notifications'),

                'bill_to':fields.many2one('ups.account.shipping','Bill to'),

                'logis_company': fields.many2one('logistic.company','Logistics Company',help='Name of the Logistics company providing the shipper services.'),
                'service': fields.many2one('ups.shipping.service.type','UPS Service',help='The specific UPS Service offered'),
                'package_type': fields.selection([('01', 'UPS Letter'),
                                ('02', 'Customer Supplied Package'),
                                ('03', 'Tube'),
                                ('04', 'PAK'),
                                ('21', 'UPS ExpressBox'),
                                ('24', 'UPS 25KG Box'),
                                ('25', 'UPS 10KG Box'),
                                ('30', 'Pallet'),
                                ('2a', 'Small Express Box'),
                                ('2b', 'Medium Express Box'),
                                ('2c', 'Large Express Box')],'Package Type',help='Indicates the type of package – taken from Package Type on the Shipment or can be overridden here on individual package.'),
                'length': fields.float('Length',help='Indicates the longest length of the box in inches.'),
                'width': fields.float('Width'),
                'height': fields.float('Height'),
                'weight': fields.float('Weight (lbs)'),
                'insured_val': fields.float('Insured Value'),
                'addn_service':fields.boolean('Additional Service Options'),
                'description': fields.char('Description', size=256,),
                'inv_no': fields.char('Invoice Number', size=64,),
                'ref_no': fields.char('Reference Number', size=64,),
                'ref1': fields.selection([  ('AJ','Accounts Receivable Customer Account'),
                            ('AT','Appropriation Number'),
                            ('BM','Bill of Lading Number'),
                            ('9V','Collect on Delivery (COD) Number'),
                            ('ON','Dealer Order Number'),
                            ('DP','Department Number'),
                            ('3Q','Food and Drug Administration (FDA) Product Code'),
                            ('IK','Invoice Number'),
                            ('MK','Manifest Key Number'),
                            ('MJ','Model Number'),
                            ('PM','Part Number'),
                            ('PC','Production Code'),
                            ('PO','Purchase Order Number'),
                            ('RQ','Purchase Request Number'),
                            ('RZ','Return Authorization Number'),
                            ('SA','Salesperson Number'),
                            ('SE','Serial Number'),
                            ('ST','Store Number'),
                            ('TN','Transaction Reference Number'),
                            ('EI','Employer ID Number'),
                            ('TJ','Federal Taxpayer ID No.'),
                            ('SY','Social Security Number'),],'Reference Code',help='Indicates the type of package'),

                'rate': fields.float('Rate',),
                ########Additional Service Options
                'sat_delivery': fields.boolean('Saturday Delivery',help='Indicates is it is appropriate to send delivery on Saturday.'),
                'verbal_confirm': fields.boolean('Verbal Confirmation of Delivery'),
                'con_name': fields.char('Name', size=128,),
                'con_phone': fields.char('Phone', size=64,),
                'cod': fields.boolean('Collect on Delivery (COD)'),
                'amount': fields.float('Amount',),
                'acc_cash_moneyorder': fields.boolean('Accept Cashier Check/Money Order Only'),
                'shipper_release': fields.boolean('Shipper Release'),
                'deliv_conf': fields.boolean('Delivery Confirmation'),
                'dc_opt': fields.selection([('', ''), ('', '')],'DC Options'),
                'deliv_conf': fields.boolean('Offset the Climate Impact of This Shipment',help="UPS Carbon Neutral"),
                'addnl_handling': fields.boolean('Additional Handling'),
                'response': fields.text('Response'),
                'logo':fields.binary('Logo'),
    }

    def onchange_sale_id(self, cr, uid, ids, saleorder_id, context=False):
        ret = {'value':{}}
        if saleorder_id:
            sale_id = self.pool.get('sale.order').browse(cr,uid,saleorder_id,context=context)
            ret['value']['recipient'] = sale_id.partner_id.id
            ret['value']['address_id'] = sale_id.partner_shipping_id.id
            ups_shipping_service_ids=self.pool.get('ups.shipping.service.type').search(cr,uid,[('description','like',sale_id.ship_method)])
            if ups_shipping_service_ids:
                ups_shipping_service=self.pool.get('ups.shipping.service.type').browse(cr,uid,ups_shipping_service_ids)[0]
                ret['value']['service'] = ups_shipping_service.id
                ret['value']['bill_to'] = ups_shipping_service.ups_account_id.id
                ret['value']['logis_company'] = ups_shipping_service.ups_account_id.logistic_company_id.id
        return ret

    def fill_addr(self,addr_id):
        ret = {'AddressLine1':addr_id and addr_id.street or '',
         'AddressLine2':addr_id and addr_id.street2 or '',
         'AddressLine3':"",
        'City':addr_id and addr_id.city or '',
        'StateProvinceCode':addr_id and addr_id.state_id.id and addr_id.state_id.code or '',
        'PostalCode':addr_id and addr_id.zip or '',
        'CountryCode':addr_id and addr_id.country_id.id and addr_id.country_id.code or '',
        'ResidentialAddress':""
        }
        if addr_id and ('zip_id' in addr_id._columns.keys()):
            ret['PostalCode'] = addr_id.zip_id.id and addr_id.zip_id.zipcode and str(addr_id.zip_id.zipcode) or ''

        return ret
    def get_package(self,quick_ship_id):
        if not quick_ship_id:
            return []
        ret = []
        ret.append({
            'Description':quick_ship_id.description or "",
            'PackagingType':{'Code':quick_ship_id.package_type or "",
                             'Description':pack_type.get(quick_ship_id.package_type, "")},#PackagingType
            'Dimensions':{'UnitOfMeasurement':{'Code':"IN",'Description':"Inches"},'Length':str(quick_ship_id.length or "0"),'Width':str(quick_ship_id.width or "0"),'Height':str(quick_ship_id.height or "0")},#Dimensions
            'PackageWeight':{'UnitOfMeasurement':{'Code':"LBS",'Description':"Pounds"},'Weight':str(quick_ship_id.weight or "0")},
#                            'LargePackageIndicator':"",
            'ReferenceNumber':{'BarCodeIndicator':"",'Code':quick_ship_id.ref1 or "",'Value':""},
                            'AdditionalHandling':"",
                            'PackageServiceOptions':{'DeliveryConfirmation':{'DCISType':"",'DCISNumber':""},#DeliveryConfirmation
                                                    'InsuredValue':{'Type':"",'Code':"",'Description':"",
                                                                    'CurrencyCode':"",'MonetaryValue':""},#InsuredValue
#                                                    'COD':{'CODCode':"",'CODFundsCode':"",'CODAmount':{'CurrencyCode':"",
#                                                                                                        'MonetaryValue':""
#                                                                                                        },#CODAmount
#                                                            },#COD
                                                    'VerbalConfirmation':{'ContactInfo':{'Name':"",'PhoneNumber':""}#ContactInfo
                                                                           },#VerbalConfirmation
                                                    'ShipperReleaseIndicator':"",
                                                    'Notification':{'NotificationCode':"",'EMailMessage':{'EMailAddress':"",'UndeliverableEMailAddress':"",
                                                                                                           'FromEMailAddress':"",'FromName':"",'Memo':"",
                                                                                                           'Subject':"",'SubjectCode':""}#EMailMessage
                                                                    },#Notification,
                                                    'ReturnsFlexibleAccessIndicator':"",
                                        }#PackageServiceOptions
            })
        return ret

    def process_ship(self,cr, uid, ids, context=None):
        quick_ship_id = self.browse(cr, uid, ids[0], context=context)
        data_for_Access_Request={
                        'AccessLicenseNumber':quick_ship_id.bill_to.id and quick_ship_id.bill_to.accesslicensenumber or '',
                        'UserId':quick_ship_id.bill_to.id and quick_ship_id.bill_to.userid or '',
                        'Password':quick_ship_id.bill_to.id and quick_ship_id.bill_to.password or ''
                    }
        data_for_confirm_request={'ShipmentConfirmRequest':{

                                  'Request':{'RequestAction':"ShipConfirm",#-----must be ShipConfirm
                                            'RequestOption':"nonvalidate",#----------adress validation
                                            'TransactionReference':{'CustomerContext':"",},#---------optional data
                                            },#Request

                                  'Shipment':{'Description':quick_ship_id.description or "Shipping",
                                              'ReturnService':{'Code':"8", 'DocumentsOnly':"" },
                                              'Shipper':{ 'Name':quick_ship_id.bill_to.id and quick_ship_id.bill_to.name or "",
                                                          'AttentionName':quick_ship_id.bill_to.id and quick_ship_id.bill_to.atten_name or "",
                                                          'ShipperNumber':quick_ship_id.bill_to.id and quick_ship_id.bill_to.acc_no or "",
                                                          'TaxIdentificationNumber':quick_ship_id.bill_to.id and quick_ship_id.bill_to.tax_id_no or "",
                                                          'PhoneNumber':quick_ship_id.bill_to.id and quick_ship_id.bill_to.address and quick_ship_id.bill_to.address.phone or "",
                                                          'FaxNumber':quick_ship_id.bill_to.id and quick_ship_id.bill_to.address and quick_ship_id.bill_to.address.fax or "",
                                                          'EMailAddress':quick_ship_id.bill_to.id and quick_ship_id.bill_to.address and quick_ship_id.bill_to.address.email or "",
                                                          'Address':self.fill_addr(quick_ship_id.address_id.id and quick_ship_id.address_id or '')
                                                         },#Shipper

                                              'ShipTo': {'CompanyName':quick_ship_id.address_id.company_id and quick_ship_id.address_id.company_id.name or '',
                                                        'AttentionName':"",
                                                        'TaxIdentificationNumber':"",
                                                        'PhoneNumber':quick_ship_id.address_id.id and quick_ship_id.address_id.phone or '',
                                                        'FaxNumber':quick_ship_id.address_id.id and quick_ship_id.address_id.fax or '',
                                                        'EMailAddress':quick_ship_id.address_id.id and quick_ship_id.address_id.email or '',
                                                        'Address':self.fill_addr(quick_ship_id.address_id.id and quick_ship_id.address_id),
                                                        'LocationID':"",
                                                         },#ShipTo



                                              'ShipFrom':{'CompanyName':"company1111",
                                                          'AttentionName':"",
                                                          'TaxIdentificationNumber':'',
                                                          'PhoneNumber':quick_ship_id.send_addr.id and quick_ship_id.send_addr.phone or '',
                                                          'FaxNumber':quick_ship_id.send_addr.id and quick_ship_id.send_addr.fax or '',
                                                          'Address':self.fill_addr(quick_ship_id.send_addr.id and quick_ship_id.send_addr),
                                                          },

#                                              'SoldTo':{'Option':"",
#                                                        'CompanyName':"",
#                                                        'AttentionName':"",
#                                                        'TaxIdentificationNumber':"",
#                                                        'PhoneNumber':"",
#                                                        'Address':{'AddressLine1':"",
#                                                                   'AddressLine2':"",
#                                                                   'City':"",
#                                                                   'StateProvinceCode':"",
#                                                                   'PostalCode':"",
#                                                                   'CountryCode':"",}
#                                                      },#SoldTo,

                                              'PaymentInformation':{'Prepaid':{
                                                                            'BillShipper':{'AccountNumber':quick_ship_id.bill_to and quick_ship_id.bill_to.acc_no or "",
#                                                                                           'CreditCard':{'Type':"",
#                                                                                                      'Number':"",
#                                                                                                      'ExpirationDate':"",
#                                                                                                      'SecurityCode':"",
#                                                                                                      'Address':{'AddressLine1':"",
#                                                                                                                 'AddressLine2':"",
#                                                                                                                 'AddressLine3':"",
#                                                                                                                 'City':"",
#                                                                                                                 'StateProvinceCode':"",
#                                                                                                                 'PostalCode':"",
#                                                                                                                 'CountryCode':"",},
#                                                                                                      },#CreditCard
                                                                                            },#BillShipper
                                                                                },#Prepaid
#                                                                            'BillThirdParty':{'BillThirdPartyShipper':{'AccountNumber':"",
#                                                                                                                            'ThirdParty':{
#                                                                                                                                          'Address':{'PostalCode':"",
#                                                                                                                                                     'CountryCode':"",},#Address
#                                                                                                                                         },#ThirdParty
#                                                                                                                        },#BillThirdPartyShipper
#
#                                                                                                        },#BillThirdParty
#
#                                                                            'FreightCollect':{'BillReceiver':{'AccountNumber':"",
#                                                                                                              'Address':{'PostalCode':"",},#Address
#                                                                                                 },#BillReceiver
#                                                                                 },#FreightCollect
#
#                                                                           'ConsigneeBilled':"",
                                                                  },#PaymentInformation,

#                                 'ItemizedPaymentInformation':{ 'ShipmentCharge':{'Type':"",
#                                                                                 'BillShipper':{'AccountNumber':"",
#                                                                                                'CreditCard':{'Type':"",
#                                                                                                              'Number':"",
#                                                                                                               'ExpirationDate':"",
#                                                                                                               'SecurityCode':"",
#                                                                                                               'Address':{'AddressLine1':"",
#                                                                                                                          'AddressLine2':"",
#                                                                                                                          'AddressLine3':"",
#                                                                                                                          'City':"",
#                                                                                                                          'StateProvinceCode':"",
#                                                                                                                          'PostalCode':"",
#                                                                                                                          'CountryCode':"",},
#                                                                                                                },#CreditCard
#                                                                                                    },#BillShipper
#                                                                             'BillReceiver':{'AccountNumber':"",'Address':{'PostalCode':"",},#Address
#                                                                                            },#BillReceiver
#                                                                             'BillThirdParty':{'BillThirdPartyShipper':{'AccountNumber':"",
#                                                                                                                        'ThirdParty':{
#                                                                                                                    'Address':{'PostalCode':"",'CountryCode':"",},#Address
#                                                                                                                                    },#ThirdParty
#                                                                                                                   },#BillThirdPartyShipper
#
#                                                                                                'BillThirdPartyConsignee':{'AccountNumber':"",'ThirdParty':{'Address':{'PostalCode':"",
#                                                                                                                                                                    'CountryCode':"",}, #Address
#                                                                                                                                                            },#ThirdParty
#                                                                                                                           },#BillThirdPartyConsignee
#                                                                                                },#BillThirdParty
#                                                                             'ConsigneeBilled':"",
#                                                                                     },#ShipmentCharge
#                                                                'SplitDutyVATIndicator':"",
#                                                            },#ItemizedPaymentInformation

                                 'GoodsNotInFreeCirculationIndicator':"",
                                 'RateInformation':{'NegotiatedRatesIndicator':""},#RateInformation
#                                 'MovementReferenceNumber':"",
#                                 'ReferenceNumber':{'BarCodeIndicator':"",'Code':"",'Value':"",},#ReferenceNumber,
                                 'Service':{'Code':quick_ship_id.service.id and str(quick_ship_id.service.shipping_service_code) or "",
                                            'Description':quick_ship_id.service.id and quick_ship_id.service.description or ""},#Service
                                 'InvoiceLineTotal':{'CurrencyCode':"",'MonetaryValue':str(quick_ship_id.amount) or ""},#InvoiceLineTotal
                                 'ShipmentServiceOptions':{'SaturdayDelivery':"",
                                                           'COD':{'CODCode':"",'CODFundsCode':"",
                                                                                        'CODAmount':{'CurrencyCode':"",
                                                                                                     'MonetaryValue':""
                                                                                                    },#CODAmount
                                                                                        },#COD
                                                           'Notification':[],
                                                           'LabelDelivery':{'LabelLinkIndicator':"",
                                                                            'EMailMessage':{'EMailAddress':"",
                                                                                           'UndeliverableEMailAddress':"",
                                                                                           'FromEMailAddress':"",
                                                                                           'FromName':"",
                                                                                           'Memo':"",
                                                                                           'Subject':"",
                                                                                           'SubjectCode':""}#EMailMessage
                                                                            },#LabelDelivery

                                                           'InternationalForms':{'FormType':"01",
                                                                                 'AdditionalDocumentIndicator':"",
                                                                                 'FormGroupIdName':"",
                                                                                 'SEDFilingOption':"",
                                                                                 'Contacts':{
                                                                                             'ForwardAgent':{'CompanyName':"",
                                                                                                             'TaxIdentificationNumber':"",
                                                                                                             'Address':{'AddressLine1':"",
                                                                                                                        'AddressLine2':"",
                                                                                                                        'City':"",
                                                                                                                        'StateProvinceCode':"",
                                                                                                                        'PostalCode':"",
                                                                                                                        'CountryCode':"",
                                                                                                                        },#Address

                                                                                                                },#ForwardAgent
                                                                                               'UltimateConsignee':{'CompanyName':"",
                                                                                                                    'Address':{'AddressLine1':"",
                                                                                                                               'AddressLine2':"",
                                                                                                                               'City':"",
                                                                                                                                'StateProvinceCode':"",
                                                                                                                                'PostalCode':"",
                                                                                                                                'CountryCode':"",
                                                                                                                                },#Address
                                                                                                                    },#UltimateConsignee

                                                                                               'IntermediateConsignee':{'CompanyName':"",
                                                                                                                        'Address':{'AddressLine1':"",
                                                                                                                                   'AddressLine2':"",
                                                                                                                                   'City':"",
                                                                                                                                   'StateProvinceCode':"",
                                                                                                                                   'PostalCode':"",
                                                                                                                                   'CountryCode':"",
                                                                                                                                   },#Address
                                                                                                                        },#IntermediateConsignee

                                                                                                'Producer':{'Option':"",
                                                                                                            'CompanyName':"",
                                                                                                            'TaxIdentificationNumber':"",
                                                                                                            'Address':{'AddressLine1':"",
                                                                                                                       'AddressLine2':"",
                                                                                                                       'City':"",
                                                                                                                       'StateProvinceCode':"",
                                                                                                                       'PostalCode':"",
                                                                                                                       'CountryCode':"",
                                                                                                                       },#Address
                                                                                                            },#Producer
                                                                                             },#Contacts
                                                                                 'Product':[],##details in another file
                                                                                'InvoiceNumber':"",
                                                                                'InvoiceDate':"",
                                                                                'PurchaseOrderNumber':"",
                                                                                'TermsOfShipment':"",
                                                                                'ReasonForExport':"SALE",
                                                                                'Comments':"",
                                                                                'DeclarationStatement':"",
#                                                                                'Discount':{'MonetaryValue':"0"},#Discount
#                                                                                'FreightCharges':{'MonetaryValue':""},#FreightCharges
#                                                                                'InsuranceCharges':{'MonetaryValue':""},#InsuranceCharges
#                                                                                'OtherCharges':{'MonetaryValue':"",'Description':""},#OtherCharges
                                                                                'CurrencyCode':"",
                                                                                'BlanketPeriod':{'BeginDate':"",'EndDate':""},#BlanketPeriod
                                                                                'ExportDate':"",'ExportingCarrier':"",'CarrierID':"",'InBondCode':"",'EntryNumber':"",
                                                                                'PointOfOrigin':"",'ModeOfTransport':"",'PortOfExport':"",'PortOfUnloading':"",
                                                                                'LoadingPier':"",'PartiesToTransaction':"",'RoutedExportTransactionIndicator':"",
                                                                                'ContainerizedIndicator':"",'License':"",'Number':"",'Date':"",'ExceptionCode':"",
                                                                                'ECCNNumber':"",
                                                                            },#InternationalForms
#                                                           'ReturnOfDocumentIndicator':"",
#                                                           'ImportControlIndicator':"",
#                                                           'LabelMethod':{'Code':"",'Description':"",},#LabelMethod
#                                                           'CommercialInvoiceRemovalIndicator':"",
#                                                           'UPScarbonneutralIndicator':"",
#                                                           'DeliveryConfirmation':{'DCISType':"1",}#DeliveryConfirmation
                                                         },#ShipmentServiceOptions,
                                 'Package':self.get_package(quick_ship_id)## this is done in different file
                                                },#Shipment
                'LabelSpecification':{'LabelPrintMethod':{'Code':"GIF", 'Description':"GIF",},
                                      'HTTPUserAgent':"",
                                      'LabelStockSize':{'Height':"4",'Width':"6"},
                                      'LabelImageFormat':{'Code':"GIF",'Description':"GIF"},
                                    },#LabelSpecification

            },#ShipmentConfirmRequest
        }

#        print data_for_confirm_request




        doc1 = Document()
        AccessRequest = doc1.createElement("AccessRequest")
        AccessRequest.setAttribute("xml:lang", "en-US")
        doc1.appendChild(AccessRequest)


        AccessLicenseNumber = doc1.createElement("AccessLicenseNumber")
        ptext = doc1.createTextNode(data_for_Access_Request["AccessLicenseNumber"])
        AccessLicenseNumber.appendChild(ptext)
        AccessRequest.appendChild(AccessLicenseNumber)

        UserId = doc1.createElement("UserId")
        ptext = doc1.createTextNode(data_for_Access_Request["UserId"])
        UserId.appendChild(ptext)
        AccessRequest.appendChild(UserId)

        Password = doc1.createElement("Password")
        ptext = doc1.createTextNode(data_for_Access_Request["Password"])
        Password.appendChild(ptext)
        AccessRequest.appendChild(Password)

        Request_string1=doc1.toprettyxml()
#        print Request_string1



        ######SECOND PART

        Notification_count=0#max value =3
        Product_count=0#max value =50
        Package_count=0#max value =200
        doc2 = Document()



        ShipmentConfirmRequest = doc2.createElement("ShipmentConfirmRequest")
        doc2.appendChild(ShipmentConfirmRequest)



        Request = doc2.createElement("Request")
        ShipmentConfirmRequest.appendChild(Request)

        RequestAction = doc2.createElement("RequestAction")
        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Request"]["RequestAction"])
        RequestAction.appendChild(ptext)
        Request.appendChild(RequestAction)

        RequestOption = doc2.createElement("RequestOption")
        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Request"]["RequestOption"])
        RequestOption.appendChild(ptext)
        Request.appendChild(RequestOption)

        TransactionReference = doc2.createElement("TransactionReference")
        Request.appendChild(TransactionReference)

        CustomerContext = doc2.createElement("CustomerContext")
        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Request"]["TransactionReference"]['CustomerContext'])
        CustomerContext.appendChild(ptext)
        TransactionReference.appendChild(CustomerContext)

        Shipment = doc2.createElement("Shipment")
        ShipmentConfirmRequest.appendChild(Shipment)

        Description = doc2.createElement("Description")
        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["Description"])
        Description.appendChild(ptext)
        Shipment.appendChild(Description)
        if 0:#quick_ship_id.with_ret_service:
            ReturnService = doc2.createElement("ReturnService")
            Shipment.appendChild(ReturnService)


            Code = doc2.createElement("Code")
            ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ReturnService"]["Code"])
            Code.appendChild(ptext)
            ReturnService.appendChild(Code)
#
#        DocumentsOnly = doc2.createElement("DocumentsOnly")
#        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ReturnService"]["DocumentsOnly"])
#        DocumentsOnly.appendChild(ptext)
#        ReturnService.appendChild(DocumentsOnly)

        Shipper = doc2.createElement("Shipper")
        Shipment.appendChild(Shipper)

        Name = doc2.createElement("Name")
        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["Shipper"]["Name"])
        Name.appendChild(ptext)
        Shipper.appendChild(Name)

        AttentionName = doc2.createElement("AttentionName")
        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["Shipper"]["AttentionName"])
        AttentionName.appendChild(ptext)
        Shipper.appendChild(AttentionName)

        ShipperNumber = doc2.createElement("ShipperNumber")
        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["Shipper"]["ShipperNumber"])
        ShipperNumber.appendChild(ptext)
        Shipper.appendChild(ShipperNumber)

        TaxIdentificationNumber = doc2.createElement("TaxIdentificationNumber")
        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["Shipper"]["TaxIdentificationNumber"])
        TaxIdentificationNumber.appendChild(ptext)
        Shipper.appendChild(TaxIdentificationNumber)
        PhoneNumber = doc2.createElement("PhoneNumber")
        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["Shipper"]["PhoneNumber"])
        PhoneNumber.appendChild(ptext)
        Shipper.appendChild(PhoneNumber)

        FaxNumber = doc2.createElement("FaxNumber")
        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["Shipper"]["FaxNumber"])
        FaxNumber.appendChild(ptext)
        Shipper.appendChild(FaxNumber)

        EMailAddress = doc2.createElement("EMailAddress")
        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["Shipper"]["EMailAddress"])
        EMailAddress.appendChild(ptext)
        Shipper.appendChild(EMailAddress)

        #################address
        Address = doc2.createElement("Address")
        Shipper.appendChild(Address)

        AddressLine1 = doc2.createElement("AddressLine1")
        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["Shipper"]["Address"]["AddressLine1"])
        AddressLine1.appendChild(ptext)
        Address.appendChild(AddressLine1)

        AddressLine2 = doc2.createElement("AddressLine2")
        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["Shipper"]["Address"]["AddressLine2"])
        AddressLine2.appendChild(ptext)
        Address.appendChild(AddressLine2)

        AddressLine3 = doc2.createElement("AddressLine3")
        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["Shipper"]["Address"]["AddressLine3"])
        AddressLine3.appendChild(ptext)
        Address.appendChild(AddressLine3)

        City = doc2.createElement("City")
        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["Shipper"]["Address"]["City"])
        City.appendChild(ptext)
        Address.appendChild(City)

        StateProvinceCode = doc2.createElement("StateProvinceCode")
        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["Shipper"]["Address"]["StateProvinceCode"])
        StateProvinceCode.appendChild(ptext)
        Address.appendChild(StateProvinceCode)

        PostalCode = doc2.createElement("PostalCode")
        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["Shipper"]["Address"]["PostalCode"])
        PostalCode.appendChild(ptext)
        Address.appendChild(PostalCode)
        CountryCode = doc2.createElement("CountryCode")
        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["Shipper"]["Address"]["CountryCode"])
        CountryCode.appendChild(ptext)
        Address.appendChild(CountryCode)

        ShipTo = doc2.createElement("ShipTo")
        Shipment.appendChild(ShipTo)
        CompanyName = doc2.createElement("CompanyName")
        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipTo"]["CompanyName"])
        CompanyName.appendChild(ptext)
        ShipTo.appendChild(CompanyName)

        AttentionName = doc2.createElement("AttentionName")
        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipTo"]["AttentionName"])
        AttentionName.appendChild(ptext)
        ShipTo.appendChild(AttentionName)

        TaxIdentificationNumber = doc2.createElement("TaxIdentificationNumber")
        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipTo"]["TaxIdentificationNumber"])
        TaxIdentificationNumber.appendChild(ptext)
        ShipTo.appendChild(TaxIdentificationNumber)

        PhoneNumber = doc2.createElement("PhoneNumber")
        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipTo"]["PhoneNumber"])
        PhoneNumber.appendChild(ptext)
        ShipTo.appendChild(PhoneNumber)

        FaxNumber = doc2.createElement("FaxNumber")
        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipTo"]["FaxNumber"])
        FaxNumber.appendChild(ptext)
        ShipTo.appendChild(FaxNumber)

        EMailAddress = doc2.createElement("EMailAddress")
        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipTo"]["EMailAddress"])
        EMailAddress.appendChild(ptext)
        ShipTo.appendChild(EMailAddress)

        LocationID = doc2.createElement("LocationID")
        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipTo"]["LocationID"])
        LocationID.appendChild(ptext)
        ShipTo.appendChild(LocationID)

        Address = doc2.createElement("Address")
        ShipTo.appendChild(Address)
        AddressLine1 = doc2.createElement("AddressLine1")
        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipTo"]["Address"]["AddressLine1"])
        AddressLine1.appendChild(ptext)
        Address.appendChild(AddressLine1)

        AddressLine2 = doc2.createElement("AddressLine2")
        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipTo"]["Address"]["AddressLine2"])
        AddressLine2.appendChild(ptext)
        Address.appendChild(AddressLine2)

        AddressLine2 = doc2.createElement("AddressLine3")
        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipTo"]["Address"]["AddressLine3"])
        AddressLine3.appendChild(ptext)
        Address.appendChild(AddressLine3)

        City = doc2.createElement("City")
        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipTo"]["Address"]["City"])
        City.appendChild(ptext)
        Address.appendChild(City)

        StateProvinceCode = doc2.createElement("StateProvinceCode")
        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipTo"]["Address"]["StateProvinceCode"])
        StateProvinceCode.appendChild(ptext)
        Address.appendChild(StateProvinceCode)

        PostalCode = doc2.createElement("PostalCode")
        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipTo"]["Address"]["PostalCode"])
        PostalCode.appendChild(ptext)
        Address.appendChild(PostalCode)

        CountryCode = doc2.createElement("CountryCode")
        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipTo"]["Address"]["CountryCode"])
        CountryCode.appendChild(ptext)
        Address.appendChild(CountryCode)

        ShipFrom = doc2.createElement("ShipFrom")
        Shipment.appendChild(ShipFrom)
        CompanyName = doc2.createElement("CompanyName")
        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipFrom"]["CompanyName"])
        CompanyName.appendChild(ptext)
        ShipFrom.appendChild(CompanyName)

        AttentionName = doc2.createElement("AttentionName")
        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipFrom"]["AttentionName"])
        AttentionName.appendChild(ptext)
        ShipFrom.appendChild(AttentionName)

        TaxIdentificationNumber = doc2.createElement("TaxIdentificationNumber")
        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipFrom"]["TaxIdentificationNumber"])
        TaxIdentificationNumber.appendChild(ptext)
        ShipFrom.appendChild(TaxIdentificationNumber)

        PhoneNumber = doc2.createElement("PhoneNumber")
        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipFrom"]["PhoneNumber"])
        PhoneNumber.appendChild(ptext)
        ShipFrom.appendChild(PhoneNumber)

        FaxNumber = doc2.createElement("FaxNumber")
        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipFrom"]["FaxNumber"])
        FaxNumber.appendChild(ptext)
        ShipFrom.appendChild(FaxNumber)

        Address = doc2.createElement("Address")
        ShipFrom.appendChild(Address)
        AddressLine1 = doc2.createElement("AddressLine1")
        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipFrom"]["Address"]["AddressLine1"])
        AddressLine1.appendChild(ptext)
        Address.appendChild(AddressLine1)

        AddressLine2 = doc2.createElement("AddressLine2")
        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipFrom"]["Address"]["AddressLine2"])
        AddressLine2.appendChild(ptext)
        Address.appendChild(AddressLine2)

        AddressLine3 = doc2.createElement("AddressLine3")
        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipFrom"]["Address"]["AddressLine3"])
        AddressLine3.appendChild(ptext)
        Address.appendChild(AddressLine3)

        City = doc2.createElement("City")
        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipFrom"]["Address"]["City"])
        City.appendChild(ptext)
        Address.appendChild(City)

        StateProvinceCode = doc2.createElement("StateProvinceCode")
        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipFrom"]["Address"]["StateProvinceCode"])
        StateProvinceCode.appendChild(ptext)
        Address.appendChild(StateProvinceCode)

        PostalCode = doc2.createElement("PostalCode")
        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipFrom"]["Address"]["PostalCode"])
        PostalCode.appendChild(ptext)
        Address.appendChild(PostalCode)

        CountryCode = doc2.createElement("CountryCode")
        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipFrom"]["Address"]["CountryCode"])
        CountryCode.appendChild(ptext)
        Address.appendChild(CountryCode)



#            SoldTo = doc2.createElement("SoldTo")
#            Shipment.appendChild(SoldTo)
#            CompanyName = doc2.createElement("CompanyName")
#            ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["SoldTo"]["CompanyName"])
#            CompanyName.appendChild(ptext)
#            SoldTo.appendChild(CompanyName)
#
#            AttentionName = doc2.createElement("AttentionName")
#            ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["SoldTo"]["AttentionName"])
#            AttentionName.appendChild(ptext)
#            SoldTo.appendChild(AttentionName)
#
#            TaxIdentificationNumber = doc2.createElement("TaxIdentificationNumber")
#            ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["SoldTo"]["TaxIdentificationNumber"])
#            TaxIdentificationNumber.appendChild(ptext)
#            SoldTo.appendChild(TaxIdentificationNumber)
#
#            PhoneNumber = doc2.createElement("PhoneNumber")
#            ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["SoldTo"]["PhoneNumber"])
#            PhoneNumber.appendChild(ptext)
#            SoldTo.appendChild(PhoneNumber)
#
#            Option = doc2.createElement("Option")
#            ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["SoldTo"]["Option"])
#            Option.appendChild(ptext)
#            SoldTo.appendChild(Option)
#
#            Address = doc2.createElement("Address")
#            SoldTo.appendChild(Address)
#            AddressLine1 = doc2.createElement("AddressLine1")
#            ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["SoldTo"]["Address"]["AddressLine1"])
#            AddressLine1.appendChild(ptext)
#            Address.appendChild(AddressLine1)
#
#            AddressLine2 = doc2.createElement("AddressLine2")
#            ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["SoldTo"]["Address"]["AddressLine2"])
#            AddressLine2.appendChild(ptext)
#            Address.appendChild(AddressLine2)
#
#            AddressLine3 = doc2.createElement("AddressLine3")
#            ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["SoldTo"]["Address"]["AddressLine3"])
#            AddressLine3.appendChild(ptext)
#            Address.appendChild(AddressLine3)
#
#            City = doc2.createElement("City")
#            ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["SoldTo"]["Address"]["City"])
#            City.appendChild(ptext)
#            Address.appendChild(City)
#
#            StateProvinceCode = doc2.createElement("StateProvinceCode")
#            ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["SoldTo"]["Address"]["StateProvinceCode"])
#            StateProvinceCode.appendChild(ptext)
#            Address.appendChild(StateProvinceCode)
#
#            PostalCode = doc2.createElement("PostalCode")
#            ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["SoldTo"]["Address"]["PostalCode"])
#            PostalCode.appendChild(ptext)
#            Address.appendChild(PostalCode)
#
#            CountryCode = doc2.createElement("CountryCode")
#            ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["SoldTo"]["Address"]["CountryCode"])
#            CountryCode.appendChild(ptext)
#            Address.appendChild(CountryCode)


        PaymentInformation = doc2.createElement("PaymentInformation")
        Shipment.appendChild(PaymentInformation)
        Prepaid = doc2.createElement("Prepaid")
        PaymentInformation.appendChild(Prepaid)

        BillShipper = doc2.createElement("BillShipper")
        Prepaid.appendChild(BillShipper)

        AccountNumber = doc2.createElement("AccountNumber")
        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["PaymentInformation"]["Prepaid"]["BillShipper"]["AccountNumber"])
        AccountNumber.appendChild(ptext)
        BillShipper.appendChild(AccountNumber)

#        CreditCard = doc2.createElement("CreditCard")
#        BillShipper.appendChild(CreditCard)
#        Type = doc2.createElement("Type")
#        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["PaymentInformation"]["Prepaid"]["BillShipper"]["CreditCard"]["Type"])
#        Type.appendChild(ptext)
#        CreditCard.appendChild(Type)
#
#        ExpirationDate = doc2.createElement("ExpirationDate")
#        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["PaymentInformation"]["Prepaid"]["BillShipper"]["CreditCard"]["ExpirationDate"])
#        ExpirationDate.appendChild(ptext)
#        CreditCard.appendChild(ExpirationDate)
#
#        Number = doc2.createElement("Number")
#        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["PaymentInformation"]["Prepaid"]["BillShipper"]["CreditCard"]["Number"])
#        Number.appendChild(ptext)
#        CreditCard.appendChild(Number)
#
#        SecurityCode = doc2.createElement("SecurityCode")
#        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["PaymentInformation"]["Prepaid"]["BillShipper"]["CreditCard"]["SecurityCode"])
#        SecurityCode.appendChild(ptext)
#        CreditCard.appendChild(SecurityCode)
#
#
#        Address = doc2.createElement("Address")
#        CreditCard.appendChild(Address)
#        AddressLine1 = doc2.createElement("AddressLine1")
#        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["PaymentInformation"]["Prepaid"]["BillShipper"]["CreditCard"]["Address"]["AddressLine1"])
#        AddressLine1.appendChild(ptext)
#        Address.appendChild(AddressLine1)
#
#        AddressLine2 = doc2.createElement("AddressLine2")
#        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["PaymentInformation"]["Prepaid"]["BillShipper"]["CreditCard"]["Address"]["AddressLine2"])
#        AddressLine2.appendChild(ptext)
#        Address.appendChild(AddressLine2)
#
#        AddressLine3 = doc2.createElement("AddressLine3")
#        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["PaymentInformation"]["Prepaid"]["BillShipper"]["CreditCard"]["Address"]["AddressLine3"])
#        AddressLine3.appendChild(ptext)
#        Address.appendChild(AddressLine3)
#
#
#        City = doc2.createElement("City")
#        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["PaymentInformation"]["Prepaid"]["BillShipper"]["CreditCard"]["Address"]["City"])
#        City.appendChild(ptext)
#        Address.appendChild(City)
#
#        StateProvinceCode = doc2.createElement("StateProvinceCode")
#        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["PaymentInformation"]["Prepaid"]["BillShipper"]["CreditCard"]["Address"]["StateProvinceCode"])
#        StateProvinceCode.appendChild(ptext)
#        Address.appendChild(StateProvinceCode)
#
#        PostalCode = doc2.createElement("PostalCode")
#        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["PaymentInformation"]["Prepaid"]["BillShipper"]["CreditCard"]["Address"]["PostalCode"])
#        PostalCode.appendChild(ptext)
#        Address.appendChild(PostalCode)
#
#        CountryCode = doc2.createElement("CountryCode")
#        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["PaymentInformation"]["Prepaid"]["BillShipper"]["CreditCard"]["Address"]["CountryCode"])
#        PostalCode.appendChild(ptext)
#        Address.appendChild(CountryCode)




#        BillThirdParty = doc2.createElement("BillThirdParty")
#        PaymentInformation.appendChild(BillThirdParty)
#        BillThirdPartyShipper = doc2.createElement("BillThirdPartyShipper")
#        BillThirdParty.appendChild(BillThirdPartyShipper)
#
#        AccountNumber = doc2.createElement("AccountNumber")
#        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["PaymentInformation"]["BillThirdParty"]['BillThirdPartyShipper']['AccountNumber'])
#        AccountNumber.appendChild(ptext)
#        BillThirdPartyShipper.appendChild(AccountNumber)
#
#        ThirdParty = doc2.createElement("ThirdParty")
#        BillThirdPartyShipper.appendChild(ThirdParty)
#        Address = doc2.createElement("Address")
#        ThirdParty.appendChild(Address)
#
#        PostalCode = doc2.createElement("PostalCode")
#        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["PaymentInformation"]["BillThirdParty"]['BillThirdPartyShipper']['ThirdParty']['Address']['PostalCode'])
#        PostalCode.appendChild(ptext)
#        Address.appendChild(PostalCode)
#
#        CountryCode = doc2.createElement("CountryCode")
#        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["PaymentInformation"]["BillThirdParty"]['BillThirdPartyShipper']['ThirdParty']['Address']['CountryCode'])
#        CountryCode.appendChild(ptext)
#        Address.appendChild(CountryCode)



#        FreightCollect = doc2.createElement("FreightCollect")
#        PaymentInformation.appendChild(FreightCollect)
#        BillReceiver = doc2.createElement("BillReceiver")
#        FreightCollect.appendChild(BillReceiver)
#        AccountNumber = doc2.createElement("AccountNumber")
#        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["PaymentInformation"]["FreightCollect"]["BillReceiver"]["AccountNumber"])
#        AccountNumber.appendChild(ptext)
#        BillReceiver.appendChild(AccountNumber)
#
#        Address = doc2.createElement("Address")
#        BillReceiver.appendChild(Address)
#        PostalCode = doc2.createElement("PostalCode")
#        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["PaymentInformation"]["FreightCollect"]["BillReceiver"]["Address"]["PostalCode"])
#        AccountNumber.appendChild(ptext)
#        Address.appendChild(PostalCode)

#        ConsigneeBilled = doc2.createElement("ConsigneeBilled")
#        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["PaymentInformation"]["ConsigneeBilled"])
#        ConsigneeBilled.appendChild(ptext)
#        PaymentInformation.appendChild(ConsigneeBilled)




#        ItemizedPaymentInformation = doc2.createElement("ItemizedPaymentInformation")
#        Shipment.appendChild(ItemizedPaymentInformation)
#        ShipmentCharge = doc2.createElement("ShipmentCharge")
#        ItemizedPaymentInformation.appendChild(ShipmentCharge)
#        Type = doc2.createElement("Type")
#        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ItemizedPaymentInformation"]["ShipmentCharge"]["Type"])
#        Type.appendChild(ptext)
#        ShipmentCharge.appendChild(Type)
#
#        BillShipper = doc2.createElement("BillShipper")
#        ShipmentCharge.appendChild(BillShipper)
#
#        AccountNumber = doc2.createElement("AccountNumber")
#        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ItemizedPaymentInformation"]["ShipmentCharge"]["BillShipper"]["AccountNumber"])
#        AccountNumber.appendChild(ptext)
#        BillShipper.appendChild(AccountNumber)
#
#        CreditCard = doc2.createElement("CreditCard")
#        BillShipper.appendChild(CreditCard)
#
#        Type = doc2.createElement("Type")
#        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ItemizedPaymentInformation"]["ShipmentCharge"]["BillShipper"]["CreditCard"]["Type"])
#        Type.appendChild(ptext)
#        CreditCard.appendChild(Type)
#
#        ExpirationDate = doc2.createElement("ExpirationDate")
#        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ItemizedPaymentInformation"]["ShipmentCharge"]["BillShipper"]["CreditCard"]["ExpirationDate"])
#        ExpirationDate.appendChild(ptext)
#        CreditCard.appendChild(ExpirationDate)
#
#        SecurityCode = doc2.createElement("SecurityCode")
#        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ItemizedPaymentInformation"]["ShipmentCharge"]["BillShipper"]["CreditCard"]["SecurityCode"])
#        SecurityCode.appendChild(ptext)
#        CreditCard.appendChild(SecurityCode)
#
#        Address = doc2.createElement("Address")
#        CreditCard.appendChild(Address)
#        AddressLine1 = doc2.createElement("AddressLine1")
#        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ItemizedPaymentInformation"]["ShipmentCharge"]["BillShipper"]['CreditCard']["Address"]["AddressLine1"])
#        AddressLine1.appendChild(ptext)
#        Address.appendChild(AddressLine1)
#
#        AddressLine2 = doc2.createElement("AddressLine2")
#        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ItemizedPaymentInformation"]["ShipmentCharge"]["BillShipper"]['CreditCard']["Address"]["AddressLine2"])
#        AddressLine2.appendChild(ptext)
#        Address.appendChild(AddressLine2)
#
#        AddressLine3 = doc2.createElement("AddressLine3")
#        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ItemizedPaymentInformation"]["ShipmentCharge"]["BillShipper"]['CreditCard']["Address"]["AddressLine3"])
#        AddressLine3.appendChild(ptext)
#        Address.appendChild(AddressLine3)
#
#        City = doc2.createElement("City")
#        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ItemizedPaymentInformation"]["ShipmentCharge"]["BillShipper"]['CreditCard']["Address"]["City"])
#        City.appendChild(ptext)
#        Address.appendChild(City)
#
#        StateProvinceCode = doc2.createElement("StateProvinceCode")
#        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ItemizedPaymentInformation"]["ShipmentCharge"]["BillShipper"]['CreditCard']["Address"]["StateProvinceCode"])
#        StateProvinceCode.appendChild(ptext)
#        Address.appendChild(StateProvinceCode)
#
#        PostalCode = doc2.createElement("PostalCode")
#        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ItemizedPaymentInformation"]["ShipmentCharge"]["BillShipper"]['CreditCard']["Address"]["PostalCode"])
#        PostalCode.appendChild(ptext)
#        Address.appendChild(PostalCode)
#
#        CountryCode = doc2.createElement("CountryCode")
#        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ItemizedPaymentInformation"]["ShipmentCharge"]["BillShipper"]['CreditCard']["Address"]["CountryCode"])
#        CountryCode.appendChild(ptext)
#        Address.appendChild(CountryCode)
#
#        BillReceiver = doc2.createElement("BillReceiver")
#        ShipmentCharge.appendChild(BillReceiver)
#
#        AccountNumber = doc2.createElement("AccountNumber")
#        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ItemizedPaymentInformation"]["ShipmentCharge"]["BillReceiver"]["AccountNumber"])
#        AccountNumber.appendChild(ptext)
#        BillReceiver.appendChild(AccountNumber)
#
#        Address = doc2.createElement("Address")
#        BillReceiver.appendChild(Address)
#        PostalCode = doc2.createElement("PostalCode")
#        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ItemizedPaymentInformation"]["ShipmentCharge"]["BillReceiver"]["Address"]["PostalCode"])
#        PostalCode.appendChild(ptext)
#        Address.appendChild(PostalCode)
#
#
#        BillThirdParty = doc2.createElement("BillThirdParty")
#        ShipmentCharge.appendChild(BillThirdParty)
#        BillThirdPartyShipper = doc2.createElement("BillThirdPartyShipper")
#        BillThirdParty.appendChild(BillThirdPartyShipper)
#
#        AccountNumber = doc2.createElement("AccountNumber")
#        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ItemizedPaymentInformation"]["ShipmentCharge"]["BillThirdParty"]["BillThirdPartyShipper"]["AccountNumber"])
#        AccountNumber.appendChild(ptext)
#        BillThirdPartyShipper.appendChild(AccountNumber)
#
#        ThirdParty = doc2.createElement("ThirdParty")
#        BillThirdPartyShipper.appendChild(ThirdParty)
#        Address = doc2.createElement("Address")
#        ThirdParty.appendChild(Address)
#        PostalCode = doc2.createElement("PostalCode")
#        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ItemizedPaymentInformation"]["ShipmentCharge"]["BillThirdParty"]["BillThirdPartyShipper"]["ThirdParty"]["Address"]["PostalCode"])
#        PostalCode.appendChild(ptext)
#        Address.appendChild(PostalCode)
#
#        CountryCode = doc2.createElement("CountryCode")
#        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ItemizedPaymentInformation"]["ShipmentCharge"]["BillThirdParty"]["BillThirdPartyShipper"]["ThirdParty"]["Address"]["CountryCode"])
#        CountryCode.appendChild(ptext)
#        Address.appendChild(CountryCode)
#
#
#        BillThirdPartyConsignee = doc2.createElement("BillThirdPartyConsignee")
#        BillThirdParty.appendChild(BillThirdPartyConsignee)
#
#        AccountNumber = doc2.createElement("AccountNumber")
#        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ItemizedPaymentInformation"]["ShipmentCharge"]["BillThirdParty"]["BillThirdPartyConsignee"]["AccountNumber"])
#        AccountNumber.appendChild(ptext)
#        BillThirdPartyConsignee.appendChild(AccountNumber)
#
#        ThirdParty = doc2.createElement("ThirdParty")
#        BillThirdPartyConsignee.appendChild(ThirdParty)
#        Address = doc2.createElement("Address")
#        ThirdParty.appendChild(Address)
#        PostalCode = doc2.createElement("PostalCode")
#        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ItemizedPaymentInformation"]["ShipmentCharge"]["BillThirdParty"]["BillThirdPartyConsignee"]["ThirdParty"]["Address"]["PostalCode"])
#        PostalCode.appendChild(ptext)
#        Address.appendChild(PostalCode)
#
#        CountryCode = doc2.createElement("CountryCode")
#        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ItemizedPaymentInformation"]["ShipmentCharge"]["BillThirdParty"]["BillThirdPartyConsignee"]["ThirdParty"]["Address"]["CountryCode"])
#        CountryCode.appendChild(ptext)
#        Address.appendChild(CountryCode)
#
#        ConsigneeBilled = doc2.createElement("ConsigneeBilled")
#        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ItemizedPaymentInformation"]["ShipmentCharge"]["ConsigneeBilled"])
#        ConsigneeBilled.appendChild(ptext)
#        ShipmentCharge.appendChild(ConsigneeBilled)
#
#        SplitDutyVATIndicator = doc2.createElement("SplitDutyVATIndicator")
#        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ItemizedPaymentInformation"]["SplitDutyVATIndicator"])
#        SplitDutyVATIndicator.appendChild(ptext)
#        ItemizedPaymentInformation.appendChild(SplitDutyVATIndicator)

        GoodsNotInFreeCirculationIndicator = doc2.createElement("GoodsNotInFreeCirculationIndicator")
#        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["GoodsNotInFreeCirculationIndicator"])
#        GoodsNotInFreeCirculationIndicator.appendChild(ptext)
        Shipment.appendChild(GoodsNotInFreeCirculationIndicator)

        RateInformation = doc2.createElement("RateInformation")
        Shipment.appendChild(RateInformation)
        NegotiatedRatesIndicator = doc2.createElement("NegotiatedRatesIndicator")
#        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["RateInformation"]["NegotiatedRatesIndicator"])
#        NegotiatedRatesIndicator.appendChild(ptext)
        RateInformation.appendChild(NegotiatedRatesIndicator)

#        MovementReferenceNumber = doc2.createElement("MovementReferenceNumber")
#        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["MovementReferenceNumber"])
#        MovementReferenceNumber.appendChild(ptext)
#        Shipment.appendChild(MovementReferenceNumber)

#        ReferenceNumber = doc2.createElement("ReferenceNumber")
#        Shipment.appendChild(ReferenceNumber)
#        BarCodeIndicator = doc2.createElement("BarCodeIndicator")
#        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ReferenceNumber"]["BarCodeIndicator"])
#        BarCodeIndicator.appendChild(ptext)
#        ReferenceNumber.appendChild(BarCodeIndicator)
#
#        Code = doc2.createElement("Code")
#        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ReferenceNumber"]["Code"])
#        Code.appendChild(ptext)
#        ReferenceNumber.appendChild(Code)
#
#        Value = doc2.createElement("Value")
#        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ReferenceNumber"]["Value"])
#        Value.appendChild(ptext)
#        ReferenceNumber.appendChild(Value)

        Service = doc2.createElement("Service")
        Shipment.appendChild(Service)
        Code = doc2.createElement("Code")
        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["Service"]["Code"])
        Code.appendChild(ptext)
        Service.appendChild(Code)
        Description = doc2.createElement("Description")
        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["Service"]["Description"])
        Description.appendChild(ptext)
        Service.appendChild(Description)

        if (quick_ship_id.send_addr.id and quick_ship_id.send_addr.country_id.code == 'US') and  quick_ship_id.address_id.id and quick_ship_id.address_id.country_id.code in ('PR','CA'):
            InvoiceLineTotal = doc2.createElement("InvoiceLineTotal")
            Shipment.appendChild(InvoiceLineTotal)

#            CurrencyCode = doc2.createElement("CurrencyCode")
#            ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["InvoiceLineTotal"]["CurrencyCode"])
#            CurrencyCode.appendChild(ptext)
#            InvoiceLineTotal.appendChild(CurrencyCode)

            MonetaryValue = doc2.createElement("MonetaryValue")
            ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["InvoiceLineTotal"]["MonetaryValue"])
            MonetaryValue.appendChild(ptext)
            InvoiceLineTotal.appendChild(MonetaryValue)


        ShipmentServiceOptions = doc2.createElement("ShipmentServiceOptions")
        Shipment.appendChild(ShipmentServiceOptions)
        if quick_ship_id.sat_delivery:
            SaturdayDelivery = doc2.createElement("SaturdayDelivery")
    #        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["SaturdayDelivery"])
    #        SaturdayDelivery.appendChild(ptext)
            ShipmentServiceOptions.appendChild(SaturdayDelivery)

        if quick_ship_id.cod:
            COD = doc2.createElement("COD")
            ShipmentServiceOptions.appendChild(COD)
            CODCode = doc2.createElement("CODCode")
            ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["COD"]["CODCode"])
            CODCode.appendChild(ptext)
            COD.appendChild(CODCode)

            CODFundsCode = doc2.createElement("CODFundsCode")
            ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["COD"]["CODFundsCode"])
            CODFundsCode.appendChild(ptext)
            COD.appendChild(CODFundsCode)

            CODAmount = doc2.createElement("CODAmount")
            COD.appendChild(CODAmount)
            CurrencyCode = doc2.createElement("CurrencyCode")
            ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["COD"]["CODAmount"]["CurrencyCode"])
            CurrencyCode.appendChild(ptext)
            CODAmount.appendChild(CurrencyCode)

            MonetaryValue = doc2.createElement("MonetaryValue")
            ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["COD"]["CODAmount"]["MonetaryValue"])
            MonetaryValue.appendChild(ptext)
            CODAmount.appendChild(MonetaryValue)

        Notification_count = len(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["Notification"])
        for i in range (0,Notification_count):
            Notification = doc2.createElement("Notification")
            ShipmentServiceOptions.appendChild(Notification)
            NotificationCode = doc2.createElement("NotificationCode")
            ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["Notification"][i]["NotificationCode"])
            NotificationCode.appendChild(ptext)
            Notification.appendChild(NotificationCode)
            EMailMessage = doc2.createElement("EMailMessage")
            Notification.appendChild(EMailMessage)
            UndeliverableEMailAddress = doc2.createElement("UndeliverableEMailAddress")
            ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["Notification"][i]["EMailMessage"]["UndeliverableEMailAddress"])
            UndeliverableEMailAddress.appendChild(ptext)
            EMailMessage.appendChild(UndeliverableEMailAddress)

            EMailAddress = doc2.createElement("EMailAddress")
            ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["Notification"][i]["EMailMessage"]["EMailAddress"])
            EMailAddress.appendChild(ptext)
            EMailMessage.appendChild(EMailAddress)

            FromEMailAddress = doc2.createElement("FromEMailAddress")
            ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["Notification"][i]["EMailMessage"]["FromEMailAddress"])
            FromEMailAddress.appendChild(ptext)
            EMailMessage.appendChild(FromEMailAddress)

            FromName = doc2.createElement("FromName")
            ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["Notification"][i]["EMailMessage"]["FromName"])
            FromName.appendChild(ptext)
            EMailMessage.appendChild(FromName)

            Memo = doc2.createElement("Memo")
            ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["Notification"][i]["EMailMessage"]["Memo"])
            Memo.appendChild(ptext)
            EMailMessage.appendChild(Memo)

            Subject = doc2.createElement("Subject")
            ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["Notification"][i]["EMailMessage"]["Subject"])
            Subject.appendChild(ptext)
            EMailMessage.appendChild(Subject)

            SubjectCode = doc2.createElement("SubjectCode")
            ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["Notification"][i]["EMailMessage"]["SubjectCode"])
            SubjectCode.appendChild(ptext)
            EMailMessage.appendChild(SubjectCode)
        ##//////////////////////end for loop in code
        if 0:#deliv_order.with_ret_service:
            LabelDelivery = doc2.createElement("LabelDelivery")
            ShipmentServiceOptions.appendChild(LabelDelivery)
            LabelLinkIndicator = doc2.createElement("LabelLinkIndicator")
    #        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["LabelDelivery"]["LabelLinkIndicator"])
    #        LabelLinkIndicator.appendChild(ptext)
            LabelDelivery.appendChild(LabelLinkIndicator)
#        EMailMessage = doc2.createElement("EMailMessage")
#        LabelDelivery.appendChild(EMailMessage)
#        UndeliverableEMailAddress = doc2.createElement("UndeliverableEMailAddress")
#        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["LabelDelivery"]["EMailMessage"]["UndeliverableEMailAddress"])
#        UndeliverableEMailAddress.appendChild(ptext)
#        EMailMessage.appendChild(UndeliverableEMailAddress)
#
#
#        EMailAddress = doc2.createElement("EMailAddress")
#        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["LabelDelivery"]["EMailMessage"]["EMailAddress"])
#        EMailAddress.appendChild(ptext)
#        EMailMessage.appendChild(EMailAddress)
#
#        FromEMailAddress = doc2.createElement("FromEMailAddress")
#        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["LabelDelivery"]["EMailMessage"]["FromEMailAddress"])
#        FromEMailAddress.appendChild(ptext)
#        EMailMessage.appendChild(FromEMailAddress)
#
#        FromName = doc2.createElement("FromName")
#        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["LabelDelivery"]["EMailMessage"]["FromName"])
#        FromName.appendChild(ptext)
#        EMailMessage.appendChild(FromName)
#
#        Memo = doc2.createElement("Memo")
#        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["LabelDelivery"]["EMailMessage"]["Memo"])
#        Memo.appendChild(ptext)
#        EMailMessage.appendChild(Memo)
#
#        Subject = doc2.createElement("Subject")
#        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["LabelDelivery"]["EMailMessage"]["Subject"])
#        Subject.appendChild(ptext)
#        EMailMessage.appendChild(Subject)
#
#        SubjectCode = doc2.createElement("SubjectCode")
#        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["LabelDelivery"]["EMailMessage"]["SubjectCode"])
#        SubjectCode.appendChild(ptext)
#        EMailMessage.appendChild(SubjectCode)

#        InternationalForms = doc2.createElement("InternationalForms")
#        ShipmentServiceOptions.appendChild(InternationalForms)
#        FormType = doc2.createElement("FormType")
#        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["InternationalForms"]["FormType"])
#        FormType.appendChild(ptext)
#        InternationalForms.appendChild(FormType)
#
#        AdditionalDocumentIndicator = doc2.createElement("AdditionalDocumentIndicator")
#        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["InternationalForms"]["AdditionalDocumentIndicator"])
#        AdditionalDocumentIndicator.appendChild(ptext)
#        InternationalForms.appendChild(AdditionalDocumentIndicator)
#
#        FormGroupIdName = doc2.createElement("FormGroupIdName")
#        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["InternationalForms"]["FormGroupIdName"])
#        FormGroupIdName.appendChild(ptext)
#        InternationalForms.appendChild(FormGroupIdName)
#
#        Contacts = doc2.createElement("Contacts")
#        InternationalForms.appendChild(Contacts)
#        ForwardAgent = doc2.createElement("ForwardAgent")
#        Contacts.appendChild(ForwardAgent)
#
#
#        CompanyName = doc2.createElement("CompanyName")
#        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["InternationalForms"]["Contacts"]['ForwardAgent']['CompanyName'])
#        CompanyName.appendChild(ptext)
#        ForwardAgent.appendChild(CompanyName)
#
#        TaxIdentificationNumber = doc2.createElement("TaxIdentificationNumber")
#        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["InternationalForms"]["Contacts"]['ForwardAgent']['TaxIdentificationNumber'])
#        TaxIdentificationNumber.appendChild(ptext)
#        ForwardAgent.appendChild(TaxIdentificationNumber)
#
#        Address = doc2.createElement("Address")
#        ForwardAgent.appendChild(Address)
#
#        AddressLine1 = doc2.createElement("AddressLine1")
#        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["InternationalForms"]["Contacts"]['ForwardAgent']['Address']['AddressLine1'])
#        AddressLine1.appendChild(ptext)
#        Address.appendChild(AddressLine1)
#
#        AddressLine2 = doc2.createElement("AddressLine2")
#        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["InternationalForms"]["Contacts"]['ForwardAgent']['Address']['AddressLine2'])
#        AddressLine2.appendChild(ptext)
#        Address.appendChild(AddressLine2)
#
#        #AddressLine3 = doc2.createElement("AddressLine3")
#        #ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["InternationalForms"]["Contacts"]['ForwardAgent']['Address']['AddressLine3'])
#        #AddressLine3.appendChild(ptext)
#        #Address.appendChild(AddressLine3)
#
#        City = doc2.createElement("City")
#        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["InternationalForms"]["Contacts"]['ForwardAgent']['Address']['City'])
#        City.appendChild(ptext)
#        Address.appendChild(City)
#
#        StateProvinceCode = doc2.createElement("StateProvinceCode")
#        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["InternationalForms"]["Contacts"]['ForwardAgent']['Address']['StateProvinceCode'])
#        StateProvinceCode.appendChild(ptext)
#        Address.appendChild(StateProvinceCode)
#
#        PostalCode = doc2.createElement("PostalCode")
#        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["InternationalForms"]["Contacts"]['ForwardAgent']['Address']['PostalCode'])
#        PostalCode.appendChild(ptext)
#        Address.appendChild(PostalCode)
#
#        CountryCode = doc2.createElement("CountryCode")
#        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["InternationalForms"]["Contacts"]['ForwardAgent']['Address']['CountryCode'])
#        CountryCode.appendChild(ptext)
#        Address.appendChild(CountryCode)
#
#
#
#        UltimateConsignee = doc2.createElement("UltimateConsignee")
#        Contacts.appendChild(UltimateConsignee)
#
#
#        CompanyName = doc2.createElement("CompanyName")
#        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["InternationalForms"]["Contacts"]['UltimateConsignee']['CompanyName'])
#        CompanyName.appendChild(ptext)
#        UltimateConsignee.appendChild(CompanyName)
#
#
#
#        Address = doc2.createElement("Address")
#        UltimateConsignee.appendChild(Address)
#
#        AddressLine1 = doc2.createElement("AddressLine1")
#        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["InternationalForms"]["Contacts"]['UltimateConsignee']['Address']['AddressLine1'])
#        AddressLine1.appendChild(ptext)
#        Address.appendChild(AddressLine1)
#
#        AddressLine2 = doc2.createElement("AddressLine2")
#        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["InternationalForms"]["Contacts"]['UltimateConsignee']['Address']['AddressLine2'])
#        AddressLine2.appendChild(ptext)
#        Address.appendChild(AddressLine2)
#
#        #AddressLine3 = doc2.createElement("AddressLine3")
#        #ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["InternationalForms"]["Contacts"]['UltimateConsignee']['Address']['AddressLine3'])
#        #AddressLine3.appendChild(ptext)
#        #Address.appendChild(AddressLine3)
#
#        City = doc2.createElement("City")
#        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["InternationalForms"]["Contacts"]['UltimateConsignee']['Address']['City'])
#        City.appendChild(ptext)
#        Address.appendChild(City)
#
#        StateProvinceCode = doc2.createElement("StateProvinceCode")
#        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["InternationalForms"]["Contacts"]['UltimateConsignee']['Address']['StateProvinceCode'])
#        StateProvinceCode.appendChild(ptext)
#        Address.appendChild(StateProvinceCode)
#
#        PostalCode = doc2.createElement("PostalCode")
#        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["InternationalForms"]["Contacts"]['UltimateConsignee']['Address']['PostalCode'])
#        PostalCode.appendChild(ptext)
#        Address.appendChild(PostalCode)
#
#        CountryCode = doc2.createElement("CountryCode")
#        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["InternationalForms"]["Contacts"]['UltimateConsignee']['Address']['CountryCode'])
#        CountryCode.appendChild(ptext)
#        Address.appendChild(CountryCode)
#
#
#
#
#        IntermediateConsignee = doc2.createElement("IntermediateConsignee")
#        Contacts.appendChild(IntermediateConsignee)
#
#
#        CompanyName = doc2.createElement("CompanyName")
#        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["InternationalForms"]["Contacts"]['IntermediateConsignee']['CompanyName'])
#        CompanyName.appendChild(ptext)
#        IntermediateConsignee.appendChild(CompanyName)
#
#
#
#        Address = doc2.createElement("Address")
#        IntermediateConsignee.appendChild(Address)
#
#        AddressLine1 = doc2.createElement("AddressLine1")
#        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["InternationalForms"]["Contacts"]['IntermediateConsignee']['Address']['AddressLine1'])
#        AddressLine1.appendChild(ptext)
#        Address.appendChild(AddressLine1)
#
#        AddressLine2 = doc2.createElement("AddressLine2")
#        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["InternationalForms"]["Contacts"]['IntermediateConsignee']['Address']['AddressLine2'])
#        AddressLine2.appendChild(ptext)
#        Address.appendChild(AddressLine2)
#
#        #AddressLine3 = doc2.createElement("AddressLine3")
#        #ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["InternationalForms"]["Contacts"]['IntermediateConsignee']['Address']['AddressLine3'])
#        #AddressLine3.appendChild(ptext)
#        #Address.appendChild(AddressLine3)
#
#        City = doc2.createElement("City")
#        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["InternationalForms"]["Contacts"]['IntermediateConsignee']['Address']['City'])
#        City.appendChild(ptext)
#        Address.appendChild(City)
#
#        StateProvinceCode = doc2.createElement("StateProvinceCode")
#        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["InternationalForms"]["Contacts"]['IntermediateConsignee']['Address']['StateProvinceCode'])
#        StateProvinceCode.appendChild(ptext)
#        Address.appendChild(StateProvinceCode)
#
#        PostalCode = doc2.createElement("PostalCode")
#        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["InternationalForms"]["Contacts"]['IntermediateConsignee']['Address']['PostalCode'])
#        PostalCode.appendChild(ptext)
#        Address.appendChild(PostalCode)
#
#        CountryCode = doc2.createElement("CountryCode")
#        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["InternationalForms"]["Contacts"]['IntermediateConsignee']['Address']['CountryCode'])
#        CountryCode.appendChild(ptext)
#        Address.appendChild(CountryCode)
#
#        Producer = doc2.createElement("Producer")
#        Contacts.appendChild(Producer)
#        Option = doc2.createElement("Option")
#        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["InternationalForms"]["Contacts"]['Producer']['Option'])
#        Option.appendChild(ptext)
#        Producer.appendChild(Option)
#
#
#        CompanyName = doc2.createElement("CompanyName")
#        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["InternationalForms"]["Contacts"]['Producer']['CompanyName'])
#        CompanyName.appendChild(ptext)
#        Producer.appendChild(CompanyName)
#
#        TaxIdentificationNumber = doc2.createElement("TaxIdentificationNumber")
#        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["InternationalForms"]["Contacts"]['Producer']['TaxIdentificationNumber'])
#        TaxIdentificationNumber.appendChild(ptext)
#        Producer.appendChild(TaxIdentificationNumber)
#
#        Address = doc2.createElement("Address")
#        Producer.appendChild(Address)
#
#        AddressLine1 = doc2.createElement("AddressLine1")
#        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["InternationalForms"]["Contacts"]['Producer']['Address']['AddressLine1'])
#        AddressLine1.appendChild(ptext)
#        Address.appendChild(AddressLine1)
#
#        AddressLine2 = doc2.createElement("AddressLine2")
#        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["InternationalForms"]["Contacts"]['Producer']['Address']['AddressLine2'])
#        AddressLine2.appendChild(ptext)
#        Address.appendChild(AddressLine2)
#
#        #AddressLine3 = doc2.createElement("AddressLine3")
#        #ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["InternationalForms"]["Contacts"]['Producer']['Address']['AddressLine3'])
#        #AddressLine3.appendChild(ptext)
#        #Address.appendChild(AddressLine3)
#
#        City = doc2.createElement("City")
#        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["InternationalForms"]["Contacts"]['Producer']['Address']['City'])
#        City.appendChild(ptext)
#        Address.appendChild(City)
#
#        StateProvinceCode = doc2.createElement("StateProvinceCode")
#        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["InternationalForms"]["Contacts"]['Producer']['Address']['StateProvinceCode'])
#        StateProvinceCode.appendChild(ptext)
#        Address.appendChild(StateProvinceCode)
#
#        PostalCode = doc2.createElement("PostalCode")
#        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["InternationalForms"]["Contacts"]['Producer']['Address']['PostalCode'])
#        PostalCode.appendChild(ptext)
#        Address.appendChild(PostalCode)
#
#        CountryCode = doc2.createElement("CountryCode")
#        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["InternationalForms"]["Contacts"]['Producer']['Address']['CountryCode'])
#        CountryCode.appendChild(ptext)
#        Address.appendChild(CountryCode)
#        Product_count = len(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["InternationalForms"]["Product"])
#        for i in range (0,Product_count):
#            #///////////
#            #########code for product dictionary
#            #///////////
#
#            Product = doc2.createElement("Product")
#            InternationalForms.appendChild(Product)
#
#            Description = doc2.createElement("Description")
#            ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["InternationalForms"]["Product"][i]['Description'])
#            Product.appendChild(ptext)
#            Product.appendChild(Description)
#
#            Unit = doc2.createElement("Unit")
#            Product.appendChild(Unit)
#            Number = doc2.createElement("Number")
#            ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["InternationalForms"]["Product"][i]['Unit']['Number'])
#            Number.appendChild(ptext)
#            Unit.appendChild(Number)
#
#            Value = doc2.createElement("Value")
#            ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["InternationalForms"]["Product"][i]['Unit']['Value'])
#            Value.appendChild(ptext)
#            Unit.appendChild(Value)
#
#            UnitOfMeasurement = doc2.createElement("UnitOfMeasurement")
#            Unit.appendChild(UnitOfMeasurement)
#            Code = doc2.createElement("Code")
#            ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["InternationalForms"]["Product"][i]['Unit']['UnitOfMeasurement']['Code'])
#            Code.appendChild(ptext)
#            UnitOfMeasurement.appendChild(Code)
#
#            Description = doc2.createElement("Description")
#            ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["InternationalForms"]["Product"][i]['Unit']['UnitOfMeasurement']['Description'])
#            Description.appendChild(ptext)
#            UnitOfMeasurement.appendChild(Description)
#
#            CommodityCode = doc2.createElement("CommodityCode")
#            ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["InternationalForms"]["Product"][i]['CommodityCode'])
#            CommodityCode.appendChild(ptext)
#            Product.appendChild(CommodityCode)
#
#            PartNumber = doc2.createElement("PartNumber")
#            ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["InternationalForms"]["Product"][i]['PartNumber'])
#            CommodityCode.appendChild(ptext)
#            Product.appendChild(PartNumber)
#
#            OriginCountryCode = doc2.createElement("OriginCountryCode")
#            ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["InternationalForms"]["Product"][i]['OriginCountryCode'])
#            OriginCountryCode.appendChild(ptext)
#            Product.appendChild(OriginCountryCode)
#
#            JointProductionIndicator = doc2.createElement("JointProductionIndicator")
#            ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["InternationalForms"]["Product"][i]['JointProductionIndicator'])
#            JointProductionIndicator.appendChild(ptext)
#            Product.appendChild(JointProductionIndicator)
#
#            NetCostCode = doc2.createElement("NetCostCode")
#            ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["InternationalForms"]["Product"][i]['NetCostCode'])
#            NetCostCode.appendChild(ptext)
#            Product.appendChild(NetCostCode)
#
#            NetCostDateRange = doc2.createElement("NetCostDateRange")
#            Product.appendChild(NetCostDateRange)
#            BeginDate = doc2.createElement("BeginDate")
#            ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["InternationalForms"]["Product"][i]['NetCostDateRange']['BeginDate'])
#            BeginDate.appendChild(ptext)
#            NetCostDateRange.appendChild(BeginDate)
#
#            EndDate = doc2.createElement("EndDate")
#            ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["InternationalForms"]["Product"][i]['NetCostDateRange']['EndDate'])
#            EndDate.appendChild(ptext)
#            NetCostDateRange.appendChild(EndDate)
#
#
#            ProducerInfo = doc2.createElement("ProducerInfo")
#            ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["InternationalForms"]["Product"][i]['ProducerInfo'])
#            ProducerInfo.appendChild(ptext)
#            Product.appendChild(ProducerInfo)
#
#            MarksAndNumbers = doc2.createElement("MarksAndNumbers")
#            ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["InternationalForms"]["Product"][i]['MarksAndNumbers'])
#            MarksAndNumbers.appendChild(ptext)
#            Product.appendChild(MarksAndNumbers)
#
#            NumberOfPackagesPerCommodity = doc2.createElement("NumberOfPackagesPerCommodity")
#            ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["InternationalForms"]["Product"][i]['NumberOfPackagesPerCommodity'])
#            NumberOfPackagesPerCommodity.appendChild(ptext)
#            Product.appendChild(NumberOfPackagesPerCommodity)
#
#
#            ProductWeight = doc2.createElement("ProductWeight")
#            Product.appendChild(ProductWeight)
#            UnitOfMeasurement = doc2.createElement("UnitOfMeasurement")
#            ProductWeight.appendChild(UnitOfMeasurement)
#            Code = doc2.createElement("Code")
#            ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["InternationalForms"]["Product"][i]['ProductWeight']['UnitOfMeasurement']['Code'])
#            Code.appendChild(ptext)
#            UnitOfMeasurement.appendChild(Code)
#
#            Description = doc2.createElement("Description")
#            ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["InternationalForms"]["Product"][i]['ProductWeight']['UnitOfMeasurement']['Description'])
#            Description.appendChild(ptext)
#            UnitOfMeasurement.appendChild(Description)
#
#            Weight = doc2.createElement("Weight")
#            ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["InternationalForms"]["Product"][i]['ProductWeight']['Weight'])
#            Weight.appendChild(ptext)
#            ProductWeight.appendChild(Weight)
#
#            VehicleID = doc2.createElement("VehicleID")
#            ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["InternationalForms"]["Product"][i]['VehicleID'])
#            VehicleID.appendChild(ptext)
#            Product.appendChild(VehicleID)
#
#            ScheduleB = doc2.createElement("ScheduleB")
#            Product.appendChild(ScheduleB)
#            Number = doc2.createElement("Number")
#            ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["InternationalForms"]["Product"][i]['ScheduleB']['Number'])
#            Number.appendChild(ptext)
#            ScheduleB.appendChild(Number)
#
#            Quantity = doc2.createElement("Quantity")
#            ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["InternationalForms"]["Product"][i]['ScheduleB']['Quantity'])
#            Quantity.appendChild(ptext)
#            ScheduleB.appendChild(Quantity)
#
#            UnitOfMeasurement = doc2.createElement("UnitOfMeasurement")
#            ScheduleB.appendChild(UnitOfMeasurement)
#            Code = doc2.createElement("Code")
#            ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["InternationalForms"]["Product"][i]['ScheduleB']['UnitOfMeasurement']['Code'])
#            Code.appendChild(ptext)
#            UnitOfMeasurement.appendChild(Code)
#
#            Description = doc2.createElement("Description")
#            ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["InternationalForms"]["Product"][i]['ScheduleB']['UnitOfMeasurement']['Description'])
#            Description.appendChild(ptext)
#            UnitOfMeasurement.appendChild(Description)
#
#            ExportType = doc2.createElement("ExportType")
#            ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["InternationalForms"]["Product"][i]['ExportType'])
#            Description.appendChild(ptext)
#            Product.appendChild(ExportType)
#
#            SEDTotalValue = doc2.createElement("SEDTotalValue")
#            ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["InternationalForms"]["Product"][i]['SEDTotalValue'])
#            SEDTotalValue.appendChild(ptext)
#            Product.appendChild(SEDTotalValue)
#
#
#        #/////////end loop in code
#
#
#        InvoiceNumber = doc2.createElement("InvoiceNumber")
#        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["InternationalForms"]["InvoiceNumber"])
#        InvoiceNumber.appendChild(ptext)
#        InternationalForms.appendChild(InvoiceNumber)
#
#        InvoiceDate = doc2.createElement("InvoiceDate")
#        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["InternationalForms"]["InvoiceDate"])
#        InvoiceNumber.appendChild(ptext)
#        InternationalForms.appendChild(InvoiceDate)
#
#        PurchaseOrderNumber = doc2.createElement("PurchaseOrderNumber")
#        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["InternationalForms"]["PurchaseOrderNumber"])
#        InvoiceNumber.appendChild(ptext)
#        InternationalForms.appendChild(PurchaseOrderNumber)
#
#        TermsOfShipment = doc2.createElement("TermsOfShipment")
#        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["InternationalForms"]["TermsOfShipment"])
#        TermsOfShipment.appendChild(ptext)
#        InternationalForms.appendChild(TermsOfShipment)
#
#        ReasonForExport = doc2.createElement("ReasonForExport")
#        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["InternationalForms"]["ReasonForExport"])
#        ReasonForExport.appendChild(ptext)
#        InternationalForms.appendChild(ReasonForExport)
#
#        Comments = doc2.createElement("Comments")
#        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["InternationalForms"]["Comments"])
#        Comments.appendChild(ptext)
#        InternationalForms.appendChild(Comments)
#
#        DeclarationStatement = doc2.createElement("DeclarationStatement")
#        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["InternationalForms"]["DeclarationStatement"])
#        DeclarationStatement.appendChild(ptext)
#        InternationalForms.appendChild(DeclarationStatement)
#
##        Discount = doc2.createElement("Discount")
##        InternationalForms.appendChild(Discount)
##
##        MonetaryValue = doc2.createElement("MonetaryValue")
##        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["InternationalForms"]['Discount']["MonetaryValue"])
##        MonetaryValue.appendChild(ptext)
##        Discount.appendChild(MonetaryValue)
##
##        FreightCharges = doc2.createElement("FreightCharges")
##        InternationalForms.appendChild(FreightCharges)
##        MonetaryValue = doc2.createElement("MonetaryValue")
##        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["InternationalForms"]['FreightCharges']["MonetaryValue"])
##        MonetaryValue.appendChild(ptext)
##        FreightCharges.appendChild(MonetaryValue)
##
##        InsuranceCharges = doc2.createElement("InsuranceCharges")
##        InternationalForms.appendChild(InsuranceCharges)
##        MonetaryValue = doc2.createElement("MonetaryValue")
##        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["InternationalForms"]['InsuranceCharges']["MonetaryValue"])
##        MonetaryValue.appendChild(ptext)
##        InsuranceCharges.appendChild(MonetaryValue)
##
##        OtherCharges = doc2.createElement("OtherCharges")
##        InternationalForms.appendChild(OtherCharges)
##        MonetaryValue = doc2.createElement("MonetaryValue")
##        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["InternationalForms"]['OtherCharges']["MonetaryValue"])
##        MonetaryValue.appendChild(ptext)
##        OtherCharges.appendChild(MonetaryValue)
##
##        Description = doc2.createElement("Description")
##        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["InternationalForms"]['OtherCharges']["Description"])
##        Description.appendChild(ptext)
##        OtherCharges.appendChild(Description)
#
#        CurrencyCode = doc2.createElement("CurrencyCode")
#        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["InternationalForms"]["CurrencyCode"])
#        CurrencyCode.appendChild(ptext)
#        InternationalForms.appendChild(CurrencyCode)
#
#        BlanketPeriod = doc2.createElement("BlanketPeriod")
#        InternationalForms.appendChild(BlanketPeriod)
#        BeginDate = doc2.createElement("BeginDate")
#        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["InternationalForms"]['BlanketPeriod']["BeginDate"])
#        BeginDate.appendChild(ptext)
#        BlanketPeriod.appendChild(BeginDate)
#
#        EndDate = doc2.createElement("EndDate")
#        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["InternationalForms"]['BlanketPeriod']["EndDate"])
#        EndDate.appendChild(ptext)
#        BlanketPeriod.appendChild(EndDate)
#
#        ExportDate = doc2.createElement("ExportDate")
#        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["InternationalForms"]['ExportDate'])
#        ExportDate.appendChild(ptext)
#        InternationalForms.appendChild(ExportDate)
#
#        ExportingCarrier = doc2.createElement("ExportingCarrier")
#        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["InternationalForms"]['ExportingCarrier'])
#        ExportingCarrier.appendChild(ptext)
#        InternationalForms.appendChild(ExportingCarrier)
#
#
#        CarrierID = doc2.createElement("CarrierID")
#        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["InternationalForms"]['CarrierID'])
#        CarrierID.appendChild(ptext)
#        InternationalForms.appendChild(CarrierID)
#
#        InBondCode = doc2.createElement("InBondCode")
#        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["InternationalForms"]['InBondCode'])
#        InBondCode.appendChild(ptext)
#        InternationalForms.appendChild(InBondCode)
#
#        EntryNumber = doc2.createElement("EntryNumber")
#        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["InternationalForms"]['EntryNumber'])
#        EntryNumber.appendChild(ptext)
#        InternationalForms.appendChild(EntryNumber)
#
#        PointOfOrigin = doc2.createElement("PointOfOrigin")
#        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["InternationalForms"]['PointOfOrigin'])
#        PointOfOrigin.appendChild(ptext)
#        InternationalForms.appendChild(PointOfOrigin)
#
#        ModeOfTransport = doc2.createElement("ModeOfTransport")
#        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["InternationalForms"]['ModeOfTransport'])
#        ModeOfTransport.appendChild(ptext)
#        InternationalForms.appendChild(ModeOfTransport)
#
#        PortOfExport = doc2.createElement("PortOfExport")
#        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["InternationalForms"]['PortOfExport'])
#        PortOfExport.appendChild(ptext)
#        InternationalForms.appendChild(PortOfExport)
#
#        PortOfUnloading = doc2.createElement("PortOfUnloading")
#        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["InternationalForms"]['PortOfUnloading'])
#        PortOfUnloading.appendChild(ptext)
#        InternationalForms.appendChild(PortOfUnloading)
#
#        LoadingPier = doc2.createElement("LoadingPier")
#        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["InternationalForms"]['LoadingPier'])
#        LoadingPier.appendChild(ptext)
#        InternationalForms.appendChild(LoadingPier)
#
#        PartiesToTransaction = doc2.createElement("PartiesToTransaction")
#        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["InternationalForms"]['PartiesToTransaction'])
#        PartiesToTransaction.appendChild(ptext)
#        InternationalForms.appendChild(PartiesToTransaction)
#
#        RoutedExportTransactionIndicator = doc2.createElement("RoutedExportTransactionIndicator")
#        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["InternationalForms"]['RoutedExportTransactionIndicator'])
#        RoutedExportTransactionIndicator.appendChild(ptext)
#        InternationalForms.appendChild(RoutedExportTransactionIndicator)
#
#        ContainerizedIndicator = doc2.createElement("ContainerizedIndicator")
#        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["InternationalForms"]['ContainerizedIndicator'])
#        ContainerizedIndicator.appendChild(ptext)
#        InternationalForms.appendChild(ContainerizedIndicator)
#
#        License = doc2.createElement("License")
#        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["InternationalForms"]['License'])
#        License.appendChild(ptext)
#        InternationalForms.appendChild(License)
#
#        Number = doc2.createElement("Number")
#        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["InternationalForms"]['Number'])
#        Number.appendChild(ptext)
#        InternationalForms.appendChild(Number)
#
#        Date = doc2.createElement("Date")
#        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["InternationalForms"]['Date'])
#        Date.appendChild(ptext)
#        InternationalForms.appendChild(Date)
#
#        ExceptionCode = doc2.createElement("ExceptionCode")
#        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["InternationalForms"]['ExceptionCode'])
#        ExceptionCode.appendChild(ptext)
#        InternationalForms.appendChild(ExceptionCode)
#
#        ECCNNumber = doc2.createElement("ECCNNumber")
#        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["InternationalForms"]['ECCNNumber'])
#        ECCNNumber.appendChild(ptext)
#        InternationalForms.appendChild(ECCNNumber)

#        ReturnOfDocumentIndicator = doc2.createElement("ReturnOfDocumentIndicator")
#        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["ReturnOfDocumentIndicator"])
#        ReturnOfDocumentIndicator.appendChild(ptext)
#        ShipmentServiceOptions.appendChild(ReturnOfDocumentIndicator)
#
#        ImportControlIndicator = doc2.createElement("ImportControlIndicator")
#        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["ImportControlIndicator"])
#        ImportControlIndicator.appendChild(ptext)
#        ShipmentServiceOptions.appendChild(ImportControlIndicator)
#
#        LabelMethod = doc2.createElement("LabelMethod")
#        ShipmentServiceOptions.appendChild(LabelMethod)
#        Code = doc2.createElement("Code")
#        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["LabelMethod"]["Code"])
#        Code.appendChild(ptext)
#        LabelMethod.appendChild(Code)
#
#        Description = doc2.createElement("Description")
#        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["LabelMethod"]["Description"])
#        Description.appendChild(ptext)
#        LabelMethod.appendChild(Description)

#        CommercialInvoiceRemovalIndicator = doc2.createElement("CommercialInvoiceRemovalIndicator")
#        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["CommercialInvoiceRemovalIndicator"])
#        CommercialInvoiceRemovalIndicator.appendChild(ptext)
#        ShipmentServiceOptions.appendChild(CommercialInvoiceRemovalIndicator)

#        UPScarbonneutralIndicator = doc2.createElement("UPScarbonneutralIndicator")
#        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["UPScarbonneutralIndicator"])
#        UPScarbonneutralIndicator.appendChild(ptext)
#        ShipmentServiceOptions.appendChild(UPScarbonneutralIndicator)

#        DeliveryConfirmation = doc2.createElement("DeliveryConfirmation")
#        ShipmentServiceOptions.appendChild(DeliveryConfirmation)
#        DCISType = doc2.createElement("DCISType")
#        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["ShipmentServiceOptions"]["DeliveryConfirmation"]["DCISType"])
#        DCISType.appendChild(ptext)
#        DeliveryConfirmation.appendChild(DCISType)

        Package_count = len(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["Package"])
        for i in range(0,Package_count):
            Package = doc2.createElement("Package")
            Shipment.appendChild(Package)

            Description = doc2.createElement("Description")
            ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["Package"][i]["Description"])
            Description.appendChild(ptext)
            Package.appendChild(Description)
            PackagingType = doc2.createElement("PackagingType")
            Package.appendChild(PackagingType)

            Code = doc2.createElement("Code")
            ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["Package"][i]["PackagingType"]['Code'])
            Code.appendChild(ptext)
            PackagingType.appendChild(Code)
            Description = doc2.createElement("Description")
            ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["Package"][i]["PackagingType"]['Description'])
            Description.appendChild(ptext)
            PackagingType.appendChild(Description)

            Dimensions = doc2.createElement("Dimensions")
            Package.appendChild(Dimensions)

            Length = doc2.createElement("Length")
            ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["Package"][i]["Dimensions"]["Length"])
            Length.appendChild(ptext)
            Dimensions.appendChild(Length)
            Width = doc2.createElement("Width")
            ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["Package"][i]["Dimensions"]["Width"])
            Width.appendChild(ptext)
            Dimensions.appendChild(Width)
            Height = doc2.createElement("Height")
            ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["Package"][i]["Dimensions"]["Height"])
            Height.appendChild(ptext)
            Dimensions.appendChild(Height)
            UnitOfMeasurement = doc2.createElement("UnitOfMeasurement")
            Dimensions.appendChild(UnitOfMeasurement)

            Code = doc2.createElement("Code")
            ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["Package"][i]["Dimensions"]['UnitOfMeasurement']["Code"])
            Code.appendChild(ptext)
            UnitOfMeasurement.appendChild(Code)
            Description = doc2.createElement("Description")
            ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["Package"][i]["Dimensions"]['UnitOfMeasurement']["Description"])
            Description.appendChild(ptext)
            UnitOfMeasurement.appendChild(Description)


            PackageWeight = doc2.createElement("PackageWeight")
            Package.appendChild(PackageWeight)

            UnitOfMeasurement = doc2.createElement("UnitOfMeasurement")
            PackageWeight.appendChild(UnitOfMeasurement)
            Code = doc2.createElement("Code")
            ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["Package"][i]['PackageWeight']['UnitOfMeasurement']["Code"])
            Code.appendChild(ptext)
            UnitOfMeasurement.appendChild(Code)

            Description = doc2.createElement("Description")
            ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["Package"][i]['PackageWeight']['UnitOfMeasurement']["Description"])
            Description.appendChild(ptext)
            UnitOfMeasurement.appendChild(Description)

            Weight = doc2.createElement("Weight")
            ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["Package"][i]['PackageWeight']["Weight"])
            Weight.appendChild(ptext)
            PackageWeight.appendChild(Weight)
#            LargePackageIndicator = doc2.createElement("LargePackageIndicator")
##            ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["Package"][i]["LargePackageIndicator"])
##            LargePackageIndicator.appendChild(ptext)
#            Package.appendChild(LargePackageIndicator)
            from_country_code = quick_ship_id.send_addr.id and quick_ship_id.send_addr.country_id.id and quick_ship_id.send_addr.country_id.code
            to_country_code = quick_ship_id.address_id.id and quick_ship_id.address_id.country_id.id and quick_ship_id.address_id.country_id.code
            if not(from_country_code == 'US' and to_country_code == 'US' or from_country_code=='PR' and to_country_code == 'PR'):

                ReferenceNumber = doc2.createElement("ReferenceNumber")
                Package.appendChild(ReferenceNumber)

                BarCodeIndicator = doc2.createElement("BarCodeIndicator")
#            ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["Package"][i]["ReferenceNumber"]["BarCodeIndicator"])
#            BarCodeIndicator.appendChild(ptext)
                ReferenceNumber.appendChild(BarCodeIndicator)

                Code = doc2.createElement("Code")
                ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["Package"][i]["ReferenceNumber"]["Code"])
                Code.appendChild(ptext)
                ReferenceNumber.appendChild(Code)

                Value = doc2.createElement("Value")
                ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["Package"][i]["ReferenceNumber"]["Value"])
                Value.appendChild(ptext)
                ReferenceNumber.appendChild(Value)

#            AdditionalHandling = doc2.createElement("AdditionalHandling")
#            ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["Package"][i]["AdditionalHandling"])
#            AdditionalHandling.appendChild(ptext)
#            Package.appendChild(AdditionalHandling)
            if quick_ship_id.cod:
                PackageServiceOptions = doc2.createElement("PackageServiceOptions")
                Package.appendChild(PackageServiceOptions)

                if quick_ship_id.verbal_confirm:
                    VerbalConfirmation = doc2.createElement("VerbalConfirmation")
                    PackageServiceOptions.appendChild(VerbalConfirmation)

                    ContactInfo = doc2.createElement("ContactInfo")
                    VerbalConfirmation.appendChild(ContactInfo)

                    Name = doc2.createElement("Name")
                    ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["Package"][i]["PackageServiceOptions"]['VerbalConfirmation']['ContactInfo']['Name'])
                    Name.appendChild(ptext)
                    ContactInfo.appendChild(Name)

                    PhoneNumber = doc2.createElement("PhoneNumber")
                    ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["Package"][i]["PackageServiceOptions"]['VerbalConfirmation']['ContactInfo']['PhoneNumber'])
                    PhoneNumber.appendChild(ptext)
                    ContactInfo.appendChild(PhoneNumber)

                if quick_ship_id.shipper_release:
                    ShipperReleaseIndicator = doc2.createElement("ShipperReleaseIndicator")
                    ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["Package"][i]["PackageServiceOptions"]['ShipperReleaseIndicator'])
                    ShipperReleaseIndicator.appendChild(ptext)
                    PackageServiceOptions.appendChild(ShipperReleaseIndicator)


#            DeliveryConfirmation = doc2.createElement("DeliveryConfirmation")
#            PackageServiceOptions.appendChild(DeliveryConfirmation)
#
#            DCISType = doc2.createElement("DCISType")
#            ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["Package"][i]["PackageServiceOptions"]['DeliveryConfirmation']['DCISType'])
#            DCISType.appendChild(ptext)
#            DeliveryConfirmation.appendChild(DCISType)
#
#            DCISNumber = doc2.createElement("DCISNumber")
#            ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["Package"][i]["PackageServiceOptions"]['DeliveryConfirmation']['DCISNumber'])
#            DCISNumber.appendChild(ptext)
#            DeliveryConfirmation.appendChild(DCISNumber)
#
#            InsuredValue = doc2.createElement("InsuredValue")
#            PackageServiceOptions.appendChild(InsuredValue)
#
#
#            Type = doc2.createElement("Type")
#            ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["Package"][i]["PackageServiceOptions"]['InsuredValue']['Type'])
#            Type.appendChild(ptext)
#            InsuredValue.appendChild(Type)
#
#            Code = doc2.createElement("Code")
#            ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["Package"][i]["PackageServiceOptions"]['InsuredValue']['Code'])
#            Code.appendChild(ptext)
#            InsuredValue.appendChild(Code)
#
#
#            Description = doc2.createElement("Description")
#            ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["Package"][i]["PackageServiceOptions"]['InsuredValue']['Description'])
#            Description.appendChild(ptext)
#            InsuredValue.appendChild(Description)
#
#            Description = doc2.createElement("CurrencyCode")
#            ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["Package"][i]["PackageServiceOptions"]['InsuredValue']['CurrencyCode'])
#            CurrencyCode.appendChild(ptext)
#            InsuredValue.appendChild(CurrencyCode)
#
#            Description = doc2.createElement("MonetaryValue")
#            ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["Package"][i]["PackageServiceOptions"]['InsuredValue']['MonetaryValue'])
#            MonetaryValue.appendChild(ptext)
#            InsuredValue.appendChild(MonetaryValue)
#
#
#            COD = doc2.createElement("COD")
#            PackageServiceOptions.appendChild(COD)
#
#
#            CODCode = doc2.createElement("CODCode")
#            ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["Package"][i]["PackageServiceOptions"]['COD']['CODCode'])
#            CODCode.appendChild(ptext)
#            COD.appendChild(CODCode)
#
#            CODFundsCode = doc2.createElement("CODFundsCode")
#            ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["Package"][i]["PackageServiceOptions"]['COD']['CODFundsCode'])
#            CODFundsCode.appendChild(ptext)
#            COD.appendChild(CODFundsCode)
#
#            CODAmount = doc2.createElement("CODAmount")
#            ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["Package"][i]["PackageServiceOptions"]['COD']['CODAmount'])
#            CODAmount.appendChild(ptext)
#            COD.appendChild(CODAmount)
#
#            CurrencyCode = doc2.createElement("CurrencyCode")
#            ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["Package"][i]["PackageServiceOptions"]['InsuredValue']['COD']['CODAmount']['CurrencyCode'])
#            CurrencyCode.appendChild(ptext)
#            InsuredValue.appendChild(CurrencyCode)
#
#            MonetaryValue = doc2.createElement("MonetaryValue")
#            ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["Package"][i]["PackageServiceOptions"]['InsuredValue']['COD']['CODAmount']['MonetaryValue'])
#            MonetaryValue.appendChild(ptext)
#            InsuredValue.appendChild(MonetaryValue)
#
#            VerbalConfirmation = doc2.createElement("VerbalConfirmation")
#            PackageServiceOptions.appendChild(VerbalConfirmation)
#
#            ContactInfo = doc2.createElement("ContactInfo")
#            VerbalConfirmation.appendChild(ContactInfo)
#
#            Name = doc2.createElement("Name")
#            ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["Package"][i]["PackageServiceOptions"]['VerbalConfirmation']['VerbalConfirmation']['Name'])
#            Name.appendChild(ptext)
#            ContactInfo.appendChild(Name)
#
#            PhoneNumber = doc2.createElement("PhoneNumber")
#            ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["Package"][i]["PackageServiceOptions"]['VerbalConfirmation']['VerbalConfirmation']['PhoneNumber'])
#            PhoneNumber.appendChild(ptext)
#            ContactInfo.appendChild(PhoneNumber)
#
#            ShipperReleaseIndicator = doc2.createElement("ShipperReleaseIndicator")
#            ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["Package"][i]["PackageServiceOptions"]['ShipperReleaseIndicator'])
#            ShipperReleaseIndicator.appendChild(ptext)
#            PackageServiceOptions.appendChild(ShipperReleaseIndicator)
#
#
#            Notification = doc2.createElement("Notification")
#            PackageServiceOptions.appendChild(Notification)
#            NotificationCode = doc2.createElement("NotificationCode")
#            ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["Package"][i]["PackageServiceOptions"]['Notification'])
#            NotificationCode.appendChild(ptext)
#            Notification.appendChild(NotificationCode)
#
#            EMailMessage = doc2.createElement("EMailMessage")
#            Notification.appendChild(EMailMessage)
#            UndeliverableEMailAddress = doc2.createElement("UndeliverableEMailAddress")
#            ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["Package"][i]["PackageServiceOptions"]["Notification"]["EMailMessage"]["UndeliverableEMailAddress"])
#            UndeliverableEMailAddress.appendChild(ptext)
#            EMailMessage.appendChild(UndeliverableEMailAddress)
#
#            EMailAddress = doc2.createElement("EMailAddress")
#            ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["Package"][i]["PackageServiceOptions"]["Notification"]["EMailMessage"]["EMailAddress"])
#            EMailAddress.appendChild(ptext)
#            EMailMessage.appendChild(EMailAddress)
#
#            FromEMailAddress = doc2.createElement("FromEMailAddress")
#            ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["Package"][i]["PackageServiceOptions"]["Notification"]["EMailMessage"]["FromEMailAddress"])
#            FromEMailAddress.appendChild(ptext)
#            EMailMessage.appendChild(FromEMailAddress)
#
#            FromName = doc2.createElement("FromName")
#            ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["Package"][i]["PackageServiceOptions"]["Notification"]["EMailMessage"]["FromName"])
#            FromName.appendChild(ptext)
#            EMailMessage.appendChild(FromName)
#
#            Memo = doc2.createElement("Memo")
#            ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["Package"][i]["PackageServiceOptions"]["Notification"]["EMailMessage"]["Memo"])
#            Memo.appendChild(ptext)
#            EMailMessage.appendChild(Memo)
#
#            Subject = doc2.createElement("Subject")
#            ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["Package"][i]["PackageServiceOptions"]["Notification"]["EMailMessage"]["Subject"])
#            Subject.appendChild(ptext)
#            EMailMessage.appendChild(Subject)
#
#            SubjectCode = doc2.createElement("SubjectCode")
#            ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["Package"][i]["PackageServiceOptions"]["Notification"]["EMailMessage"]["SubjectCode"])
#            SubjectCode.appendChild(ptext)
#            EMailMessage.appendChild(SubjectCode)
#
#            ReturnsFlexibleAccessIndicator = doc2.createElement("ReturnsFlexibleAccessIndicator")
#            ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["Shipment"]["Package"][i]["PackageServiceOptions"]['ReturnsFlexibleAccessIndicator'])
#            ReturnsFlexibleAccessIndicator.appendChild(ptext)
#            PackageServiceOptions.appendChild(ReturnsFlexibleAccessIndicator)






            #/////////end loop in code



        LabelSpecification = doc2.createElement("LabelSpecification")
        ShipmentConfirmRequest.appendChild(LabelSpecification)

        LabelPrintMethod = doc2.createElement("LabelPrintMethod")
        LabelSpecification.appendChild(LabelPrintMethod)
        Code = doc2.createElement("Code")
        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["LabelSpecification"]["LabelPrintMethod"]["Code"])
        Code.appendChild(ptext)
        LabelPrintMethod.appendChild(Code)

        Description = doc2.createElement("Description")
        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["LabelSpecification"]["LabelPrintMethod"]["Description"])
        Description.appendChild(ptext)
        LabelPrintMethod.appendChild(Description)

#        HTTPUserAgent = doc2.createElement("HTTPUserAgent")
#        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["LabelSpecification"]["LabelPrintMethod"]["HTTPUserAgent"])
#        HTTPUserAgent.appendChild(ptext)
#        LabelSpecification.appendChild(HTTPUserAgent)

        LabelStockSize = doc2.createElement("LabelStockSize")
        LabelSpecification.appendChild(LabelStockSize)
        Height = doc2.createElement("Height")
        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["LabelSpecification"]["LabelStockSize"]["Height"])
        Height.appendChild(ptext)
        LabelStockSize.appendChild(Height)
        Width = doc2.createElement("Width")
        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["LabelSpecification"]["LabelStockSize"]["Width"])
        Width.appendChild(ptext)
        LabelStockSize.appendChild(Width)

        LabelImageFormat = doc2.createElement("LabelImageFormat")
        LabelSpecification.appendChild(LabelImageFormat)

        Code = doc2.createElement("Code")
        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["LabelSpecification"]["LabelImageFormat"]["Code"])
        Code.appendChild(ptext)
        LabelImageFormat.appendChild(Code)

        Description = doc2.createElement("Description")
        ptext = doc1.createTextNode(data_for_confirm_request["ShipmentConfirmRequest"]["LabelSpecification"]["LabelImageFormat"]["Description"])
        Description.appendChild(ptext)
        LabelImageFormat.appendChild(Description)
        #print type(doc2.toprettyxml())
        ret = []
#        try:


        
        
        serv=''
        acce_port=''
        serv_path=''
        if quick_ship_id.logis_company.id:
            if quick_ship_id.logis_company.test_mode:
                acce_web = quick_ship_id.logis_company.ship_req_test_web or ''
                acce_port = quick_ship_id.logis_company.ship_req_test_port
            else:
                acce_web = quick_ship_id.logis_company.ship_req_web or ''
                acce_port = quick_ship_id.logis_company.ship_req_port
            if acce_web:
                parse_url = urlparse(acce_web)
                serv = parse_url.netloc
                serv_path = parse_url.path
            else:
                raise osv.except_osv(_('Unable to find Shipping URL!'),_("Please configure the shipping company with websites.") )
        else:
            raise osv.except_osv(_('No Company Selected!'),_("Please select a logistic company.") )
        
        
        request_xml=doc1.toprettyxml()+doc2.toprettyxml()
        conn=httplib.HTTPSConnection(serv,acce_port)
        res=conn.request("POST",serv_path,request_xml)
        res=conn.getresponse()
        result = res.read()
        response_dic=xml2dic.main(result)


        response = ''
        err_flag = False
        for res_elm in  response_dic['ShipmentConfirmResponse'][0]['Response']:
            if res_elm.get('ResponseStatusCode'):
                response = '\nResponse Status Code: '+str(res_elm['ResponseStatusCode'])
            if res_elm.get('ResponseStatusDescription'):
                response += '\nResponse Description: '+str(res_elm['ResponseStatusDescription'])
            if res_elm.get('Error'):
                err_flag = True
                for err_elm in res_elm.get('Error'):
                    if err_elm.get('ErrorCode',False):
                        response += '\nErrorCode: '+str(err_elm['ErrorCode'])
                    if err_elm.get('ErrorDescription',False):
                        response += '\nErrorDescription: '+str(err_elm['ErrorDescription'])
                self.write(cr,uid,ids,{'response':response})
        ship_digest=""
        if len(response_dic['ShipmentConfirmResponse'])>5 and response_dic['ShipmentConfirmResponse'][5].has_key('ShipmentDigest') :
            ship_digest +=str(response_dic['ShipmentConfirmResponse'][5]['ShipmentDigest'])
        #print result
        ship_move_ids=[]
        value_dic={
                   'package_weight':quick_ship_id.weight,
                   'state':'in_process',
                   'partner_id':quick_ship_id.recipient.id,
                   'service':quick_ship_id.service,
                   'package_weight':quick_ship_id.weight,
                   'ship_to':quick_ship_id.recipient.id,
                   'ship_from':quick_ship_id.sender.id,
                   'sale_id':quick_ship_id.saleorder_id.id and quick_ship_id.saleorder_id.id
                   }
        ship_move_id = self.pool.get('shipping.move').create(cr, uid, value_dic)
        if err_flag:
            ret = False
        else:
            ret={'response':response,'ship_digest':ship_digest,'ship_move_id':ship_move_id}
#            abc=self.process_ship_accept(cr, uid, ids,ship_move_ids, context=context)

#        except Exception, e:
#            print e

        return ret
#        access_string=self.create_acc_string()


#    def default_get(self, cr, uid, fields, context):
#        """ To get default values for the object.
#         @param self: The object pointer.
#         @param cr: A database cursor
#         @param uid: ID of the user currently logged in
#         @param fields: List of fields for which we want default values
#         @param context: A standard dictionary
#         @return: A dictionary which of fields with values.
#        """
#        product_id = context and context.get('active_id', False) or False
#        res = super(stock_change_product_qty, self).default_get(cr, uid, fields, context=context)
#
#        if 'new_quantity' in fields:
#            res.update({'new_quantity': 1})
#        if 'product_id' in fields:
#            res.update({'product_id': product_id})
#        return res

    def process_ship_accept(self,cr, uid, ids, ret_res, context=None):
        quick_ship_id = self.browse(cr, uid, ids[0], context=context)

        ship_digest=ret_res.get('ship_digest')

        data_for_Access_Request={
                            'AccessLicenseNumber':quick_ship_id.bill_to.id and quick_ship_id.bill_to.accesslicensenumber or '',
                            'UserId':quick_ship_id.bill_to.id and quick_ship_id.bill_to.userid or '',
                            'Password':quick_ship_id.bill_to.id and quick_ship_id.bill_to.password or ''
                            }

        data_for_Ship_Accept = {'ShipmentAcceptRequest':{
                                    'Request':{
                                      'RequestAction':'ShipAccept',
                                      'TransactionReference':{
                                        'CustomerContext':''
                                        }
                                    },
                                        'ShipmentDigest':ship_digest
                    }
                }


        doc1 = Document()
        AccessRequest = doc1.createElement("AccessRequest")
        AccessRequest.setAttribute("xml:lang", "en-US")
        doc1.appendChild(AccessRequest)


        AccessLicenseNumber = doc1.createElement("AccessLicenseNumber")
        ptext = doc1.createTextNode(data_for_Access_Request["AccessLicenseNumber"])
        AccessLicenseNumber.appendChild(ptext)
        AccessRequest.appendChild(AccessLicenseNumber)

        UserId = doc1.createElement("UserId")
        ptext = doc1.createTextNode(data_for_Access_Request["UserId"])
        UserId.appendChild(ptext)
        AccessRequest.appendChild(UserId)

        Password = doc1.createElement("Password")
        ptext = doc1.createTextNode(data_for_Access_Request["Password"])
        Password.appendChild(ptext)
        AccessRequest.appendChild(Password)

        Request_string1=doc1.toprettyxml()


        #///////////////////


        doc2 = Document()
        ShipmentAcceptRequest = doc2.createElement("ShipmentAcceptRequest")
        ShipmentAcceptRequest.setAttribute("xml:lang", "en-US")
        doc2.appendChild(ShipmentAcceptRequest)



        Request = doc2.createElement("Request")
        ShipmentAcceptRequest.appendChild(Request)


        RequestAction = doc2.createElement("RequestAction")
        ptext = doc1.createTextNode(data_for_Ship_Accept['ShipmentAcceptRequest']["Request"]['RequestAction'])
        RequestAction.appendChild(ptext)
        Request.appendChild(RequestAction)


        TransactionReference = doc2.createElement("TransactionReference")
        Request.appendChild(TransactionReference)

        CustomerContext = doc2.createElement("CustomerContext")
        ptext = doc1.createTextNode(data_for_Ship_Accept['ShipmentAcceptRequest']["Request"]['TransactionReference']['CustomerContext'])
        CustomerContext.appendChild(ptext)
        TransactionReference.appendChild(CustomerContext)

        ShipmentDigest = doc2.createElement("ShipmentDigest")
        ptext = doc1.createTextNode(data_for_Ship_Accept['ShipmentAcceptRequest']["ShipmentDigest"])
        ShipmentDigest.appendChild(ptext)
        ShipmentAcceptRequest.appendChild(ShipmentDigest)

        Request_string2=doc2.toprettyxml()
        
        
        serv=''
        acce_port=''
        serv_path=''
        if quick_ship_id.logis_company.id:
            if quick_ship_id.logis_company.test_mode:
                acce_web = quick_ship_id.logis_company.ship_accpt_test_web or ''
                acce_port = quick_ship_id.logis_company.ship_accpt_test_port
            else:
                acce_web = quick_ship_id.logis_company.ship_accpt_web or ''
                acce_port = quick_ship_id.logis_company.ship_accpt_port
            if acce_web:
                parse_url = urlparse(acce_web)
                serv = parse_url.netloc
                serv_path = parse_url.path
            else:
                raise osv.except_osv(_('Unable to find Shipping URL!'),_("Please configure the shipping company with websites.") )
        else:
            raise osv.except_osv(_('No Company Selected!'),_("Please select a logistic company.") )


        
        Request_string=Request_string1+Request_string2
        conn=httplib.HTTPSConnection(serv,acce_port)
        res=conn.request("POST",serv_path,Request_string)
        res=conn.getresponse()
        result = res.read()
        response_dic=xml2dic.main(result)
        NegotiatedRates=''
        ShipmentIdentificationNumber=''
        TrackingNumber=''
        if len(response_dic['ShipmentAcceptResponse'])>1 and response_dic['ShipmentAcceptResponse'][1].has_key('ShipmentResults'):
            if len(response_dic['ShipmentAcceptResponse'][1]['ShipmentResults'])>1 and response_dic['ShipmentAcceptResponse'][1]['ShipmentResults'][1].has_key('NegotiatedRates'):
                if  len(response_dic['ShipmentAcceptResponse'][1]['ShipmentResults'][1]['NegotiatedRates'])>0 and response_dic['ShipmentAcceptResponse'][1]['ShipmentResults'][1]['NegotiatedRates'][0].has_key('NetSummaryCharges'):
                    if len(response_dic['ShipmentAcceptResponse'][1]['ShipmentResults'][1]['NegotiatedRates'][0]['NetSummaryCharges'])>0 and response_dic['ShipmentAcceptResponse'][1]['ShipmentResults'][1]['NegotiatedRates'][0]['NetSummaryCharges'][0].has_key('GrandTotal'):
                       NegotiatedRates=response_dic['ShipmentAcceptResponse'][1]['ShipmentResults'][1]['NegotiatedRates'][0]['NetSummaryCharges'][0]['GrandTotal'][1]['MonetaryValue']

            if len(response_dic['ShipmentAcceptResponse'][1]['ShipmentResults'])>0 and response_dic['ShipmentAcceptResponse'][1]['ShipmentResults'][3].has_key('ShipmentIdentificationNumber'):
                ShipmentIdentificationNumber=response_dic['ShipmentAcceptResponse'][1]['ShipmentResults'][3]['ShipmentIdentificationNumber']

            if len(response_dic['ShipmentAcceptResponse'][1]['ShipmentResults'])>4 and response_dic['ShipmentAcceptResponse'][1]['ShipmentResults'][4].has_key('PackageResults') :
                if len(response_dic['ShipmentAcceptResponse'][1]['ShipmentResults'][4]['PackageResults'])>0:
                    TrackingNumber=response_dic['ShipmentAcceptResponse'][1]['ShipmentResults'][4]['PackageResults'][0]['TrackingNumber']

                if len(response_dic['ShipmentAcceptResponse'][1]['ShipmentResults'][4]['PackageResults'])>2 and response_dic['ShipmentAcceptResponse'][1]['ShipmentResults'][4]['PackageResults'][2].has_key('LabelImage') :
                    label_image =response_dic['ShipmentAcceptResponse'][1]['ShipmentResults'][4]['PackageResults'][2]['LabelImage'][1]['GraphicImage']

            if label_image:
                im_in_raw=base64.decodestring(label_image)
                path=tempfile.mktemp()
                temp=file(path,'wb')
                temp.write(im_in_raw)
                temp.close()
                new_im=Image.open(path)
                new_im=new_im.rotate(270)
                new_im.save(path,'JPEG')
                label_from_file=open(path,'rb')
                label_image=base64.encodestring(label_from_file.read())
                label_from_file.close()

            self.write(cr,uid,ids,{'rate':NegotiatedRates or '0','logo':label_image})
            if ret_res.get('ship_move_id'):
                self.write(cr,uid,ids,{'rate':NegotiatedRates or '0','ship_log_id':ret_res.get('ship_move_id')})
                tracking_url = ''
                if TrackingNumber:
                    tracking_url = quick_ship_id.logis_company.ship_tracking_url
                    if tracking_url:
                        try:
                            tracking_url = tracking_url%TrackingNumber
                        except Exception, e:
                            tracking_url = "Invalid tracking url on shipping company"
                ship_move_id = self.pool.get('shipping.move').write(cr, uid, [ret_res.get('ship_move_id')], {'logo':label_image,'shipment_identific_no':ShipmentIdentificationNumber,'tracking_no':TrackingNumber,'tracking_url':tracking_url})

#            value_dic={'pick_id':deliv_order.id ,
#                       'package_weight':deliv_order.tot_ship_weight,
#                       'state':deliv_order.ship_state,
#                       'tracking_no':deliv_order.tracking_no,
#                       'shipment_identific_no':deliv_order.shipment_identific_no,
#                       'ship_cost':deliv_order.negotiated_rates,
#                       'sat_delivery':deliv_order.sat_delivery
#                       }
#            shiping_move_ids=self.pool.get('shipping.move').search(cr,uid,[('pick_id','=',deliv_order.id)])
#            self.pool.get('shipping.move').write(cr, uid, ship_move_ids, value_dic)




        return

    def get_rate(self,cr, uid, ids, context=None):
        res = self.process_ship(cr,uid,ids,context=context)
        if res:
            res = self.process_ship_accept(cr,uid,ids,res,context=context)
            self.write(cr,uid,ids,{'response':''})
        return True
    def print_label(self,cr, uid, ids, context=None):
        data = self.browse(cr,uid,ids[0],context=context)
        res = [data.ship_log_id.id]
        return {
            'domain': "[('id', 'in', %s)]" % res,
            'name': _('Shipping Logs'),
            'view_type': 'form',
            'view_mode': 'tree,form',
            'res_model': 'shipping.move',
            'view_id': False,
#            'views': [(tree_res, 'tree'), (form_res, 'form')],
            'type': 'ir.actions.act_window',
        }
#        return {
#            'type': 'ir.actions.report.xml',
#            'report_name':'quick.ship.label.print',
#            'datas': {
#                    'model':'quick.ship',
#                   # 'id': ids and ids[0] or False,
#                    'ids': ids and ids or [],
#                    'report_type': 'pdf',
#                    'form': {}
#                },
#   #         'nodestroy': True
#            }


        return True

quick_ship()

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
