# -*- coding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2011 NovaPoint Group LLC (<http://www.novapointgroup.com>)
#    Copyright (C) 2004-2010 OpenERP SA (<http://www.openerp.com>)
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>
#
##############################################################################
from osv import fields, osv
import pooler


class stock_move(osv.osv):
    _name = "stock.move"
    _inherit="stock.move"

    def _get_order_no(self,cr,uid,product):
        eval=-1
        for multi_loc in product.multi_location_ids:
            if multi_loc.evaluation_order > eval:
                eval=multi_loc.evaluation_order
        return eval+1


    def _get_storage_type(self,cr,uid,product):
            for multi_loc in product.multi_location_ids:
                if multi_loc.storage_type == 'primary':
                    return 'secondary'
            return 'primary'

    def _create_multilocation(self, cr, uid,ids, vals, context={}):
        if vals.get('move_dest_id',False) or  (vals.get('state', False) and vals['state'] <> "cancel" ):
            warehouse_ids = self.pool.get('stock.warehouse').search(cr, uid, [],context=context)
            lot_stock_ids = []
            for lot_stock_id in self.pool.get('stock.warehouse').read(cr,uid,warehouse_ids,['lot_stock_id']):
                lot_stock_ids.append(lot_stock_id['lot_stock_id'][0])
            for move in self.pool.get("stock.move").browse(cr, uid, ids, context=context):
                if move.location_dest_id.usage != 'internal' or move.location_dest_id.id in lot_stock_ids:
                    continue
                if not self.pool.get('stock.multi.location').search(cr, uid, [('product_id','=',move.product_id.id), ('location_name','=',move.location_dest_id.id)], context=context):
                    evaluation_order = 1
                    storage_type = "primary"
                    for multi_location in move.product_id.multi_location_ids:
                        if multi_location.evaluation_order >= evaluation_order:
                            evaluation_order = multi_location.evaluation_order+1
                        if multi_location.storage_type =='primary':
                            storage_type = "secondary"
                    self.pool.get('stock.multi.location').create(cr, uid, {
                                                          'evaluation_order': evaluation_order,
                                                          'storage_type': storage_type,
                                                          'warehouse': self.pool.get('stock.multi.location')._get_shop_id(cr, uid, [], context=context),
                                                          'location_name': move.location_dest_id.id,
                                                          'product_uom': move.product_uom.id,
                                                          'min_level': 0,
                                                          'target_level': 0,
                                                          'qty_multiple': 1,
                                                          'location_active': True,
                                                          'product_id':move.product_id.id,
                                                                           }, context=context)
        return True


    def write(self, cr, uid, ids, vals, context=None):
        res=super(stock_move, self).write(cr, uid, ids, vals, context=context)
        if res:
            self._create_multilocation(cr, uid, ids, vals, context)
        return res

    def create(self, cr, uid, vals, context=None):
        res=super(stock_move, self).create(cr, uid, vals, context)
        if res:
            self._create_multilocation(cr, uid, [res], vals, context)
        return res

stock_move()



class stock_picking(osv.osv):
    _name = "stock.picking"
    _inherit="stock.picking"
    _columns = {
                'explode_multi_locations':fields.boolean('Explode Multi Location')
                }

    _defaults = {
                 'explode_multi_locations': True,
        }

    def explode_multilocation(self, cr, uid, ids):
        for pick in self.pool.get("stock.picking").browse(cr, uid, ids):
            if not pick.explode_multi_locations and pick.type != "out":
                continue
            for move in pick.move_lines:
                total_product = remaining_product = move.product_qty
                if total_product < 0:
                    continue
                for multi_location in move.product_id.multi_location_ids:
                    if remaining_product >0 and \
                        multi_location.location_name.location_id and \
                        (multi_location.location_name.location_id.id == move.location_id.id) and \
                        multi_location.qty_available > 0:
                        default_vals = {'location_id':multi_location.location_name.id,
                                        'product_qty' : min(remaining_product, multi_location.qty_available),
                                        'location_dest_id':move.location_dest_id.id,
                                        'date_expected':move.date_expected,
                                        'state':move.state
                                        }
                        remaining_product = remaining_product - min(remaining_product, multi_location.qty_available)
                        self.pool.get("stock.move").copy(cr, uid, move.id, default_vals)
                if remaining_product > 0:
                    cr.execute("update stock_move set product_qty = %s where id = %s" %(str(remaining_product),str(move.id)))
                else:
                    cr.execute("delete from stock_move where id = %s" %str(move.id))
        return True

    def action_assign(self, cr, uid, ids, *args):
        self.explode_multilocation(cr, uid, ids)
        ret=super(stock_picking, self).action_assign(cr, uid, ids, *args)
        return ret

    def draft_force_assign(self, cr, uid, ids, *args):
        self.explode_multilocation(cr, uid, ids)
        ret=super(stock_picking, self).draft_force_assign(cr, uid, ids, *args)
        return ret

    def draft_validate(self, cr, uid, ids, *args):
        self.explode_multilocation(cr, uid, ids)
        ret=super(stock_picking, self).draft_validate(cr, uid, ids, *args)
        return ret


stock_picking()
# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4: