# -*- coding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2011 NovaPoint Group LLC (<http://www.novapointgroup.com>)
#    Copyright (C) 2004-2010 OpenERP SA (<http://www.openerp.com>)
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>
#
##############################################################################
from osv import fields, osv
import pooler

location_type=['Supplier Location','View','Internal Location','Customer Location', 'Inventory','Procurement','Production','Transit Location for Inter-Companies Transfers']


location_type={'supplier':'Supplier Location',
               'view':'View',
               'internal':'Internal Location',
               'customer':'Customer Location',
               'inventory': 'Inventory',
               'procurement': 'Procurement',
               'production':'Production',
               'transit':'Transit Location for Inter-Companies Transfers'}


class stock_multi_location(osv.osv):

    def _calc_imm_use_stock(self, cr, uid, ids, name, args, context):
        res = {}
        for multi_location in self.pool.get('stock.multi.location').browse(cr, uid, ids,context=context):
            context['location'] = multi_location.location_name.id
            field_names=['qty_available', 'virtual_available', 'outgoing_qty']
            stock_data = multi_location.product_id._product_available(field_names,context=context)
            if stock_data.get(multi_location.product_id.id, False):
                res.update({multi_location.id : stock_data[multi_location.product_id.id]['qty_available'] - abs(stock_data[multi_location.product_id.id]['outgoing_qty']) })
        return res
  
    def _calc_bom_stock_value(self, cr, uid, ids, name, args, context):
        return 0.0

    def _calc_stock(self, cr, uid, ids, name, args, context=False):
        res = {}
        for multi_location in self.pool.get('stock.multi.location').browse(cr, uid, ids,context=context):
            context['location'] = multi_location.location_name.id
            stock_data = multi_location.product_id._product_available(field_names=['qty_available', 'virtual_available'],context=context)
            if stock_data.get(multi_location.product_id.id, False):
                res[multi_location.id] = stock_data[multi_location.product_id.id]
        return res

    def onchange_packge_no(self, cr, uid, ids, line_ids,evaluation_order,context={}):
        ret = {}
        if evaluation_order:
            ret['evaluation_order'] = evaluation_order
        else:
            for line in line_ids:
                if line and line[2] and line[2]['evaluation_order'] and evaluation_order < line[2]['evaluation_order']:
                     evaluation_order=line[2]['evaluation_order']
            evaluation_order+=1
            ret['evaluation_order'] = evaluation_order
        return {'value':ret}



    def _find_multi_loc(self, cr, uid, ids, context=None):
        result_locations = {}
        result_products = {}
        for move in self.pool.get('stock.move').browse(cr, uid, ids, context=context):
            result_locations[move.location_id.id] = True
            result_locations[move.location_dest_id.id] = True
            result_products[move.product_id.id] = True
        locations = result_locations.keys()
        products = result_products.keys()
        return self.pool.get('stock.multi.location').search(cr, uid,[('location_name', 'in' ,locations),('product_id', 'in', products) ],context=context)

    def get_storage_type(self, cr, uid, context=None):
        for line in context.get('muli_location',[]):
            if line and line[2] and line[2]['storage_type'] == 'primary':
                return 'secondary'
        return 'primary'

    def _get_shop_id(self, cr, uid, ids, context=None):
        cmpny_id = self.pool.get('res.users')._get_company(cr, uid, context=context)
        shop = self.pool.get('sale.shop').search(cr, uid, [('company_id', '=', cmpny_id)])
        if shop:
            shop_obj = self.pool.get('sale.shop').browse(cr, uid, shop[0], context=context)
            return shop_obj.warehouse_id and shop_obj.warehouse_id.id
        return shop and shop[0] or False

    _name ="stock.multi.location"
    _rec_name='name'
    _order = 'evaluation_order'
    _columns = {
                  'evaluation_order': fields.integer('Eval Order', size=10,help='The order in which  the individual  locations are  evaluated for  picking purposes. ', select=1),
                  'storage_type': fields.selection([('primary','Primary'),('secondary','Secondary'),('overstock','Overstock')], 'Storage Type',help='The type of Storage  this location is.  Options are:  Primary, Secondary,  Overstock ', select=1),
                  'warehouse': fields.many2one('stock.warehouse', 'Warehouse', help='This is the name of  the warehouse. '),
                  'location_name': fields.many2one('stock.location', 'Location Name', required=True, help='This is the location  where the product is  located.', select=1),
                  #'location_type': fields.char('Location type', size=128, readonly=True,help='This is the type  associated with the  Location Name.'),
                  'location_type': fields.related('location_name', 'usage', type='char', size=16, string='Location type', store=True, help="This is the type  associated with the  Location Name.", readonly=True),
                  'qty_available': fields.function(_calc_stock, multi="stock", method=True,
                                                    store= {'stock.multi.location': (lambda self, cr, uid, ids, c={}: ids, ['product_id','location_name'], 10),
                                                            'stock.move': (_find_multi_loc, ['product_id','location_id','location_dest_id','product_qty','state'], 10),
                                                            },
                                                             type='float',string='Real Stock',help='This is the real stock  for each specific  location.'),

                  'virtual_available': fields.function(_calc_stock, multi="stock", method=True,
                                                       store= {'stock.multi.location': (lambda self, cr, uid, ids, c={}: ids, ['product_id','location_name'], 10),
                                                             'stock.move': (_find_multi_loc, ['product_id','location_id','location_dest_id','product_qty','state'], 10),
                                                            },
                                                             type='float', string='Virtual Stock',help='This is the virtual  Stock for each  specific location.'),

                  'immediately_usable_stock': fields.function(_calc_imm_use_stock, method=True, type='float',string='Immediately Usable Stock',help='This is the  immediatley usable  stock by location.'),
                  'bom_stock_value': fields.function(_calc_bom_stock_value, method=True,type='float', string='BoM Stock Value',help='This is the  calculated BoM  Stock available by  location '),
                  'product_uom': fields.many2one('product.uom', 'Product UoM', help='This is the Products UoM taken from the Product record.'),
                  'min_level': fields.integer('Min Level', size=64,help='This shows the minimum level of inventory held in this location.' ),
                  'target_level': fields.integer('Target Level', size=64 ,help='This is the target Maximum level of Inventory held in this location.'),
                  'qty_multiple': fields.integer('Qty Multiple', size=64 ,help='This is the minimum qty to replenish the inventory with.'),
                  'location_active': fields.boolean('Active',help='This indicates whether the replenish rules are active for this product’s location.'),
                  'product_id':fields.many2one('product.product','Product', required=True, select=1),
    }

    _defaults = {
#                 'evaluation_order':lambda obj, cr, uid, context: obj.pool.get('ir.sequence').get(cr, uid, 'stock.multi.location'),
                 'location_active':lambda * a : True,
                 'storage_type': get_storage_type,
                 'warehouse': _get_shop_id

                }

stock_multi_location()




# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4: